/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.extension.internal.buildnote;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;

import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.client.IAuditableClient;
import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.client.IWorkItemListener;
import com.ibm.team.workitem.client.WorkItemChangeEvent;
import com.ibm.team.workitem.client.WorkItemOperation;
import com.ibm.team.workitem.client.WorkItemWorkingCopy;
import com.ibm.team.workitem.common.model.AttributeTypes;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.ide.ui.internal.editor.CustomAttributesTab;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditor;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditorInput;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemUIWorkingCopy;
import com.ibm.team.workitem.rcp.ui.IWorkItemUIWorkingCopy;
import com.ibm.team.workitem.rcp.ui.UIWorkItemListener;


public class AddBuildNoteAction extends Action implements IEditorActionDelegate {
	
	private class WorkItemListener implements IWorkItemListener {

		public void workItemAttributeChanged(WorkItemChangeEvent event) {
			if (fWorkingCopy == null) {
				return;
			}
			if (!event.affects(fWorkingCopy.getWorkItem()))
				return;

			if (event.affects(BUILD_NOTE_IDENTIFIER)) {
				updateUI();
			}	
		}
	
		public void workItemAttributeDependencyChanged(WorkItemChangeEvent event) {
		}
	}
	
	
	public static final String BUILD_NOTE_IDENTIFIER= "com.ibm.team.workitem.extension.buildNote"; //$NON-NLS-1$
	private static final String BUILD_NOTE_NAME= Messages.AddBuildNoteAction_BUILD_NOTE;
	
	private IEditorPart fTargetEditor;
	private WorkItemWorkingCopy fWorkingCopy;
	private WorkItemListener fWorkItemListener= new WorkItemListener();
	
	public void run() {
		if (fTargetEditor.getEditorInput() instanceof WorkItemEditorInput) {
			WorkItemOperation operation= new WorkItemOperation(Messages.AddBuildNoteAction_ADD_BUILD_NOTE, IWorkItem.FULL_PROFILE) {
				@Override
				protected void execute(WorkItemWorkingCopy workingCopy, IProgressMonitor monitor) throws TeamRepositoryException {
					fWorkingCopy= workingCopy;
					WorkItemUIWorkingCopy uiCopy= (WorkItemUIWorkingCopy) fWorkingCopy.getAdapter(IWorkItemUIWorkingCopy.class);
					UIWorkItemListener uiListener= uiCopy.getUIWorkItemListener();
					if (uiListener != null) {
						uiListener.addListener(fWorkItemListener, BUILD_NOTE_IDENTIFIER);
					}
					
					ITeamRepository teamRepository= workingCopy.getTeamRepository();
					IWorkItemClient workItemClient= (IWorkItemClient) teamRepository.getClientLibrary(IWorkItemClient.class);
					IProjectAreaHandle projectArea= workingCopy.getWorkItem().getProjectArea();
					IAttribute attribute= workItemClient.findAttribute(projectArea, BUILD_NOTE_IDENTIFIER, monitor);
					if (attribute == null) {
						attribute= workItemClient.createNewAttribute(projectArea, BUILD_NOTE_IDENTIFIER, AttributeTypes.LARGE_HTML, IAttribute.FULL_TEXT_KIND_CONTENT, monitor);
						attribute.setDisplayName(BUILD_NOTE_NAME);
						workItemClient.saveAttribute(attribute, monitor);
					}
					
					if (!workingCopy.getWorkItem().hasAttribute(attribute)) {
						workingCopy.getWorkItem().addCustomAttribute(attribute);
						IAuditableClient auditableClient= (IAuditableClient) workingCopy.getTeamRepository().getClientLibrary(IAuditableClient.class);
						workingCopy.getWorkItem().setValue(attribute, attribute.getDefaultValue(auditableClient, workingCopy.getWorkItem(), monitor));
					}
				}
				
				@Override
				protected void commit(WorkItemWorkingCopy[] workingCopies, IProgressMonitor monitor) throws TeamRepositoryException {
					
				}
			};
			WorkItemEditorInput input= (WorkItemEditorInput) fTargetEditor.getEditorInput();
			operation.runInJob(input.getWorkItemHandle());
		}
	}
	
	private void updateUI() {
		if (fTargetEditor instanceof WorkItemEditor) {
			WorkItemEditor workItemEditor= ((WorkItemEditor) fTargetEditor);
			workItemEditor.updateEditorLayout();
			setFocus(workItemEditor);
			WorkItemUIWorkingCopy uiCopy= (WorkItemUIWorkingCopy) fWorkingCopy.getAdapter(IWorkItemUIWorkingCopy.class);
			UIWorkItemListener uiListener= uiCopy.getUIWorkItemListener();
			if (uiListener != null) {
				uiListener.removeListener(fWorkItemListener, BUILD_NOTE_IDENTIFIER);
			}
		}
	}
	
	private void setFocus(WorkItemEditor workItemEditor) {
		List pages= workItemEditor.getPages();
		for (Object page: pages) {
			if (page instanceof CustomAttributesTab) {
				workItemEditor.setActiveEditor((CustomAttributesTab) page);
				((CustomAttributesTab) page).setFocus();
				break;
			}
		}
	}
	
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		fTargetEditor= targetEditor;
	}
	
	public void run(IAction action) {
		run();
	}
	
	public void selectionChanged(IAction action, ISelection selection) {
	}

}
