/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.extension.internal.buildnote;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.jface.util.UIUpdaterJob;
import com.ibm.team.process.common.IProjectArea;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.client.IAuditableClient;
import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.common.internal.util.CategoriesHelper;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.ICategory;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.ItemProfile;
import com.ibm.team.workitem.common.query.IQueryDescriptor;
import com.ibm.team.workitem.common.query.IQueryDescriptorHandle;
import com.ibm.team.workitem.common.query.IQueryResult;
import com.ibm.team.workitem.common.query.IResolvedResult;
import com.ibm.team.workitem.extension.internal.WorkItemExtensionUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.FileHelper;


public class GenerateBuildNotes implements IObjectActionDelegate {

	private ISelection fSelection;

	public void run(IAction action) {
		if (fSelection instanceof IStructuredSelection) {
			IStructuredSelection selection= (IStructuredSelection) fSelection;
			Object element= selection.getFirstElement();
			if (element instanceof IQueryDescriptorHandle) {
				final IQueryDescriptorHandle handle= (IQueryDescriptorHandle) element;
				UIUpdaterJob job= new UIUpdaterJob(Messages.GenerateBuildNotes_GENERATING_BUILD_NOTES) {
					private File fFile;
					@Override
					public IStatus runInBackground(IProgressMonitor monitor) {
						
						try {
							ITeamRepository teamRepository= (ITeamRepository) handle.getOrigin();
							IAuditableClient auditableClient= (IAuditableClient) teamRepository.getClientLibrary(IAuditableClient.class);
							IWorkItemClient workItemClient= (IWorkItemClient) teamRepository.getClientLibrary(IWorkItemClient.class);
							IQueryDescriptor query= auditableClient.resolveAuditable(handle, IQueryDescriptor.FULL_PROFILE, monitor);
							IProjectArea project= query.getProjectArea() != null ? (IProjectArea) auditableClient.resolveAuditable(query.getProjectArea(), ItemProfile.createFullProfile(IProjectArea.ITEM_TYPE), monitor) : null;
							
							IAttribute attribute= workItemClient.findAttribute(project, AddBuildNoteAction.BUILD_NOTE_IDENTIFIER, monitor);
							if (attribute == null)
								return Status.CANCEL_STATUS;
							
							fFile= File.createTempFile("tmp", "buildnotes.html"); //$NON-NLS-1$ //$NON-NLS-2$
							fFile.deleteOnExit();
							Writer writer= new OutputStreamWriter(new FileOutputStream(fFile), "UTF-8"); //$NON-NLS-1$
							try {
								
								String prefix= XMLString.createFromPlainText(project != null ? NLS.bind(Messages.GenerateBuildNotes_PA_NAME_BLANK, project.getName()) : "").getXMLText(); //$NON-NLS-1$
								String queryName= XMLString.createFromPlainText(query.getName()).getXMLText();
								String title= NLS.bind(Messages.GenerateBuildNotes_PREFIX_QUERYNAME_BUILDNOTES, prefix, queryName, Messages.GenerateBuildNotes_BUILD_NOTES);
								
								writer.write("<html>\n"); //$NON-NLS-1$
								writer.write(NLS.bind("<head><title>{0}</title></head>\n", title));//$NON-NLS-1$
								writer.write("<body>\n"); //$NON-NLS-1$
								
								writer.write(NLS.bind("<h1>{0}</h1>\n", title));//$NON-NLS-1$
								
								Map<String, List<IWorkItem>> workItems= new HashMap<String, List<IWorkItem>>();
								
								ItemProfile<IWorkItem> profile= ItemProfile.createProfile(IWorkItem.ITEM_TYPE, new String[] { IWorkItem.PROJECT_AREA_PROPERTY, IWorkItem.CATEGORY_PROPERTY, });
								profile= profile.createExtension(IWorkItem.CUSTOM_ATTRIBUTE_PROPERTIES);
								IQueryResult<IResolvedResult<IWorkItem>> result= workItemClient.getQueryClient().getResolvedQueryResults(query, profile);
								while (result.hasNext(monitor)) {
									IWorkItem workItem= result.next(monitor).getItem();
									
									if (!workItem.hasAttribute(attribute))
										continue;
									
									ICategory category= auditableClient.resolveAuditable(workItem.getCategory(), ICategory.FULL_PROFILE, monitor);
									if (category.isUnassigned())
										continue;
									
									String[] namePath= CategoriesHelper.getNamePath(category.getCategoryId());
									if (namePath.length == 0)
										continue;
									
									String topLevelName= namePath[0];
									List<IWorkItem> list= workItems.get(topLevelName);
									if (list == null) {
										list= new ArrayList<IWorkItem>();
										workItems.put(topLevelName, list);
									}
									list.add(workItem);
								}
								
								List<String> sortedCategories= new ArrayList<String>(workItems.keySet());
								Collections.sort(sortedCategories);
								for (String category: sortedCategories) {
									
									writer.write(NLS.bind("<h2>{0}</h2>\n", category));//$NON-NLS-1$
									writer.write("<ul>\n"); //$NON-NLS-1$
									
									List<IWorkItem> sortedWorkItems= workItems.get(category);
									for (IWorkItem workItem: sortedWorkItems) {
										
										Object value= workItem.getValue(attribute);
										if (value instanceof String && ((String) value).trim().length() > 0) {
											writer.write("<li>"); //$NON-NLS-1$
											writer.write((String) value);
											writer.write("</li>\n"); //$NON-NLS-1$
										}
									}
									
									writer.write("</ul>\n"); //$NON-NLS-1$
								}
								
								writer.write("</body>\n"); //$NON-NLS-1$
								writer.write("</html>\n"); //$NON-NLS-1$
								
							} finally {
								writer.close();
							}
							
						} catch (TeamRepositoryException x) {
							return new Status(IStatus.ERROR, WorkItemExtensionUIPlugin.PLUGIN_ID, IStatus.ERROR, Messages.GenerateBuildNotes_EXCEPTION_GENERATING_BUILD_NOTES, x);
						} catch (IOException x) {
							return new Status(IStatus.ERROR, WorkItemExtensionUIPlugin.PLUGIN_ID, IStatus.ERROR, Messages.GenerateBuildNotes_EXCEPTION_GENERATING_BUILD_NOTES, x);
						}
						
						return super.runInBackground(monitor);
					}
					@Override
					public IStatus runInUI(IProgressMonitor monitor) {
						
						try {
							FileHelper.open(fFile);
						} catch (CoreException x) {
							return new Status(IStatus.ERROR, WorkItemExtensionUIPlugin.PLUGIN_ID, IStatus.ERROR, Messages.GenerateBuildNotes_EXCEPTION_GENERATING_BUILD_NOTES, x);
						}
						
						return super.runInUI(monitor);
					}
				};
				job.schedule();
			}
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
		fSelection= selection;
	}

	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}
}
