/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import com.ibm.team.foundation.common.util.FoundationLog;
import com.ibm.team.jface.JazzResources;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditorToolkit;
import com.ibm.team.workitem.ide.ui.internal.editor.comments.SharedFormColors;
import com.ibm.team.workitem.ide.ui.internal.queryeditor.contributions.ControlPresentationBindingRegistry;


public class WorkItemIDEUIPlugin extends AbstractUIPlugin {

	public final static String PLUGIN_ID= "com.ibm.team.workitem.ide.ui"; //$NON-NLS-1$
	private final static String MID_SHADOW = "com.ibm.team.workitem.ide.ui.MID_SHADOW"; //$NON-NLS-1$
	private final static String DARK_SHADOW = "com.ibm.team.workitem.ide.ui.DARK_SHADOW"; //$NON-NLS-1$
	
	/**
	 * @deprecated use an image descriptor from the ImagePool instead
	 */
	@Deprecated
	public static final String DROP_DOWN_IMAGE_KEY= "dropdown"; //$NON-NLS-1$

	/**
	 * @deprecated use an image descriptor from the ImagePool instead
	 */
	@Deprecated
	public static final String UNKNOWN_IMAGE_KEY= "unknown"; //$NON-NLS-1$
	
	private static WorkItemIDEUIPlugin fgPlugin;
	
	private ResourceManager fResourceManager;
	private SharedFormColors fSharedFormColors;
	
	private final Object fPreferenceStoreLock= new Object();
	
	private WorkItemEditorToolkit fWorkItemEditorToolkit;

	
	public WorkItemIDEUIPlugin() {
		fgPlugin= this;
	}
	
	public static WorkItemIDEUIPlugin getDefault() {
		return fgPlugin;
	}
	
	/*
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		
		if (fWorkItemEditorToolkit != null) {
			fWorkItemEditorToolkit.doDispose();
			fWorkItemEditorToolkit= null;
		}
		
		if (fSharedFormColors != null) {
			fSharedFormColors.dispose();
			fSharedFormColors= null;
		}
		
		if (fResourceManager != null) {
			fResourceManager.dispose();
			fResourceManager= null;
		}
		
		ControlPresentationBindingRegistry.getInstance().stop();
		
		super.stop(context);
		fgPlugin= null;
	}

	/////////////////////////////////////////////////////////////////////////////////////
	// good stuff
	/////////////////////////////////////////////////////////////////////////////////////
	
	public ResourceManager getResourceManager() {
		if (fResourceManager == null)
			fResourceManager= new LocalResourceManager(JFaceResources.getResources());
		return fResourceManager;
	}

	/**
	 * Get an ImageDescriptor for the given relative path.
	 * The ImageDescriptor is registered with ImageUtils so that we can get a local URL for it.
	 * 
     * @param pathRelativeToPlugin the relative path of the image file, relative to the
     * 	root of the plug-in; the path must be legal
     * @return an image descriptor, or <code>null</code> if no image could be found
	 */
	public static ImageDescriptor getImageDescriptor(String pathRelativeToPlugin) {
		ImageDescriptor id= imageDescriptorFromPlugin(PLUGIN_ID, pathRelativeToPlugin);
		JazzResources.registerPath(id, PLUGIN_ID, pathRelativeToPlugin);
		return id;
	}

	/**
	 * Returns an image for the given image descriptor.
	 * <p>
	 * WARNING: The image's lifecycle is managed by the plugin and must not be
	 * disposed by the caller. It is only disposed when the application exits.
	 * This method is meant to be used only for images that cannot be managed by
	 * a LocalResourceManager. Most of the time you should use a
	 * LocalResourceManager instead.
	 * </p>
	 * 
	 * @param imageDesc
	 *            the image descriptor
	 * @return the image
	 */
	public static Image getImage(ImageDescriptor imageDesc) {
		return JazzResources.getImageWithDefault(getDefault().getResourceManager(), imageDesc);
	}
	
	public void log(String message, Throwable exception) {
		log(new Status(IStatus.ERROR, PLUGIN_ID, IStatus.ERROR, message, exception));
	}

	public void log(IStatus status) {
		FoundationLog.log(status);
	}

	/////////////////////////////////////////////////////////////////////////////////////

	public FormColors getSharedFormColors(Display display) {
		if (fSharedFormColors == null)
			fSharedFormColors= new SharedFormColors();
		return fSharedFormColors.getFormColors(display);
	}
	
	@Override
	public IPreferenceStore getPreferenceStore() {
		IPreferenceStore store;
		synchronized(fPreferenceStoreLock) {
			store= super.getPreferenceStore();
		}
		return store;
	}
	
	/*
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#initializeImageRegistry(org.eclipse.jface.resource.ImageRegistry)
	 */
	@Override
	protected void initializeImageRegistry(ImageRegistry registry) {
		super.initializeImageRegistry(registry);

//		registry.put("droptarget", AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/view16/droptarget.gif"));
		registry.put("search", AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/etool16/search.gif")); //$NON-NLS-1$ //$NON-NLS-2$

//		registry.put("workflow_none", AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/progress/none_disabled.gif"));
//		registry.put("workflow_info", AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/progress/info.gif"));
//		registry.put("workflow_error", AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/progress/error.gif"));

//		registry.put(PRODUCT_IMAGE_KEY, AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/obj16/product.gif"));
//		registry.put(REFACTORING_IMAGE_KEY, AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/obj16/refactoring.gif"));
//		registry.put(CHANGE_IMAGE_KEY, AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, "icons/obj16/change.gif"));

		registry.put(DROP_DOWN_IMAGE_KEY, ImagePool.DROPDOWN);
		registry.put(UNKNOWN_IMAGE_KEY, ImagePool.STATE_UNKNOWN);
}
	
	public FormToolkit getWorkItemEditorToolkit() {
		if (fWorkItemEditorToolkit == null) {
			fWorkItemEditorToolkit= new WorkItemEditorToolkit(WorkItemIDEUIPlugin.getDefault().getSharedFormColors(Display.getDefault()));
			fWorkItemEditorToolkit.markShared();
		}
		return fWorkItemEditorToolkit;
	}
	
	/**
	 * Return the color for medium shadows.
	 * @return Color
	 */
	public Color getMidShadow(){
		return PlatformUI.getWorkbench().getThemeManager().getCurrentTheme()
			.getColorRegistry().get(MID_SHADOW);
	}
	
	/**
	 * Return the color for dark shadows.
	 * @return Color
	 */
	public Color getDarkShadow(){
		return PlatformUI.getWorkbench().getThemeManager().getCurrentTheme()
			.getColorRegistry().get(DARK_SHADOW);
	}
}
