/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.client.WorkItemOperation;
import com.ibm.team.workitem.client.WorkItemWorkingCopy;
import com.ibm.team.workitem.common.model.ISubscriptions;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.common.model.ItemProfile;

public abstract class AbstractSubscriptionWorkItemAction implements IObjectActionDelegate {
	private ISelection fSelection;

	/*
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		final Map<ITeamRepository, List<IWorkItemHandle>> items= new HashMap<ITeamRepository, List<IWorkItemHandle>>();
		if (!(fSelection instanceof IStructuredSelection)) {
			return;
		}
		
		@SuppressWarnings("rawtypes")
		List list= ((IStructuredSelection)fSelection).toList();
		for (Object entry : list) {
			if (!(entry instanceof IWorkItemHandle)) {
				continue;
			}
			Object origin= ((IWorkItemHandle)entry).getOrigin();
			if (!(origin instanceof ITeamRepository)) {
				continue;
			}

			List<IWorkItemHandle> itemsPerRepository= items.get((ITeamRepository)origin);
			if (itemsPerRepository == null) {
				itemsPerRepository= new ArrayList<IWorkItemHandle>();
				items.put((ITeamRepository)origin, itemsPerRepository);
			}
			itemsPerRepository.add((IWorkItemHandle)entry);
		}

		if(items.isEmpty()){
			return;
		}
		editSubscriptions(items);
	}

	protected void editSubscriptions(Map<ITeamRepository, List<IWorkItemHandle>> items) {
		ItemProfile<IWorkItem> profile= ItemProfile.<IWorkItem> createProfile(IWorkItem.ITEM_TYPE, IWorkItem.SUBSCRIPTIONS_PROPERTY);
		for (Entry<ITeamRepository, List<IWorkItemHandle>> entry : items.entrySet()) {
			final ITeamRepository repository= entry.getKey();
			WorkItemOperation workItemOperation= new WorkItemOperation(Messages.AbstractSubscriptionWorkItemAction_SUBSCRIBE_TO_WORKITEMS, profile) {
				@Override
				protected void execute(WorkItemWorkingCopy[] workingCopies, IProgressMonitor monitor) throws TeamRepositoryException {
					editSubscriptionToRepo(repository, workingCopies, monitor);
				}
			};

			List<IWorkItemHandle> workItems= entry.getValue();
			workItemOperation.runInJob(workItems.toArray(new IWorkItemHandle[workItems.size()]));
		}
	}

	protected void editSubscriptionToRepo(ITeamRepository repo, WorkItemWorkingCopy[] workingCopies, IProgressMonitor monitor) {
		monitor.beginTask(Messages.AbstractSubscriptionWorkItemAction_ADJUSTING_WORK_ITEMS_TASK_NAME, workingCopies.length);
		for (WorkItemWorkingCopy item : workingCopies) {
			ISubscriptions subscriptions= item.getWorkItem().getSubscriptions();
			editSubscriptionToWorkItem(item.getWorkItem(), subscriptions);
			monitor.worked(1);
		}
		monitor.done();
	}

	protected abstract boolean editSubscriptionToWorkItem(IWorkItem item, ISubscriptions subscriptions);

	/*
	 * @see
	 * org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action
	 * .IAction, org.eclipse.jface.viewers.ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		fSelection= selection;
	}

	/*
	 * @see
	 * org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.
	 * action.IAction, org.eclipse.ui.IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {}

}