/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.HTMLTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.foundation.rcp.ui.internal.util.Utils;
import com.ibm.team.foundation.rcp.ui.util.FoundationUIUtils;
import com.ibm.team.jface.preview.DomainAdapterUtils;
import com.ibm.team.jface.preview.IDomainAdapter;
import com.ibm.team.jface.util.UIUpdaterJob;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.Location;
import com.ibm.team.repository.common.TeamRepositoryException;

import com.ibm.team.workitem.client.IAuditableClient;
import com.ibm.team.workitem.common.internal.rest.IAttachmentRestService;
import com.ibm.team.workitem.common.model.IAttachment;
import com.ibm.team.workitem.common.model.IAttachmentHandle;
import com.ibm.team.workitem.common.text.WorkItemTextUtilities;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;

public class CopyAttachmentToClipboardAction implements IObjectActionDelegate {

	private static class InternalUIUpdaterJob extends UIUpdaterJob {
		
		public InternalUIUpdaterJob(String name) {
			super(name);
		}
		
		public boolean needsRunInBackground() {
			return true;
		}
		
		public void run() {
			if (needsRunInBackground())
				schedule();
			else
				runInUI(null);
		}
	}
	
	private ISelection fSelection;
	private IWorkbenchPart fWorkbenchPart;

	public CopyAttachmentToClipboardAction() {
		super();
	}

	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		fWorkbenchPart= targetPart;
	}

	public void selectionChanged(IAction action, ISelection selection) {
		fSelection= selection;
		if (action != null)
			action.setEnabled(isEnabled(selection));
	}

	private boolean isEnabled(ISelection selection) {
		return selection instanceof IStructuredSelection && !((IStructuredSelection) selection).isEmpty();
	}

	public void run(IAction action) {
		InternalUIUpdaterJob job= new InternalUIUpdaterJob(Messages.CopyAttachmentToClipboardAction_COPY_TO_CLIPBOARD) {
			
			private List<IAttachment> fData= new ArrayList<IAttachment>();
			
			@Override
			public boolean needsRunInBackground() {
				IStructuredSelection selection= ((IStructuredSelection) fSelection);
				for (Iterator iter= selection.iterator(); iter.hasNext();) {
					IAttachmentHandle attachmentHandle= (IAttachmentHandle) iter.next();
					ITeamRepository teamRepository= (ITeamRepository) attachmentHandle.getOrigin();
					IAuditableClient auditableClient= (IAuditableClient) teamRepository.getClientLibrary(IAuditableClient.class);
					IAttachment attachment= auditableClient.findCachedAuditable(attachmentHandle, IAttachment.DEFAULT_PROFILE);
					if (attachment == null)
						return true;
					fData.add(attachment);
				}
				return false;
			}
			
			@Override
			public IStatus runInBackground(IProgressMonitor monitor) {
				try {
					IStructuredSelection selection= ((IStructuredSelection) fSelection);
					for (Iterator iter= selection.iterator(); iter.hasNext();) {
						IAttachmentHandle attachmentHandle= (IAttachmentHandle) iter.next();
						ITeamRepository teamRepository= (ITeamRepository) attachmentHandle.getOrigin();
						IStatus status= FoundationUIUtils.validateRepository(teamRepository, true, monitor);
						if (!status.isOK())
							return status;
						IAuditableClient auditableClient= (IAuditableClient) teamRepository.getClientLibrary(IAuditableClient.class);
						IAttachment attachment= auditableClient.resolveAuditable(attachmentHandle, IAttachment.DEFAULT_PROFILE, monitor);
						if (attachment == null)
							return null;
						fData.add(attachment);
					}
				} catch (TeamRepositoryException x) {
					return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, IStatus.OK, Messages.CopyAttachmentToClipboardAction_EXCEPTION_RESOLVING_ATTACHMENT, x);
				}
				return Status.OK_STATUS;
			}
			
			@Override
			public IStatus runInUI(IProgressMonitor monitor) {
				Iterator iterator= ((IStructuredSelection) fSelection).iterator();
				StringBuffer buffer= new StringBuffer();
				StringBuffer htmlBuffer= new StringBuffer();
				while (iterator.hasNext()) {
					IAttachmentHandle attachmentHandle= (IAttachmentHandle) iterator.next();
					ITeamRepository teamRepository= (ITeamRepository) attachmentHandle.getOrigin();
					IAuditableClient auditableClient= (IAuditableClient) teamRepository.getClientLibrary(IAuditableClient.class);
					IAttachment attachment= auditableClient.findCachedAuditable(attachmentHandle, IAttachment.DEFAULT_PROFILE);
					if (attachment != null) {
						try {
							String link= getLink(teamRepository, attachment);
							
							// plain text
							buffer.append(getLabel(attachment, link));
							if (iterator.hasNext())
								buffer.append("\r\n"); //$NON-NLS-1$
							
							// html text
							htmlBuffer.append("<a itemId=\""); //$NON-NLS-1$
							htmlBuffer.append(attachment.getItemId().getUuidValue());
							htmlBuffer.append("\" href=\""); //$NON-NLS-1$
							htmlBuffer.append(link);
							htmlBuffer.append("\">"); //$NON-NLS-1$
							htmlBuffer.append(getHTMLLabel(attachment));
							htmlBuffer.append("</a>"); //$NON-NLS-1$
							if (iterator.hasNext())
								htmlBuffer.append("<br/>"); //$NON-NLS-1$
							
						} catch (IllegalStateException e) {
							WorkItemIDEUIPlugin.getDefault().log(Messages.CopyAttachmentToClipboardAction_ERROR_CREATING_URL, e);
						} catch (TeamRepositoryException e) {
							WorkItemIDEUIPlugin.getDefault().log(Messages.CopyAttachmentToClipboardAction_ERROR_CREATING_URL, e);
						}
					}
				}
				Clipboard cb= new Clipboard(fWorkbenchPart.getSite().getShell().getDisplay());
				try {
					cb.setContents(
						new Object[] { buffer.toString(), Utils.escapeForHTMLTransfer(htmlBuffer.toString()), },
						new Transfer[] { TextTransfer.getInstance(), HTMLTransfer.getInstance(), }
					);
				} finally {
					cb.dispose();
				}
				return Status.OK_STATUS;
			}
		};
		job.run();
	}
	
	private String getLink(ITeamRepository repository, IAttachment attachment) throws IllegalStateException, TeamRepositoryException {
		Location location= Location.namedLocation(attachment, getRepositoryURI(repository), null, IAttachmentRestService.class.getName());
		return location.toAbsoluteUri().toString();
	}
	
	protected String getRepositoryURI(ITeamRepository repository) {
		String publicUriRoot= repository.publicUriRoot();
		if (publicUriRoot != null) {
			return publicUriRoot;
		}
		return repository.getRepositoryURI();
	}

	protected String getLabel(IAttachment attachment, String link) {
		IDomainAdapter domainAdapter= DomainAdapterUtils.getDomainAdapter(attachment);
		if (domainAdapter != null)
			return domainAdapter.generateTitle(attachment);
		return  XMLString.createFromXMLText(getHTMLLabel(attachment)).getPlainText();
	}

	protected String getHTMLLabel(IAttachment attachment) {
		return XMLString.createFromPlainText(attachment.getName()).getXMLText() + " (" + WorkItemTextUtilities.getAttachmentId(attachment) + ")";  //$NON-NLS-1$ //$NON-NLS-2$
	}
}
