/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.ide.ui.internal.actions;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.foundation.ide.ui.internal.styledtextviewer.actions.StyledEditingAction;
import com.ibm.team.foundation.rcp.core.text.IStyledDocument;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.staticrichtext.HyperLinkCreationDialog;


@SuppressWarnings("restriction")
public class InsertHyperLinkAction extends StyledEditingAction {

	private static final String INSERT_HYPER_LINK= "com.ibm.magnolia.edit.inserthyperlink"; //$NON-NLS-1$

	public InsertHyperLinkAction() {
		this(null, null);
	}

	public InsertHyperLinkAction(IWorkbenchPart part, ISourceViewer sourceViewer) {
		this(Messages.InsertHyperLinkAction_INSERT_HYPER_LINK_LABEL, Messages.InsertHyperLinkAction_INSERT_HYPER_LINK_DESCRIPTION, InsertHyperLinkAction.INSERT_HYPER_LINK, null, part, sourceViewer);
	}

	private InsertHyperLinkAction(String text, String tooltip, String cmdId, ImageDescriptor image, IWorkbenchPart part, ISourceViewer sourceViewer) {
		super(text, cmdId, image, sourceViewer);
		setToolTipText(tooltip);
		setPart(part);
		update();
	}

	@Override
	public void update() {
		setEnabled(canRun());
	}

	@Override
	public void run() {
		update();
		if (!isEnabled()) {
			return;
		}

		HyperLinkCreationDialog dialog= new HyperLinkCreationDialog(getPart().getSite().getShell());
		if (Dialog.OK == dialog.open()) {
			try {
				pasteReference(dialog.getLabel(), new URI(dialog.getURL()));
			} catch (URISyntaxException ex) {
				WorkItemIDEUIPlugin.getDefault().log(Messages.InsertHyperLinkAction_EXCEPTION_RESOLVING_URL, ex);
			}
		}
	}

	private boolean canRun() {
		if (!(getPart() instanceof IEditorPart)) {
			return false;
		}

		ISourceViewer sourceViewer= getSourceViewer();
		if (sourceViewer == null 
				|| !(sourceViewer.getDocument() instanceof IStyledDocument) 
				|| !(sourceViewer.getSelectionProvider().getSelection() instanceof ITextSelection) 
				|| !sourceViewer.isEditable()) {
			return false;
		}
		return true;
	}

	private void pasteReference(String label, URI uri) {
		try {
			ITextSelection textSelection= (ITextSelection)getSourceViewer().getSelectionProvider().getSelection();
			int offset= textSelection.getOffset();
			IStyledDocument document= (IStyledDocument)getSourceViewer().getDocument();
			document.replaceHTML(offset, textSelection.getLength(), XMLString.createFromXMLText("<a href=\"" + uri.toString() + "\">" + label + "</a>")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			getSourceViewer().getSelectionProvider().setSelection(new TextSelection(offset + label.length(), 0));
		} catch (BadLocationException x) {
			WorkItemIDEUIPlugin.getDefault().log(Messages.InsertHyperLinkAction_EXCEPTION_INSERTING_REFERENCES, x);
		}
	}
}
