/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.ide.ui.internal.actions;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IWorkbench;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.workitem.common.model.ISubscriptions;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.SubscriptionEditorDialog;
import com.ibm.team.workitem.ide.ui.internal.editor.SubscriptionInfoSet;

public class ManageSubscriptionsToWorkItemAction extends AbstractSubscriptionWorkItemAction {
	private SubscriptionInfoSet fSubscriptions;

	@Override
	protected void editSubscriptions(Map<ITeamRepository, List<IWorkItemHandle>> items) {
		IWorkbench workBench= WorkItemIDEUIPlugin.getDefault().getWorkbench();
		if (workBench == null || workBench.getActiveWorkbenchWindow() == null) {
			return;
		}

		if (items.keySet().size() > 1) {
			MessageDialog.openInformation(workBench.getActiveWorkbenchWindow().getShell(), "Manage Subscriptions", "Cannot manage subscriptions for work items in multiple repositories");
			return;
		}

		Entry<ITeamRepository, List<IWorkItemHandle>> first= items.entrySet().iterator().next();
		SubscriptionEditorDialog dialog= new SubscriptionEditorDialog(workBench.getActiveWorkbenchWindow().getShell(), first.getKey(), first.getValue());
		int result= dialog.open();
		fSubscriptions= dialog.getResult();

		if (result == Window.OK && fSubscriptions != null && fSubscriptions.hasChanges()) {
			super.editSubscriptions(items);
		}
	}

	@Override
	protected boolean editSubscriptionToWorkItem(IWorkItem item, ISubscriptions subscriptions) {
		boolean changes= false;
		for (IContributor contributor : fSubscriptions.getRemoved()) {
			if (subscriptions.contains(contributor)) {
				subscriptions.remove(contributor);
				changes= true;
			}
		}
		for (IContributor contributor : fSubscriptions.getAdded()) {
			if (!subscriptions.contains(contributor)) {
				subscriptions.add(contributor);
				changes= true;
			}
		}
		for (IContributor contributor : fSubscriptions.getAssignedToAll()) {
			if (!subscriptions.contains(contributor)) {
				subscriptions.add(contributor);
				changes= true;
			}
		}

		return changes;
	}
}
