/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.ide.ui.internal.actions;

import java.net.URI;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.foundation.ide.ui.internal.styledtextviewer.actions.StyledEditingAction;
import com.ibm.team.foundation.rcp.core.text.IStyledDocument;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.client.IAuditableClient;
import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.client.URIService;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.common.model.IWorkItemType;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditorInput;
import com.ibm.team.workitem.rcp.ui.workitempicker.WorkItemSelectionDialog;


public class ReferenceWorkItemAction extends StyledEditingAction {
	
	public static final String REFERENCE_WORK_ITEM= "com.ibm.magnolia.edit.referenceworkitem"; //$NON-NLS-1$

	public ReferenceWorkItemAction() {
		this(null, null);
	}
	
	public ReferenceWorkItemAction(IWorkbenchPart part, ISourceViewer sourceViewer) {
		this(Messages.ReferenceWorkItemAction_INSERT_WORKITEM, Messages.ReferenceWorkItemAction_INSERT_REFERENCE_TO_WORKITEM, ReferenceWorkItemAction.REFERENCE_WORK_ITEM, null, part, sourceViewer);
	}
	
	private ReferenceWorkItemAction(String text, String tooltip, String cmdId, ImageDescriptor image, IWorkbenchPart part, ISourceViewer sourceViewer) {
		super(text, cmdId, image, sourceViewer);
		setToolTipText(tooltip);
		setPart(part);
		update();
	}
	
	@Override
	public void update() {
		setEnabled(canRun());
	}
	
	@Override
	public void run() {
		
		update();
		
		if (!isEnabled())
			return;
		
		WorkItemEditorInput input= (WorkItemEditorInput) ((IEditorPart) getPart()).getEditorInput();
		IWorkItemHandle handle= WorkItemSelectionDialog.getWorkItem(getPart().getSite().getShell(), input.getWorkingCopy().getWorkItem().getProjectArea());
		if (handle == null)
			return;

		try {
			URI uri= URIService.createAuditableURI(((ITeamRepository) handle.getOrigin()), handle);
			String label= createLabel(handle, false);
			pasteReference(label, uri);
		} catch (TeamRepositoryException x) {
			WorkItemIDEUIPlugin.getDefault().log(Messages.ReferenceWorkItemAction_EXCEPTION_RESOLVING_WORKITEM, x);
		}			
	}

	public String createLabel(IWorkItemHandle handle, boolean isSentenceBegin) throws TeamRepositoryException {

		ITeamRepository teamRepository= (ITeamRepository) handle.getOrigin();
		IAuditableClient auditableClient= (IAuditableClient) teamRepository.getClientLibrary(IAuditableClient.class);
		IWorkItemClient workItemClient= (IWorkItemClient) teamRepository.getClientLibrary(IWorkItemClient.class);
		
		IWorkItem workItem= auditableClient.resolveAuditable(handle, IWorkItem.SMALL_PROFILE, null);
		IWorkItemType type= workItemClient.findWorkItemType(workItem.getProjectArea(), workItem.getWorkItemType(), null);
		String typeName= type != null ? type.getDisplayName() : workItem.getWorkItemType();
		if (!isSentenceBegin)
			typeName= typeName.toLowerCase();
		return typeName + " " + workItem.getId(); //$NON-NLS-1$
	}

	private boolean canRun() {
		
		if (!(getPart() instanceof IEditorPart))
			return false;
		
		ISourceViewer sourceViewer= getSourceViewer();
		if (sourceViewer == null)
			return false;
		
		if (!(sourceViewer.getDocument() instanceof IStyledDocument))
			return false;
		
		if (!(sourceViewer.getSelectionProvider().getSelection() instanceof ITextSelection))
			return false;
		
		if (!sourceViewer.isEditable())
			return false;
		
		IEditorInput input= ((IEditorPart) getPart()).getEditorInput();
		return input instanceof WorkItemEditorInput && ((WorkItemEditorInput) input).isResolved();
	}
	
	private void pasteReference(String label, URI uri) {
		try {
			ITextSelection textSelection= (ITextSelection) getSourceViewer().getSelectionProvider().getSelection();
			int offset= textSelection.getOffset();
			IStyledDocument document= (IStyledDocument) getSourceViewer().getDocument();
			document.replaceHTML(offset, textSelection.getLength(), XMLString.createFromXMLText("<a href=\"" + uri.toString() + "\">" + label + "</a>")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			getSourceViewer().getSelectionProvider().setSelection(new TextSelection(offset + label.length(), 0));
		} catch (BadLocationException x) {
			WorkItemIDEUIPlugin.getDefault().log(Messages.ReferenceWorkItemAction_EXCEPTION_INSERTING_REFERENCES, x);
		}
	}
}