/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressService;

import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditor;


public class RevertWorkItemEditorAction extends Action {
	
	private IEditorPart fEditor;
	
	public RevertWorkItemEditorAction(String text, String tooltip, String cmdId, ImageDescriptor image) {
		setText(text);
		setToolTipText(tooltip);
		setActionDefinitionId(cmdId);
		setImageDescriptor(image);
	}
	
	public void run() {
		if (fEditor instanceof WorkItemEditor) {
			IProgressService progressService= PlatformUI.getWorkbench().getProgressService();
			try {
				progressService.runInUI(PlatformUI.getWorkbench().getActiveWorkbenchWindow(), new IRunnableWithProgress() {
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						((WorkItemEditor) fEditor).doRevert(monitor);
					}
				}, null);
			} catch (InvocationTargetException x) {
				WorkItemIDEUIPlugin.getDefault().log(Messages.RevertWorkItemEditorAction_ERROR_RETRIEVING_EDITOR, x);
			} catch (InterruptedException x) {
				WorkItemIDEUIPlugin.getDefault().log(Messages.RevertWorkItemEditorAction_ERROR_RETRIEVING_EDITOR, x);
			}
		}
	}
	
	public void setEditor(IEditorPart editor) {
		fEditor= editor;
	}
}
