/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.actions;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.foundation.client.util.FoundationJob;
import com.ibm.team.jface.util.UIUpdaterJob;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.client.IAuditableClient;
import com.ibm.team.workitem.common.model.IAttachment;
import com.ibm.team.workitem.common.model.IAttachmentHandle;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.editor.part.AttachmentsPart;

public class SaveAttachmentAction implements IObjectActionDelegate {
	
	private ISelection fSelection;
	private Shell fShell;

	public void run(IAction action) {
		
		if (!shouldEnable() || fShell == null)
			return;
		
		final List<IAttachmentHandle> handles = getHandleList();
		UIUpdaterJob job= new UIUpdaterJob(Messages.SaveAttachmentAction_RESOLVING_ATTACHMENTS) {
			List<IAttachment> fAttachments;
			@Override
			public IStatus runInBackground(IProgressMonitor monitor) {
				try {
					ITeamRepository teamRepository= (ITeamRepository) handles.get(0).getOrigin();
					List<IAttachment> resolvedStates= new ArrayList<IAttachment>();
					for (IAttachmentHandle handle: handles) {
						if (handle instanceof IAttachment && ((IAttachment)handle).isComplete()) {
							resolvedStates.add((IAttachment)handle);
						}
					}
					handles.removeAll(resolvedStates);
					IAuditableClient auditableClient= (IAuditableClient) teamRepository.getClientLibrary(IAuditableClient.class);
					if (handles.size() == 1) {
						fAttachments= auditableClient.resolveAuditablesPermissionAware(handles, IAttachment.SMALL_PROFILE, monitor);
					} else {
						fAttachments= auditableClient.resolveAuditables(handles, IAttachment.SMALL_PROFILE, monitor);
					}
					if (fAttachments.isEmpty()) {
						fAttachments= new ArrayList<IAttachment>();
					}
					fAttachments.addAll(resolvedStates);
				} catch (TeamRepositoryException x) {
					return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.SaveAttachmentAction_EXCEPTION_RESOLVING_ATTACHMENTS, x);
				}
				return Status.OK_STATUS;
			}
			@Override
			public IStatus runInUI(IProgressMonitor monitor) {
				run(fAttachments);
				return Status.OK_STATUS;
			}
		};
		job.schedule();
	}
	
	private void run(List<IAttachment> list) {
		ITeamRepository teamRepository= (ITeamRepository) list.get(0).getOrigin();
		
		if (list.size() == 1) { // save as
			FileDialog dialog= new FileDialog(fShell, SWT.SAVE);
			dialog.setText(Messages.SaveAttachmentAction_SAVE_ATTACHMENT);
			String path= WorkItemIDEUIPlugin.getDefault().getPreferenceStore().getString(AttachmentsPart.ATTACHMENTS_SAVE_LOCATION);
			if (path != null) {
				dialog.setFilterPath(path);
			}
			dialog.setFileName(list.get(0).getName());
			String fname;
			DIALOG_REENTRY: while ((fname= dialog.open()) != null) {
				File file= new File(fname);
				if (file.exists()) {
					MessageDialog md= new MessageDialog(fShell, Messages.SaveAttachmentAction_SAVE_ATTACHMENT, null, NLS.bind(Messages.SaveAttachmentAction_ALREADY_EXISTS, fname), MessageDialog.WARNING, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL}, 1);
					if (md.open() != 0) {
						continue DIALOG_REENTRY;
					}
				}
				new FileSaveJob(teamRepository, list.get(0), file).schedule();
				break DIALOG_REENTRY;
			}
			WorkItemIDEUIPlugin.getDefault().getPreferenceStore().setValue(AttachmentsPart.ATTACHMENTS_SAVE_LOCATION, Path.fromOSString(dialog.getFilterPath()).makeAbsolute().toString());
		} else if (list.size() > 1) { // save all
			DirectoryDialog dialog= new DirectoryDialog(fShell);
			dialog.setText(Messages.SaveAttachmentAction_SAVE_ATTACHMENTS);
			String path= WorkItemIDEUIPlugin.getDefault().getPreferenceStore().getString(AttachmentsPart.ATTACHMENTS_SAVE_LOCATION);
			if (path != null) {
				dialog.setFilterPath(path);
			}
			dialog.setMessage(Messages.SaveAttachmentAction_SELECT_DIRECTORY);
			String dir;
			if ((dir= dialog.open()) != null) {
				boolean yestoall= false;
				List<IAttachment> attachments= new ArrayList<IAttachment>();
				List<File> files= new ArrayList<File>();
				for (IAttachment handle: list) {
					if (handle == null) {
						continue;
					}
					File file= new File(dir, handle.getName());
					if (!yestoall && file.exists()) {
						MessageDialog md= new MessageDialog(fShell, Messages.SaveAttachmentAction_SAVE_ATTACHMENT, null, NLS.bind(Messages.SaveAttachmentAction_ALREADY_EXISTS, file.getPath()), MessageDialog.WARNING, new String[] { IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL, IDialogConstants.YES_TO_ALL_LABEL}, 1);
						int r= md.open();
						if (r == 2) {
							yestoall= true;
						} else if (r != 0) {
							continue;
						}
					}
					attachments.add(handle);
					files.add(file);
				}
				new FileSaveJob(teamRepository, attachments.toArray(new IAttachment[attachments.size()]), files.toArray(new File[files.size()])).schedule();
			}
			WorkItemIDEUIPlugin.getDefault().getPreferenceStore().setValue(AttachmentsPart.ATTACHMENTS_SAVE_LOCATION, Path.fromOSString(dialog.getFilterPath()).makeAbsolute().toString());
		}
	}
	
	private List<IAttachmentHandle> getHandleList() {
		ArrayList<IAttachmentHandle> result = new ArrayList<IAttachmentHandle>();
		for (Object o: ((IStructuredSelection)fSelection).toList()){
			if (o instanceof IAttachmentHandle) {
				result.add((IAttachmentHandle)o);
			}
		}
		return result;
	}
	
	public void selectionChanged(IAction action, ISelection selection) {
		fSelection= selection;
		action.setEnabled(shouldEnable());
	}
	
	private boolean shouldEnable() {
		return !fSelection.isEmpty() && fSelection instanceof IStructuredSelection && ((IStructuredSelection) fSelection).getFirstElement() instanceof IAttachmentHandle;
	}
	
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		fShell=targetPart.getSite().getShell();
	}
	
	private static class FileSaveJob extends FoundationJob {
		
		private ITeamRepository fTeamRepository;
		private IAttachment[] fAttachments;
		private File[] fFiles;
		
		public FileSaveJob(ITeamRepository teamRepository, IAttachment[] attachments, File[] files) {
			super(Messages.SaveAttachmentAction_SAVING_ATTACHMENTS);
			setUser(true);
			fTeamRepository= teamRepository;
			fAttachments= attachments;
			fFiles= files;
		}

		public FileSaveJob(ITeamRepository teamRepository, IAttachment attachment, File file) {
			this(teamRepository, new IAttachment[] { attachment }, new File[] { file });
		}

		protected IStatus runProtected(IProgressMonitor monitor) {
			int n= fAttachments.length;
			IStatus[] stati= new IStatus[n];
			String taskMessage= NLS.bind(n == 1 ? Messages.SaveAttachmentAction_ATTACHMENT_NOUN_SINGULAR
												: Messages.SaveAttachmentAction_ATTACHMENT_NOUN_PLURAL, Integer.toString(n));
			monitor.beginTask(taskMessage, n);
			for (int i= 0; i < n; i++) {
				SubProgressMonitor submon= new SubProgressMonitor(monitor, 1);
				try {
					checkCancelled(monitor);
					IAuditableClient auditableClient= (IAuditableClient) fTeamRepository.getClientLibrary(IAuditableClient.class);
					IAttachment attachment= fAttachments[i];
					if (!attachment.isComplete()) {
						attachment = auditableClient.resolveAuditable(attachment, IAttachment.DEFAULT_PROFILE, submon);
					}
					checkCancelled(monitor);
					IContent content= attachment.getContent();
					submon.beginTask(null, (int) content.getEstimatedConvertedLength());
					submon.subTask(NLS.bind(Messages.SaveAttachmentAction_DOWNLOADING, attachment.getName()));
					try {
						OutputStream out= new FileOutputStream(fFiles[i]);
						try {
							fTeamRepository.contentManager().retrieveContent(content, out, submon);
						} finally {
							out.close();
						}
					} finally {
						submon.done();
					}
					stati[i]= Status.OK_STATUS;
				} catch (OperationCanceledException e) {
					for (int j= i; j < n; j++) {
						stati[j]= Status.CANCEL_STATUS;
					}
					break;
				} catch (TeamRepositoryException x) {
					stati[i]= new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, IStatus.OK, NLS.bind(Messages.SaveAttachmentAction_ERROR_RETRIEVING_FROM_REPO, fAttachments[i].getName()), x);
				} catch (IOException x) {
					stati[i]= new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, IStatus.OK, NLS.bind(Messages.SaveAttachmentAction_ERROR_SAVING_TODISK, fAttachments[i].getName()), x);
				}
			}
			monitor.done();
			return new MultiStatus(WorkItemIDEUIPlugin.PLUGIN_ID, IStatus.OK, stati, Messages.SaveAttachmentAction_ERRORS_SAVING_ATTACHMENTS, null);
		}
		
		private static void checkCancelled(IProgressMonitor monitor) throws OperationCanceledException {
			if(monitor.isCanceled()) {
				throw new OperationCanceledException();
			}
		}
	}
}
