/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.process.ide.ui.ProcessAspect;

import com.ibm.team.workitem.ide.ui.internal.aspecteditor.enumeration.UIEnumeration.UILiteral;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeCategory.Type;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow.WFAction;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow.WFResolution;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow.WFState;


public class AspectEditorLabelProvider extends LabelProvider {
	
	private ProcessAspect fAspect;
	private ResourceManager fResourceManager;

	public AspectEditorLabelProvider(ProcessAspect aspect, ResourceManager resourceManager) {
		fAspect= aspect;
		fResourceManager= resourceManager;
	}

	@Override
	public String getText(Object element) {
		if (element == null) {
			return null;
		}
		if (element instanceof ModeledElement) {
			return getModeledElementLabel((ModeledElement) element);
		}
		if (element instanceof UILiteral) {
			return ((UILiteral) element).getName();
		}
		// TODO: Implement more types
		return element.toString();
	}

	@Override
	public Image getImage(final Object element) {
		if (element instanceof Type) {
			Type type= (Type) element;
			return getImage(type, type.getIcon());
		}
		if (element instanceof WFState) {
			WFState state= (WFState) element;
			return getImage(state, state.getIcon());
		}
		if (element instanceof WFResolution) {
			WFResolution resolution= (WFResolution) element;
			return getImage(resolution, resolution.getIcon());
		}
		if (element instanceof WFAction) {
			WFAction action= (WFAction) element;
			return getImage(action, action.getIcon());
		}
		if (element instanceof UILiteral) {
			UILiteral literal= (UILiteral) element;
			return getImage(literal, literal.getIcon());
		}
		// TODO: Implement more types
		return null;
	}

	private String getModeledElementLabel(ModeledElement element) {
		String name= element.getName();
		if (name != null) {
			return name;
		} else {
			return element.getIdentifier();
		}
	};
	
	private Image getImage(final Object element, String iconUrl) {
		if (iconUrl != null) {
			return AspectEditorUtil.getImage(fAspect.getProcessContainerWorkingCopy(), iconUrl, fResourceManager, new Runnable() {
				public void run() {
					fireLabelProviderChanged(new LabelProviderChangedEvent(AspectEditorLabelProvider.this, element));
				}
			});
		}
		return null;
	}
}
