/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.icu.text.Collator;
import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.foundation.rcp.ui.internal.util.Utils;
import com.ibm.team.jface.JazzResources;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.common.registry.IEndPointDescriptor;
import com.ibm.team.links.common.registry.ILinkType;
import com.ibm.team.links.common.registry.ILinkTypeRegistry;
import com.ibm.team.process.client.workingcopies.IProcessAttachmentsWorkingCopy;
import com.ibm.team.process.client.workingcopies.IProcessContainerWorkingCopy;
import com.ibm.team.process.common.IProcessAttachment;
import com.ibm.team.process.common.IProcessAttachmentHandle;
import com.ibm.team.process.common.IProcessContainerHandle;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.common.validation.IItemValidator;
import com.ibm.team.repository.common.validation.IStringSizeConstraintErrorStatus;
import com.ibm.team.workitem.client.internal.util.RemoteImages;
import com.ibm.team.workitem.common.internal.ResourceLocation;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.WorkItemLinkTypes;
import com.ibm.team.workitem.ide.ui.internal.ImagePool;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.teamoperation.WIAttribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeCategory;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeCategory.Type;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow;
import com.ibm.team.workitem.rcp.ui.WorkItemUI;
import com.ibm.team.workitem.rcp.ui.internal.LinkLabelProvider;

public class AspectEditorUtil {
	public final static String ID= "id"; //$NON-NLS-1$
	public static final String WORKFLOWS_CONFIGURATION_POINT= "com.ibm.team.workitem.configuration.workflow"; //$NON-NLS-1$
	public static final String WORKFLOW_BINDING_CONFIGURATION_POINT= "com.ibm.team.workitem.configuration.workflowBinding"; //$NON-NLS-1$
	public static final String ENUMERATIONS_CONFIGURATION_POINT = "com.ibm.team.workitem.configuration.enumerations"; //$NON-NLS-1$
	public static final String QUICKINFORMATION_CONFIGURATION_POINT= "com.ibm.team.workitem.editor.configuration.quickinformationconfiguration"; //$NON-NLS-1$
	public static final String EDITOR_PRESENTATIONS_CONFIGURATION_POINT	= "com.ibm.team.workitem.editor.configuration.presentations"; //$NON-NLS-1$
	
	public static final String ERROR= "error"; //$NON-NLS-1$
	public static final String MESSAGE= "message"; //$NON-NLS-1$
	
	public static final int MAX_SPECIFICATION_STRING_LENGHT= 250;

	private static Set<String> validSuffixes= new HashSet<String>();
	static {
		validSuffixes.add("jpg"); //$NON-NLS-1$
		validSuffixes.add("jpeg"); //$NON-NLS-1$
		validSuffixes.add("ico"); //$NON-NLS-1$
		validSuffixes.add("gif"); //$NON-NLS-1$
		validSuffixes.add("tif"); //$NON-NLS-1$
		validSuffixes.add("tiff"); //$NON-NLS-1$
		validSuffixes.add("bmp"); //$NON-NLS-1$
		validSuffixes.add("gif"); //$NON-NLS-1$
		validSuffixes.add("png"); //$NON-NLS-1$
	}
	
	public static boolean equals(Object o1, Object o2) {
		if (o1 == null && o2 == null) {
			return true;
		}
		if (o1 != null) {
			return o1.equals(o2);
		}
		return false;
	}
	
	public static boolean equalMaps(Map map1, Map map2) {
		for (Object key: map1.keySet()) {
			if (!map2.containsKey(key)) {
				return false;
			}
			if (!AspectEditorUtil.equals(map1.get(key), map2.get(key))) {
				return false;
			}
		}
		for (Object key: map2.keySet()) {
			if (!map1.containsKey(key)) {
				return false;
			}
			if (!AspectEditorUtil.equals(map1.get(key), map2.get(key))) {
				return false;
			}
		}
		return true;
	}
	

	public static int getIdNumberSuffix(String id) {
		int dot=id.lastIndexOf("."); //$NON-NLS-1$
		if (id.length() < dot+2) {
			return -1;
		}
		String number= id.substring(dot+2);
		try {
			int i= Integer.valueOf(number);
			return i;
		} catch (NumberFormatException e) {
			// ignore
		}
		return -1;
	}
	
	public static void adaptCombo(Combo combo) {
		combo.setVisibleItemCount(Utils.MAX_VISIBLE_ITEMS_PER_COMBO);
	}
	
	public static String attributeListAsString(List<WIAttribute> attrs){
		Iterator<WIAttribute> iter = attrs.iterator();
		String list = iter.next().getName();
		while (iter.hasNext()) {
			list = NLS.bind(Messages.AspectEditorUtil_LIST_OF_TYPES, list, iter.next().getName());
		}
		return list;
	}
	
	public static String getTypeCategoryDisplayName(TypeCategory category) {
		if (category == null || category.getTypes() == null || category.getTypes().isEmpty())
			return null;
		Iterator<Type> iter = category.getTypes().iterator();
		String list = iter.next().getName();
		while (iter.hasNext()) {
			list = NLS.bind(Messages.AspectEditorUtil_LIST_OF_TYPES, list, iter.next().getName());
		}
		return NLS.bind(Messages.AspectEditorUtil_CATEGORYID_LIST, list, category.getIdentifier());
	}
	
	/**
	 * Can be used to insert some space when using a gridlayout
	 * @param parent the parent composite
	 * @param toolkit the formtoolkit, may be null
	 * @param additionalHeight the desired height additional to the layouts verticalSpacing
	 */
	public static void createFiller(Composite parent, FormToolkit toolkit, int additionalHeight) {
		Composite filler;
		if (toolkit == null) {
			filler= new Composite(parent, SWT.NONE);
		} else {
			filler = toolkit.createComposite(parent, SWT.NONE);
		}
		GridData data= new GridData();
		data.heightHint= additionalHeight;
		filler.setLayoutData(data);
	}
	
	private static String[] checkFileNames(Shell shell, String[] names) {
		List<String> files= new ArrayList<String>();
		List<String> invalides= new ArrayList<String>();
		for (String name: names) {
			int dot= name.lastIndexOf('.');
			if (dot == -1) {
				invalides.add(name);
				continue;
			}
			String suffix= name.substring(dot+1);
			if (validSuffixes.contains(suffix.toLowerCase(Locale.ROOT))) {
				files.add(name);
			} else {
				invalides.add(name);
			}
		}
		if (!invalides.isEmpty()) {
			String fil= ""; //$NON-NLS-1$
			for (String f:invalides) {
				fil =NLS.bind(Messages.AspectEditorUtil_APPEND_FILENAME_TO_LIST, fil, f);
			}
			String message= NLS.bind(Messages.AspectEditorUtil_INVALID_TYPES_MESSAGE, fil);
			MessageDialog.openInformation(shell, Messages.AspectEditorUtil_INVALID_TYPES_TITLE, message);
		}
		return files.toArray(new String[files.size()]);
	}
	
	public static Image getImage(IProcessContainerWorkingCopy workingCopy, String iconUrl, ResourceManager resourceManager, Runnable handler) {
		ImageDescriptor image= getImageDescriptor(workingCopy, iconUrl, handler);
		if (image == null) {
			return null;
		}
		return JazzResources.getImage(resourceManager, image, null);
	}
	
	private static ImageDescriptor getImageDescriptor(IProcessContainerWorkingCopy workingCopy, String iconUrl, Runnable handler) {
		
		/* Process Attachment */
		if (iconUrl.startsWith(ResourceLocation.PROCESS_ATTACHMENT_RESOURCE_SCHEME)) {
			for (IProcessAttachmentHandle handle: workingCopy.getProcessAttachments().getAttachments()) {
				if (extractPath(iconUrl).equals(workingCopy.getProcessAttachments().getPath(handle))) {
					File localFile= workingCopy.getProcessAttachments().getLocalFile(handle);
					if (localFile != null) {
						return ImageDescriptor.createFromFile(null, localFile.getAbsolutePath());
					}
					break;
				}
			}
		}
		
		/* Local File */
		else if (iconUrl.startsWith(ResourceLocation.FILE_RESOURCE_SCHEME)) {
			try {
				return ImageDescriptor.createFromURL(new URL(iconUrl));
			} catch (MalformedURLException e1) {
				// ignore
			}
		}

		/* Remote Image */
		try {
			ITeamRepository repository= (ITeamRepository)workingCopy.getUnderlyingProcessItem().getOrigin();
			URL url= ResourceLocation.getResourceURL((IProcessContainerHandle)workingCopy.getUnderlyingProcessItem(), new URI(iconUrl), repository.getRepositoryURI());
			if (ResourceLocation.isImageContentServiceURL(url)) {
				URL cached= RemoteImages.getCachedImageContentServiceURL(url.toURI());
				if (cached == null) {
					new IconResolver(repository, url, handler).schedule();
					return ImagePool.EMPTY_ICON;
				}
				return WorkItemUI.getImageDescriptor(cached);
			}
			return WorkItemUI.getImageDescriptor(url);
		} catch (Exception e) {
			// ignore
		}
		
		return null;
	}
	
	private static String extractPath(String relativeIconUrl) {
		int index= relativeIconUrl.indexOf("://")+2; //$NON-NLS-1$
		if (index == 1) {
			index= relativeIconUrl.indexOf(":/")+1; //$NON-NLS-1$
		}
		if (index > 0) {
			return relativeIconUrl.substring(index);
		}
		return relativeIconUrl;
	}
	
	public static void addProcessAttachement(final Shell shell, final IProcessAttachmentsWorkingCopy processAttachmentWorkingCopy, String iconPrefix) {
		FileDialog dialog = new FileDialog(shell, SWT.MULTI);
		dialog.setText(Messages.AspectEditorUtil_CHOOSE_FILE);
		dialog.setFilterExtensions(new String[] { "*.bmp;*.gif;*.jpg;*.png", "*"}); //$NON-NLS-1$ //$NON-NLS-2$
		final String filePath = dialog.open();
		if (filePath != null) {
			final String prefix;
			if (!iconPrefix.startsWith("/")) { //$NON-NLS-1$
				prefix= "/" + iconPrefix; //$NON-NLS-1$
			} else {
				prefix= iconPrefix;
			}
			if (dialog.getFileNames().length > 0) {
				final String path= dialog.getFilterPath();
				final String[] fileNames= checkFileNames(shell, dialog.getFileNames());
				PathChecker checker= new PathChecker(processAttachmentWorkingCopy);
				ArrayList resizedImages = new ArrayList();
				for (final String f: fileNames) {
					String name= prefix + "/" + f; //$NON-NLS-1$
					name= name.replace(' ', '_');
					boolean canceled= false;
					while (!canceled && !checker.check(name)) {
						final Object[] values= new Object[]{name, new Boolean(canceled)};
						shell.getDisplay().syncExec(new Runnable() {
							public void run() {
								InputDialog inputDialog = new InputDialog(shell, Messages.AspectEditorUtil_ATTACHMENT_EXISTS_TITLE, NLS.bind(Messages.AspectEditorUtil_ATTACHMENT_EXISTS_MESSAGE, f), new Path(f).lastSegment(), new IInputValidator() {
									public String isValid(String newText) {
										if (newText == null || newText.trim().length() == 0) {
											return Messages.AspectEditorUtil_SPECIFY_PATH;
										}
										IStatus valid= IItemValidator.INSTANCE.validateAttribute(IProcessAttachment.ITEM_TYPE, IProcessAttachment.PATH_PROPERTY_ID , newText);
										if (valid.getSeverity() != IStatus.OK) {
											if (valid instanceof IStringSizeConstraintErrorStatus) {
												return NLS.bind(Messages.AspectEditorUtil_INVALID_PATH, valid.getMessage());
											}
										}
										return null;
									}
								});
								if (inputDialog.open() == Window.OK) {
									String fName = inputDialog.getValue();
									values[0]= prefix + "/" + fName; //$NON-NLS-1$
								} else {
									values[1]= Boolean.TRUE;
								}
							}
						});
						name= (String)values[0];
						canceled= ((Boolean)values[1]).booleanValue();
					}
					if (canceled) {
						continue;
					}
					
					File file = new File(path + "/" + f); //$NON-NLS-1$
					if (!file.exists()) {
						MessageBox mb = new MessageBox(shell, SWT.OK);
			            mb.setText(Messages.AspectEditorUtil_FILE_DNE_TITLE);
			            mb.setMessage(NLS.bind(Messages.AspectEditorUtil_FILE_DNE, file));
			            mb.open();
			            continue;
					}
					
					Image img = new Image(shell.getDisplay(), path + "/" + f); //$NON-NLS-1$
					if (img.getBounds().height>16 || img.getBounds().width>16){
						resizedImages.add(name);
			            
			            try {
			    			img = new Image(Display.getDefault(), scale(img.getImageData()));
			    			file = java.io.File.createTempFile(f, ".png"); //$NON-NLS-1$
							ImageLoader imageLoader = new ImageLoader();
							imageLoader.data = new ImageData[] {img.getImageData()};
							imageLoader.save(file.getPath(), SWT.IMAGE_PNG);
							processAttachmentWorkingCopy.createAttachment(name, file);
			    		} catch (IOException e) {
							e.printStackTrace();
						} finally {
			    			if(file != null){
			    				file.deleteOnExit();
			    			}
			    			if(img != null){
			    				img.dispose();
			    			}
			    		}
					}
					else {
						img.dispose();
						processAttachmentWorkingCopy.createAttachment(name, file);
					}

				}
				
				int resizedImagesSize = resizedImages.size();
				if(resizedImagesSize>0){
					MessageBox mb = new MessageBox(shell, SWT.OK);
		            mb.setText(Messages.AspectEditorUtil_ICON_TOO_LARGE_TITLE);
		            String msg = Messages.AspectEditorUtil_ICON_TOO_LARGE_PLURAL;
		            if (resizedImagesSize==1){
		            	msg = Messages.AspectEditorUtil_ICON_TOO_LARGE_SINGULAR;
		            }
		            for (int i=0; i<resizedImagesSize; i++){
		            	msg+= resizedImages.get(i)+"\n"; //$NON-NLS-1$
		            }
		            mb.setMessage(msg);
		            mb.open();
				}
			}
		}
	}
	
		public static ImageData scale(ImageData imageData) {
		
		if (imageData == null) {
			return null;
		}


		int PHOTO_FRAME_HEIGHT = 16;
		int PHOTO_FRAME_WIDTH = 16;
			
		try {
			imageData = imageData.scaledTo(PHOTO_FRAME_WIDTH, PHOTO_FRAME_HEIGHT);
		} catch (ArrayIndexOutOfBoundsException e) {
			throw new SWTException(SWT.ERROR_INVALID_IMAGE, NLS.bind("Messages.ImageUtilities_0", PHOTO_FRAME_WIDTH, PHOTO_FRAME_HEIGHT)); //$NON-NLS-1$
		}
		
		return imageData;
	}
	public static boolean isValidPathSegment(String string) {
		return string != null && string.length() > 0 && !string.contains(":") && !string.contains("/") && !string.contains("\\"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	public static boolean isValidLength(String string, int length) {
		if (string == null) {
			return true;
		}
		return string.length() <= length;
	}
	
	public static String safeGetLinkText(IReference ref, LinkLabelProvider labelProvider) {
		String text= null;
		if (labelProvider != null) {
			text= labelProvider.getText(ref);
		}
		if (text == null || "".equals(text.trim())) { //$NON-NLS-1$
			text= ref.getComment();
		}
		return XMLString.createFromPlainText(text).getXMLText();
	}
	
	private static class PathChecker {
		private final Set<String> existingPaths= new HashSet<String>();
		PathChecker(IProcessAttachmentsWorkingCopy workingCopy) {
			for (IProcessAttachmentHandle handle: workingCopy.getAttachments()) {
				existingPaths.add(workingCopy.getPath(handle));
			}
		}
		/**
		 * If the path does not yet exist, true is returned and the
		 * parameter added to the list of existing paths
		 * Thus: calling this method twice with the same parameter
		 * will always return false on the second call!
		 */
		public boolean check(String path) {
			if (existingPaths.contains(path)) {
				return false;
			}
			existingPaths.add(path);
			return true;
		}
	}
	
	public static class WorkflowComparator implements Comparator<WFWorkflow> {
		public int compare(WFWorkflow o1, WFWorkflow o2) {
			String name1= o1.getName();
			if (name1 == null || "".equals(name1.trim())) { //$NON-NLS-1$
				name1= o1.getIdentifier();
			}
			String name2= o2.getName();
			if (name2 == null || "".equals(name2.trim())) { //$NON-NLS-1$
				name2= o2.getIdentifier();
			}
			return Collator.getInstance().compare(name1, name2);
		}
	}
	
	public static class ProcessAttachmentSorter extends ViewerSorter {
		private static Collator collator= Collator.getInstance();
		static{
			collator.setStrength(Collator.SECONDARY);
		}
		@Override
		public int compare(Viewer viewer, Object e1, Object e2) {
			if (e1 instanceof IProcessAttachment && e2 instanceof IProcessAttachment) {
				String name1= ((IProcessAttachment)e1).getPath();
				if (name1.lastIndexOf("/") > 0) { //$NON-NLS-1$
					name1= name1.substring(name1.lastIndexOf("/")+1) + " "; //$NON-NLS-1$ //$NON-NLS-2$
				}
				String name2= ((IProcessAttachment)e2).getPath();
				if (name2.lastIndexOf("/") > 0) { //$NON-NLS-1$
					name2= name2.substring(name2.lastIndexOf("/")+1) + " "; //$NON-NLS-1$ //$NON-NLS-2$
				}
				return collator.compare(name1, name2);
			}
			return 0;
		}
	}
	
	public static class IconLabelProvider extends LabelProvider implements ITableLabelProvider {
		private final int fLabelColumn;
		private final int fImageColumn;
		private final IPrefixProvider fPrefixProvider;
		private final ResourceManager fResourceManager;
		
		public IconLabelProvider(IPrefixProvider prefixProvider, int labelColumn, int imageColumn, ResourceManager resourceManager) {
			fPrefixProvider= prefixProvider;
			fLabelColumn= labelColumn;
			fImageColumn= imageColumn;
			fResourceManager= resourceManager;
		}
		public String getColumnText(Object element, int columnIndex) {
			if (element instanceof IProcessAttachmentHandle ) {
				IProcessAttachmentHandle file= (IProcessAttachmentHandle) element;
				if (columnIndex == fLabelColumn) {
					String path= fPrefixProvider.getProcessContainer().getProcessAttachments().getPath(file);
					if (path.lastIndexOf("/") > 0) { //$NON-NLS-1$
						path= path.substring(path.lastIndexOf("/")+1) + " "; //$NON-NLS-1$ //$NON-NLS-2$
					}
					return path;
				}
			}
			return null;
		}

		public Image getColumnImage(Object element, int columnIndex) {
			if (element instanceof IProcessAttachmentHandle ) {
				IProcessAttachmentHandle file= (IProcessAttachmentHandle) element;
				if (columnIndex == fImageColumn) {
					try {
						return getImage(element, ResourceLocation.createRelativeProcessAttachmentURI(fPrefixProvider.getProcessContainer().getProcessAttachments().getPath(file)).toString());
					} catch (URISyntaxException e) {
						//ignore
					}
				}
			}
			return null;
		}
		
		private Image getImage(final Object element, String iconPath) {
			return AspectEditorUtil.getImage(fPrefixProvider.getProcessContainer(), iconPath, fResourceManager, new Runnable() {
				public void run() {
					fireChanged(element);
				}
			});
		}
		
		private void fireChanged(Object element) {
			fireLabelProviderChanged(new LabelProviderChangedEvent(this, element));
		}
	}

	public static String compatible(String prefix, String state) {
		if (state != null && state.startsWith(prefix)) {
			try {
				Integer.parseInt(state.substring(prefix.length()));
				return state.substring(prefix.length());
			} catch (NumberFormatException e) {
				// qualified id
			}
		}
		return state;
	}
	
	private static List<IEndPointDescriptor> fEndPoints;
	public static List<IEndPointDescriptor> getEndPoints() {
		
		if (fEndPoints == null) {
			ArrayList<IEndPointDescriptor> endpoints= new ArrayList<IEndPointDescriptor>();
			for (ILinkType type: (Collection<ILinkType>)ILinkTypeRegistry.INSTANCE.allEntries()) {
				if (type.isInternal()) {
					continue;
				}
				
				if (WorkItemLinkTypes.isSymmetric(type.getSourceEndPointDescriptor()!= null?type.getSourceEndPointDescriptor():type.getTargetEndPointDescriptor())) {
					if ((type.getSourceEndPointDescriptor() != null && type.getSourceEndPointDescriptor().getReferencedItemType() == IWorkItem.ITEM_TYPE)
							||(type.getTargetEndPointDescriptor() != null && type.getTargetEndPointDescriptor().getReferencedItemType() == IWorkItem.ITEM_TYPE)) {
						endpoints.add(type.getTargetEndPointDescriptor()!= null?type.getTargetEndPointDescriptor():type.getSourceEndPointDescriptor());
					}
					continue;
				}
				if (type.getSourceEndPointDescriptor() != null) {
					if (type.getTargetEndPointDescriptor() != null && type.getTargetEndPointDescriptor().getReferencedItemType() ==  IWorkItem.ITEM_TYPE) {
						endpoints.add(type.getSourceEndPointDescriptor());
					}
				}
				if (type.getTargetEndPointDescriptor() != null) {
					if (type.getSourceEndPointDescriptor() != null && type.getSourceEndPointDescriptor().getReferencedItemType() ==  IWorkItem.ITEM_TYPE) {
						endpoints.add(type.getTargetEndPointDescriptor());
					}
				}
			}
			Collections.sort(endpoints, new Comparator<IEndPointDescriptor>() {
				public int compare(IEndPointDescriptor o1, IEndPointDescriptor o2) {
					if (o1 == null) {
						return 1;
					}
					if (o2 == null) {
						return -1;
					}
					return Collator.getInstance().compare(o1.getLinkType().getLinkTypeId(), o2.getLinkType().getLinkTypeId());
				}
				
			});
			fEndPoints= endpoints;
		}
		return fEndPoints;

	}
	
	private static Map<String, String> fXMLStringsToDisplayNames;
	
	/**
	 * @return
	 * 	cached displaynames of rend points of references by their XML strings.
	 */
	// XXX: jgillibr-I'm not sure why it's OK to cache these--it seems like the display
	// names can change as the the user modifies the configuration, but this works fine when
	// I tested it, and other data are being cached on this class already, seemingly without
	// a problem.
	public static Map<String, String> getEndPointXMLStringsToDisplayNames()
	{
		if (fXMLStringsToDisplayNames != null) return fXMLStringsToDisplayNames;
		
		fXMLStringsToDisplayNames = new HashMap<String, String>();
		
		final String nullValue= Messages.AspectEditorUtil_UNCONFIGURED_LINK_TYPE;
		fXMLStringsToDisplayNames.put(nullValue, nullValue);
		
		List<IEndPointDescriptor> endPoints = AspectEditorUtil.getEndPoints();
		
		for (IEndPointDescriptor desc: endPoints) {
			String xmlString = AspectEditorUtil.getXMLString(desc);
			fXMLStringsToDisplayNames.put(xmlString, desc.getDisplayName());
		}
		
		return fXMLStringsToDisplayNames;
	}
	
	public static IEndPointDescriptor getFromXMLString(String xml) {
		if (xml == null || !xml.contains("/")) { //$NON-NLS-1$
			return null;
		}
		String[] split= xml.split("/", 2); //$NON-NLS-1$
		String linkType= split[0];
		String endpoint;
		if (split.length == 2) {
			endpoint= split[1];
		} else {
			endpoint= ""; //$NON-NLS-1$
		}
		for (ILinkType type: (Collection<ILinkType>)ILinkTypeRegistry.INSTANCE.allEntries()) {
			if (type.getLinkTypeId().equals(linkType)) {
				if ("source".equals(endpoint)) { //$NON-NLS-1$
					return type.getSourceEndPointDescriptor();
				} else if ("target".equals(endpoint)) { //$NON-NLS-1$
					return type.getTargetEndPointDescriptor();
				}
				if (endpoint.equals("")) { //$NON-NLS-1$
					if (!WorkItemLinkTypes.isSymmetric(type)) {
						return null;
					}
					return type.getTargetEndPointDescriptor() != null? type.getTargetEndPointDescriptor():type.getSourceEndPointDescriptor();
				}
				if (type.getTargetEndPointDescriptor() != null && endpoint.equals(type.getTargetEndPointDescriptor().getId())) {
					return type.getTargetEndPointDescriptor();
				} else if (type.getSourceEndPointDescriptor() != null && endpoint.equals(type.getSourceEndPointDescriptor().getId())) {
					return type.getSourceEndPointDescriptor();
				}
				return null;
			}
		}
		return null;
	}
	
	public static String getXMLString(IEndPointDescriptor desc) {
		String ret= desc.getLinkType().getLinkTypeId() + "/"; //$NON-NLS-1$
		if (WorkItemLinkTypes.isSymmetric(desc)) {
			 return ret;
		}
		if (desc.getId() != null) {
			 return ret + desc.getId();
		}
		if (desc.isSource()) {
			return ret + "source"; //$NON-NLS-1$
		}
		return ret + "target"; //$NON-NLS-1$
	}
	
	public static Set<String> getPresentationPartIds(ModelElement editorPresentationConfiguration) {
		Set<String> partIds= new HashSet<String>();
		if (editorPresentationConfiguration != null) {
			for (ModelElement wf: (List<ModelElement>)editorPresentationConfiguration.getChildElements()) {
				if (wf.getName().equals(EditorPresentationManager.SECTION)) {
					for (ModelElement s: (List<ModelElement>)wf.getChildElements()) {
						if (s.getName().equals(EditorPresentationManager.PRESENTATION)) {
							String id= s.getAttribute(EditorPresentationManager.ID);
							if (id == null) {
								id= s.getAttribute(EditorPresentationManager.ATTRIBUTE_ID);
							}
							if (id == null) {
								id= s.getAttribute(EditorPresentationManager.KIND);
							}
							if (id == null) {
								continue;
							}
							partIds.add(id);
						}
					}
				}
				
			}
		}
		return partIds;
	}
	
}
