/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.process.client.workingcopies.IProcessAreaWorkingCopy;
import com.ibm.team.process.client.workingcopies.IProcessContainerWorkingCopy;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProjectArea;
import com.ibm.team.process.ide.ui.AbstractDynamicActionProvider;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.client.IEnumerationClient;
import com.ibm.team.workitem.common.internal.EnumerationsHelper;
import com.ibm.team.workitem.common.internal.enumeration.Enumeration;

import java.util.ArrayList;
import java.util.List;

/**
 * Computes the list of persisted enumerations to allow for permission changes
 * with regards to saving new literals per enumeration.
 */
public class EnumerationsDynamicActionProvider extends AbstractDynamicActionProvider {

	/*
	 * @see com.ibm.team.process.ide.ui.AbstractDynamicActionProvider#computeActions(org.eclipse.core.runtime.IPath, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public DynamicAction[] computeActions(IPath actionPath, IProgressMonitor monitor) throws TeamRepositoryException {
		List<DynamicAction> actions= new ArrayList<DynamicAction>();

		if (EnumerationsHelper.CREATE_LITERAL_ACTION_ID.equals(actionPath.toString())) {
			Enumeration[] enumerations= findPersistedEnumerations(monitor);
			for (Enumeration enumeration : enumerations) {
				DynamicAction action= new DynamicAction(enumeration.getIdentifier(), NLS.bind(Messages.EnumerationsDynamicActionProvider_CREATE_ENUM_VALUE, enumeration.getName()), null);
				actions.add(action);
			}
		}

		return (DynamicAction[]) actions.toArray(new DynamicAction[actions.size()]);
	}

	private Enumeration[] findPersistedEnumerations(IProgressMonitor monitor) throws TeamRepositoryException {
		IProcessContainerWorkingCopy container= getProcessContainer();
		if (container instanceof IProcessAreaWorkingCopy) {
			IProcessArea processArea= ((IProcessAreaWorkingCopy) container).getUnderlyingProcessArea();
			if (processArea != null && processArea instanceof IProjectArea) {
				IProjectArea projectArea= (IProjectArea) processArea;

				ITeamRepository teamRepository= getTeamRepository();
				IEnumerationClient enumClient= (IEnumerationClient) teamRepository.getClientLibrary(IEnumerationClient.class);

				return enumClient.resolveEnumerations(projectArea, monitor);
			}
		}
		return new Enumeration[0];
	}
}