/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.util.List;

import com.ibm.team.workitem.ide.ui.internal.aspecteditor.enumeration.UIEnumeration;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.quickinformation.Quickinformation;

/**
 * Represents a class that contains a list of items with IDs that can be
 * reordered and moved around. The intent of this interface was to be used along
 * with the {@link TableViewerReorderingSupport} class to provide drag and
 * drop and keyboard shortcuts to reorder the elements within a TableViewer. The
 * elements in TableViewer as far as the classes I've edited were all backed by
 * another class that contained a list of items and that list of items
 * represents the data in the list. A few examples of such a class are
 * {@link UIEnumeration} and {@link Quickinformation}.
 * 
 * @author jsteele
 */
public interface IReorderableList {

	/**
	 * Allows an item that may or may not be in the list to be inserted or moved
	 * to a new location within the list. The item is first removed from the
	 * list and then added back, if it does not exist then nothing will be
	 * removed and the item will just be added.
	 * 
	 * @param location
	 *            The location in the list the item should be moved to.
	 * @param object
	 *            The object that should be add/moved within the list.
	 */
	public void moveTo(int location, Object object);
		
	
	/**
	 * Finds the index of an item within the list or returns -1 if the item was
	 * not found in the list.
	 * 
	 * @param object
	 *            The object to find the location of.
	 * @return The objects location or -1 if not found.
	 */
	public int indexOf(Object object);
	
	
	/**
	 * Gets an unmodifiable list of the current objects held within the list.
	 * 
	 * @return An unmodifiable list of the current objects in the list.
	 */
	public List<Object> getCurrentObjects();
	
	
	/**
	 * Gets the unique ID of an object contained within the list. The users of
	 * this interface are dealing with only objects and don't know about the
	 * implementation of the objects, so it is up to the implementors of the
	 * interface to ensure that their backing objects have IDs.
	 * 
	 * @param object
	 *            The object to find the ID of.
	 * @return The ID of the object.
	 */
	public String getObjectsId(Object object);
}
