/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import com.ibm.team.jface.util.UIUpdaterJob;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;

import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.client.internal.WorkItemClient;
import com.ibm.team.workitem.client.internal.util.RemoteImages;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;

public class IconResolver extends UIUpdaterJob {

	private final ITeamRepository fRepository;
	private final URL fRemoteURL;
	private final Runnable fCallback;

	public IconResolver(ITeamRepository repository, URL remoteURL, Runnable callback) {
		super(Messages.IconResolver_RESOLVING_ICONS);
		Assert.isNotNull(repository);
		Assert.isNotNull(remoteURL);
		fRemoteURL= remoteURL;
		fRepository=repository;
		fCallback= callback;
	}
	
	@Override
	public IStatus runInBackground(IProgressMonitor monitor) {
			WorkItemClient client= (WorkItemClient) fRepository.getClientLibrary(IWorkItemClient.class);
			try {
				client.resolveURL(fRemoteURL, monitor);
			} catch (TeamRepositoryException e) {
				WorkItemIDEUIPlugin.getDefault().log(NLS.bind("Unable to resolve icon: {0}", fRemoteURL.toExternalForm()), e); //$NON-NLS-1$
			}
		return super.runInBackground(monitor);
	}
	
	@Override
	public IStatus runInUI(IProgressMonitor monitor) {
		if (fCallback != null && isLocallyAvailable())
			fCallback.run();
		return super.runInUI(monitor);
	}
	
	private boolean isLocallyAvailable() {
		try {
			return RemoteImages.getCachedImageContentServiceURL(fRemoteURL.toURI()) != null;
		} catch (MalformedURLException e) {
		} catch (URISyntaxException e) {
		}
		return false;
	}
	
}