/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.accessibility.ACC;
import org.eclipse.swt.accessibility.AccessibleAdapter;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchActionConstants;

import com.ibm.team.foundation.rcp.ui.util.FoundationUIJob;
import com.ibm.team.process.client.workingcopies.IProcessAttachmentsWorkingCopy;
import com.ibm.team.process.client.workingcopies.IWorkingCopyListener;
import com.ibm.team.process.client.workingcopies.WorkingCopyChangeEvent;
import com.ibm.team.process.common.IProcessAttachment;
import com.ibm.team.process.common.IProcessAttachmentHandle;
import com.ibm.team.process.common.IProcessItemHandle;
import com.ibm.team.process.internal.ide.ui.ProcessIdeUIPlugin;
import com.ibm.team.repository.client.IContentManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.common.internal.ResourceLocation;
import com.ibm.team.workitem.common.internal.util.ItemHandleAwareHashSet;
import com.ibm.team.workitem.ide.ui.internal.WITeamFormPart;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AbstractWorkItemAspectEditor.IAspectMessageProvider;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil.IconLabelProvider;

public class ProcessAttachmentIconSelectionDialog extends ValidationDialog {
		private Label fIcon;
		private TableViewer fIconViewer;
		private ItemHandleAwareHashSet<IProcessAttachmentHandle> fFiles;
		private final IPrefixProvider fPrefixProvider;
		ResourceManager fResourceManager;
		private String fIconUrl;
		private final String fTitle;
		private final IWorkingCopyListener fIconListener= new MinimalisticIconListener();
		
		private static final String NAME_LABEL= Messages.ProcessAttachmentIconSelectionDialog_NAME_LABEL;
		private static final String ICON_LABEL= Messages.ProcessAttachmentIconSelectionDialog_ICON_LABEL;
		
		private static int LOWER_BOUND_X= 350;
		private static int LOWER_BOUND_Y= 150;
		private static int UPPER_BOUND_X= 600;
		private static int UPPER_BOUND_Y= 600;
		private Button fClearIcon;
		
		
		private IProcessAttachmentsWorkingCopy fWorkingCopy;
		private Set<IProcessAttachmentHandle> fIcons= new ItemHandleAwareHashSet<IProcessAttachmentHandle>();
		private Action fAddAttachmentAction;
		private Action fDeleteAttachmentAction;
		private Action fDownloadAttachmentAction;
		private Button fDownloadIconButton;
		private Button fDeleteIconButton;
		private Button fUploadIconButton;
		private List<WITeamFormPart> fTeamFormPartsInSameAspectEditor;
		private Composite fIconPickerArea;
		private Label fIconLabel;
		private Table fIconTable;
		
	protected ProcessAttachmentIconSelectionDialog(Shell parentShell, String title, String iconUrl, IPrefixProvider prefixProvider, ResourceManager resourceManager) {
		super(parentShell, resourceManager);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		fTitle= title;
		fPrefixProvider= prefixProvider;
		fResourceManager= resourceManager;
		fIconUrl= iconUrl;
	}
	
	protected void setIconTableHeight(int iconsToShow) {
		if (fIconTable != null && !fIconTable.isDisposed()) {
			final GridData d= (GridData)fIconTable.getLayoutData();
			d.heightHint= fIconTable.getItemHeight() * iconsToShow;
		}
	}

	/**
	
	 * This constructor can be used when the Icon Selection Dialog is associated with another TeamFormPart like TypesPart.
	 */
	protected ProcessAttachmentIconSelectionDialog(Shell parentShell, String title, String iconUrl, IPrefixProvider prefixProvider, ResourceManager resourceManager, List<WITeamFormPart> associatedTeamFormParts) {
		super(parentShell, resourceManager);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		fTitle= title;
		fPrefixProvider= prefixProvider;
		fResourceManager= resourceManager;
		fIconUrl= iconUrl;
		
		if (associatedTeamFormParts!=null){
			fTeamFormPartsInSameAspectEditor = associatedTeamFormParts;
		}
		else fTeamFormPartsInSameAspectEditor = new ArrayList<WITeamFormPart>();
	}
	
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(fTitle);
	}
	
	@Override
	protected Point getInitialSize() {
		Point initialSize= super.getInitialSize();
		int x= Math.max(initialSize.x+20, LOWER_BOUND_X);
		x= Math.min(x+20, UPPER_BOUND_X);
		int y= Math.max(initialSize.y, LOWER_BOUND_Y);
		y= Math.min(y, UPPER_BOUND_Y);
		return new Point(x, y);
	}
	
	private void createActions(final TableViewer tableViewer) {
		fAddAttachmentAction = new AddAttachmentAction();
		fDeleteAttachmentAction = new DeleteAttachmentAction();
		fDownloadAttachmentAction = new DownloadAttachmentAction();
	}
	
	private class AddAttachmentAction extends Action {

		public AddAttachmentAction() {
			setText(Messages.ProcessAttachmentIconSelectionDialog_ADD_ICONS);
		}

		/*
		 * @see org.eclipse.jface.action.Action#run()
		 */
		public void run() {
			try{
				AspectEditorUtil.addProcessAttachement(fIconViewer.getControl().getShell(), fPrefixProvider.getProcessContainer().getProcessAttachments(), fPrefixProvider.getIconPrefix());
			} catch (SWTException x) {
				switch (x.code) {
				case SWT.ERROR_IO:
				case SWT.ERROR_INVALID_IMAGE:
				case SWT.ERROR_UNSUPPORTED_DEPTH:
				case SWT.ERROR_UNSUPPORTED_FORMAT:
					MessageBox mb = new MessageBox(fIconViewer.getControl().getShell(), SWT.OK);
		            mb.setText(Messages.ProcessAttachmentIconSelectionDialog_ERROR);
		            mb.setMessage(x.getMessage());
		            mb.open();
					break;
				default:
					throw x;
				}
			}
		}
	}
	
	private class DeleteAttachmentAction extends Action {

		public DeleteAttachmentAction() {
			setText(Messages.ProcessAttachmentIconSelectionDialog_REMOVE_ICONS);
		}

		/*
		 * @see org.eclipse.jface.action.Action#run()
		 */
		public void run() {
			ISelection selection= fIconViewer.getSelection();
			if (selection instanceof IStructuredSelection) {
				IStructuredSelection ss= (IStructuredSelection)selection;
				for (Iterator<?> e= ss.iterator(); e.hasNext();) {
					IProcessAttachmentHandle attachmentHandle= (IProcessAttachmentHandle)e.next();
					String usage= ""; //$NON-NLS-1$
					for (int i= 0; i < fTeamFormPartsInSameAspectEditor.size(); i++) {
						WITeamFormPart tfp= fTeamFormPartsInSameAspectEditor.get(i);
						if (tfp != null) {
							List<String> iconUsage= tfp.getIconUsage(fWorkingCopy.getPath(attachmentHandle));
							for (int j= 0; j < iconUsage.size(); j++) {
								usage= usage + iconUsage.get(j);
							}
						}
					}
					if (!usage.equals("")) { //$NON-NLS-1$
						MessageBox mb= new MessageBox(fIconViewer.getControl().getShell(), SWT.YES | SWT.NO);
						mb.setText(Messages.ProcessAttachmentIconSelectionDialog_ICON_IN_USE_TITLE);
						mb.setMessage(NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_ICON_IN_USE_TEXT, usage));
						int response= mb.open();
						if (response != SWT.YES) {
							continue;
						}
					}

					// Clear the selected icon
					fIconUrl= null;
					fIconViewer.setSelection(StructuredSelection.EMPTY);
					setIcon(true);

					fWorkingCopy.deleteAttachment(attachmentHandle);
				}
			}
		}
	}
	
	private class DownloadAttachmentAction extends Action {

		public DownloadAttachmentAction() {
			setText(Messages.ProcessAttachmentIconSelectionDialog_DOWNLOAD_ICONS);
			setToolTipText(Messages.ProcessAttachmentIconSelectionDialog_DOWNLOAD_ICONS_TOOLTIP);
		}

		/*
		 * @see org.eclipse.jface.action.Action#run()
		 */
		public void run() {
			ISelection selection= fIconViewer.getSelection();
			if (selection instanceof IStructuredSelection) {
				IStructuredSelection ss= (IStructuredSelection)selection;
				final IProcessAttachmentHandle attachmentHandle= (IProcessAttachmentHandle)ss.getFirstElement();
				String path= fWorkingCopy.getPath(attachmentHandle);
				Shell shell= fIconViewer.getControl().getShell();
				FileDialog dialog= new FileDialog(shell, SWT.SAVE);
				final Path filePath= new Path(path);
				dialog.setFileName(filePath.lastSegment());
				// Set "Save as type" to *. extension of file name.
				dialog.setFilterExtensions(new String[] { "*." + filePath.getFileExtension() }); //$NON-NLS-1$
				final String localFilePath= dialog.open();
				if (localFilePath != null) {
					final File existing= new File(localFilePath);
					if (existing.exists()) {
						if (!MessageDialog.openConfirm(shell, Messages.ProcessAttachmentIconSelectionDialog_OVERWRITE_DIALOG_TITLE, Messages.ProcessAttachmentIconSelectionDialog)) {
							return;
						}
					}
					Job job= new Job(NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_DOWNLOADING, path)) {
						protected IStatus run(IProgressMonitor monitor) {
							if (existing.exists()) {
								if (!existing.delete()) {
									return new Status(IStatus.ERROR, ProcessIdeUIPlugin.PLUGIN_ID, NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_FAILED_DELETE, localFilePath));
								}
							}
							File localFile= new File(localFilePath);
							try {
								if (localFile.createNewFile()) {
									FileOutputStream stream= new FileOutputStream(localFile);
									ITeamRepository repo= (ITeamRepository)attachmentHandle.getOrigin();
									IContentManager contentManager= repo.contentManager();
									contentManager.retrieveContent(fWorkingCopy.getContent(attachmentHandle), stream, monitor);
								} else {
									return new Status(IStatus.ERROR, ProcessIdeUIPlugin.PLUGIN_ID, NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_FAILED_CREATE, localFilePath));
								}
							} catch (FileNotFoundException e) {
								return new Status(IStatus.ERROR, ProcessIdeUIPlugin.PLUGIN_ID, NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_FAILED_SAVE, localFilePath), e);
							} catch (IOException e) {
								return new Status(IStatus.ERROR, ProcessIdeUIPlugin.PLUGIN_ID, NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_FAILED_SAVE, localFilePath), e);
							} catch (TeamRepositoryException e) {
								return new Status(IStatus.ERROR, ProcessIdeUIPlugin.PLUGIN_ID, NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_FAILED_SAVE, localFilePath), e);
							}
							return Status.OK_STATUS;
						}
					};
					job.setUser(true);
					job.schedule();
				}
			}
		}
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite= (Composite)super.createDialogArea(parent);
		Composite allContent= new Composite(composite, SWT.NONE);
		GridLayout layout= new GridLayout(2, false);
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		layout.verticalSpacing= 3;
		allContent.setLayout(layout);
		allContent.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		createUpperDialogPart(allContent);

		createIconDialogPart(allContent);
		setIconTableHeight(8);

		createErrorStatusDialogPart(composite);

		applyDialogFont(composite);
		return composite;
	}

	/**
	 * Hides the icon picker area if not needed.
	 */
	protected void hideIconPicker() {
		fIconLabel.setVisible(false);
		GridDataFactory.fillDefaults().hint(0, 0).applyTo(fIconLabel);
		
		fIconPickerArea.setVisible(false);
		GridDataFactory.fillDefaults().hint(0, 0).applyTo(fIconPickerArea);
	}

	private void initializeUI() {
		//check whether disposed
		if (fResourceManager == null) {
			return;
		}
		fWorkingCopy= fPrefixProvider.getProcessContainer().getProcessAttachments();
		// we might be already registered
		fWorkingCopy.removeWorkingCopyListener(fIconListener);
		fWorkingCopy.addWorkingCopyListener(fIconListener);
		IProcessAttachmentHandle[] handles= fWorkingCopy.getAttachments();
		fIcons= new ItemHandleAwareHashSet<IProcessAttachmentHandle>();
		for (IProcessAttachmentHandle handle: handles) {
			String path= fWorkingCopy.getPath(handle);
			if (path.startsWith(fPrefixProvider.getIconPrefix())) {
				fIcons.add(handle);
			}
		}
		if (!fIconViewer.getControl().isDisposed()) {
			fIconViewer.setInput(fIcons);
		}
		//fInitialized= true;
	}
	
	private void hookContextMenu(final TableViewer tableViewer) {
	//private void hookContextMenu(ITeamFormPartSite site, final TableViewer tableViewer) {

		MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
				IStructuredSelection selection = (IStructuredSelection) tableViewer.getSelection();
				if (selection.size() == 0) {
					manager.add(fAddAttachmentAction);
				} else if (selection.size() == 1) {
					manager.add(fAddAttachmentAction);
					fDeleteAttachmentAction.setEnabled(allInSelectionCanBeDeleted(selection));
					manager.add(fDeleteAttachmentAction);
					
					boolean enabled= true;
					if (selection.getFirstElement() instanceof IProcessAttachment) {
						enabled= ((IProcessAttachment)selection.getFirstElement()).getContent() != null;
					}
					
					fDownloadAttachmentAction.setEnabled(enabled);
					
					manager.add(fDownloadAttachmentAction);
				} else {
					fDeleteAttachmentAction.setEnabled(allInSelectionCanBeDeleted(selection));
					manager.add(fDeleteAttachmentAction);
				}
			}
		});
		Menu menu= menuMgr.createContextMenu(tableViewer.getControl());
		tableViewer.getControl().setMenu(menu);
		//site.registerContextMenu(site.getId(null), menuMgr, fIconViewer);
	}
	
	/**
	 * The parent has a GridLayout with two columns
	 * The first one should be used to show a label, the
	 * second one to provide an input field.
	 */
	protected void createUpperDialogPart(Composite parent) {
	}
	
	protected void createIconDialogPart(Composite parent) {
		
		fIconLabel= new Label(parent, SWT.NONE);
		fIconLabel.setText(Messages.ProcessAttachmentIconSelectionDialog_ICON);
		fIconLabel.setLayoutData(new GridData());
		
		fIconPickerArea = new Composite(parent, SWT.NONE);
		GridLayout layout= new GridLayout(3, false);
		layout.marginWidth= 0;
		fIconPickerArea.setLayout(layout);
		fIconPickerArea.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		
		fIcon= new Label(fIconPickerArea, SWT.BORDER);
		GridData data = new GridData(SWT.LEFT, SWT.CENTER, false, false);
		data.widthHint= 18;
		fIcon.setLayoutData(data);
		fIcon.setBackground(fIcon.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		setIcon(false);
		fClearIcon = new Button(fIconPickerArea, SWT.FLAT);
		fClearIcon.setText(Messages.ProcessAttachmentIconSelectionDialog_CLEAR_ICON);
		fClearIcon.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fIconUrl= null;
				fIconViewer.setSelection(StructuredSelection.EMPTY);
				setIcon(true);
			}
		});
		fClearIcon.setLayoutData(new GridData());
		fClearIcon.setEnabled(fIconUrl != null);
		
		Composite viewer= new Composite(parent, SWT.NONE);
		viewer.setLayout(GridLayoutFactory.fillDefaults().numColumns(2).create());
		data = new GridData(SWT.FILL, SWT.FILL, true, true, 2 , 3);
		viewer.setLayoutData(data);
		createIconViewer(viewer);

		fPrefixProvider.getProcessContainer().getProcessAttachments().addWorkingCopyListener(fIconListener);
		
		Composite iconButtons= new Composite(viewer, SWT.NONE);
		iconButtons.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, false, false));
		iconButtons.setLayout(GridLayoutFactory.fillDefaults().create());
		
		fUploadIconButton= new Button(iconButtons, SWT.FLAT);
		fUploadIconButton.setText(fAddAttachmentAction.getText());
		fUploadIconButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fAddAttachmentAction.run();
			}
		});
		fUploadIconButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		fUploadIconButton.setEnabled(true);
		
		fDeleteIconButton= new Button(iconButtons, SWT.FLAT);
		fDeleteIconButton.setText(fDeleteAttachmentAction.getText());
		fDeleteIconButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fDeleteAttachmentAction.run();
			}
		});
		fDeleteIconButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		fDeleteIconButton.setEnabled(false);
		
		fDownloadIconButton= new Button(iconButtons, SWT.FLAT);
		fDownloadIconButton.setText(fDownloadAttachmentAction.getText());
		fDownloadIconButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fDownloadAttachmentAction.run();
			}
		});
		fDownloadIconButton.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		fDownloadIconButton.setEnabled(false);
		fDownloadIconButton.setToolTipText(fDownloadAttachmentAction.getToolTipText());
		
		initializeUI();
		hookContextMenu(fIconViewer);
	}
	
	@Override
	protected Control createButtonBar(Composite parent) {
		Control c= super.createButtonBar(parent);
			if (isValid().getSeverity() == IStatus.ERROR) {
				Button button = getButton(IDialogConstants.OK_ID);
				if (button != null && !button.isDisposed()) {
					button.setEnabled(false);
				}
		}
		return c;
	}
	
	private void setIcon(boolean notifyClient) {
		if (fIconUrl != null) {
			try {
				fIcon.setImage(AspectEditorUtil.getImage(fPrefixProvider.getProcessContainer(), fIconUrl, fResourceManager, new Runnable() {
					public void run() {
						if (fIcon != null && !fIcon.isDisposed()) {
							fIcon.setImage(AspectEditorUtil.getImage(fPrefixProvider.getProcessContainer(), fIconUrl, fResourceManager, null));
						}
					}
				}));
			} catch (Exception e) {
				// ignore
			}
		} else {
			fIcon.setImage(null);
		}
		if (fClearIcon != null && !fClearIcon.isDisposed()) {
			fClearIcon.setEnabled(fIconUrl != null);
		}
		if (notifyClient) {
			iconChanged();
		}
	}
	
	protected void iconChanged(){
		// intentionally left empty
	}
	
	/**
	 * Return the url as string from the selected icon, or null if none was selected.
	 * The url is in sync with the UI and can thus be used by validators to verify an icon is selected.
	 */
	public String getIconUrl() {
		return fIconUrl;
	}
	
	@Override
	public boolean close() {
		if (fPrefixProvider != null) {
			fPrefixProvider.getProcessContainer().getProcessAttachments().removeWorkingCopyListener(fIconListener);
		}
		return super.close();
	}
	
	private void createIconViewer(Composite parent) {
		fIconTable= new Table(parent, SWT.BORDER | SWT.FULL_SELECTION);
		GridData data= new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		data.heightHint= fIconTable.getItemHeight()*6;
		fIconTable.setLayoutData(data);
		TableColumn c= new TableColumn(fIconTable, SWT.NONE);
		c.setWidth(25);
		c.setResizable(true);
		c= new TableColumn(fIconTable, SWT.NONE);
		c.setWidth(150);
		c.setResizable(true);
		c.setText(NAME_LABEL);

		fIconViewer= new TableViewer(fIconTable);
		fIconViewer.setColumnProperties(new String[]{ICON_LABEL, NAME_LABEL});
		fIconViewer.setLabelProvider(new IconLabelProvider(fPrefixProvider, 1, 0, fResourceManager));
		fIconViewer.setContentProvider(new IStructuredContentProvider() {
			public Object[] getElements(Object inputElement) {
				return fFiles.toArray();
			}

			public void dispose() {
			}

			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		});
		fIconViewer.setSorter(new AspectEditorUtil.ProcessAttachmentSorter());
		fIconViewer.getTable().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IProcessAttachmentHandle img= ((IProcessAttachmentHandle)((IStructuredSelection)fIconViewer.getSelection()).getFirstElement());
				if (img != null) {
					try {
						fIconUrl= ResourceLocation.createRelativeProcessAttachmentURI(fPrefixProvider.getProcessContainer().getProcessAttachments().getPath(img)).toString();
					} catch (URISyntaxException e1) {
						fIconUrl= null;
					}
				} else {
					fIconUrl= null;
				}
				setIcon(true);
			}
		});
		populateIconViewer();
		if (fIconUrl != null) {
			try {
				for (IProcessAttachmentHandle file: fFiles) {
					if (fIconUrl.equals(ResourceLocation.createRelativeProcessAttachmentURI(fPrefixProvider.getProcessContainer().getProcessAttachments().getPath(file)).toString())) {
						fIconViewer.setSelection(new StructuredSelection(file));
					}
				}
			} catch (URISyntaxException e3) {
			}
		}
		
		fIconViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection ss= (IStructuredSelection)event.getSelection();
				if (ss.isEmpty()) {
					fDeleteIconButton.setEnabled(false);
					fDownloadIconButton.setEnabled(false);
				} else if (ss.size() == 1) {
					fDeleteIconButton.setEnabled(allInSelectionCanBeDeleted(ss));
					
					boolean enabled = true;
					if (ss.getFirstElement() instanceof IProcessAttachment) {
						enabled = ((IProcessAttachment) ss.getFirstElement()).getContent() != null;
					}
					fDownloadIconButton.setEnabled(enabled);
				} else if (ss.size() > 1) {
					fDeleteIconButton.setEnabled(allInSelectionCanBeDeleted(ss));
					fDownloadIconButton.setEnabled(false);
				}
			}
		});

		fIconTable.getAccessible().addAccessibleListener(new AccessibleAdapter(){
			@Override
			public void getName(AccessibleEvent e) {
				if(e.childID == ACC.CHILDID_SELF){
					e.result= Messages.ProcessAttachmentIconSelectionDialog_ICONS;
				}
			}
			
		});
		createActions(fIconViewer);
	}
	
	private boolean allInSelectionCanBeDeleted(IStructuredSelection selection) {
		if (selection.size() == 0) {
			return false;
		}
		for (Iterator<?> si= selection.iterator(); si.hasNext();) {
			Object selected= si.next();
			if (!(selected instanceof IProcessAttachmentHandle) || !fWorkingCopy.isModifiable((IProcessAttachmentHandle)selected)) {
				return false;
			}
		}
		return true;
	}
	
	private void populateIconViewer() {
		IProcessAttachmentsWorkingCopy workingCopy= fPrefixProvider.getProcessContainer().getProcessAttachments();
		IProcessAttachmentHandle[] handles= workingCopy.getAttachments();
		fFiles= new ItemHandleAwareHashSet<IProcessAttachmentHandle>();
		for (IProcessAttachmentHandle handle: handles) {
			String path= workingCopy.getPath(handle);
			if (path.startsWith(fPrefixProvider.getIconPrefix())) {
				fFiles.add(handle);
			}
		}
		if (!fIconViewer.getControl().isDisposed()) {
			fIconViewer.setInput(fFiles);
		}
	}
	
	private final class MinimalisticIconListener implements IWorkingCopyListener {
		private final FoundationUIJob fPopulateJob= new FoundationUIJob(Messages.ProcessAttachmentIconSelectionDialog_REFRESH_ICONS) {
			@Override
			protected IStatus runProtectedInUI(IProgressMonitor monitor) throws Exception {
				populateIconViewer();
				return Status.OK_STATUS;
			}
		};
		private void scheduleJob() {
			if (!fPopulateJob.isSystem()) {
				fPopulateJob.setSystem(true);
			}
			fPopulateJob.schedule(500);
		}
		public void workingCopyChanged(WorkingCopyChangeEvent event) {
			if (fPrefixProvider.getProcessContainer().getProcessAttachments() != event.getSource()) {
				return;
			}
			String property = event.getProperty();
			if (property == null) {
				return;
			}
			
			if (property.startsWith(IProcessAttachmentsWorkingCopy.EVENT_PROPERTY_PREFIX)) {
				if (IProcessAttachmentsWorkingCopy.CHANGE_EVENT_PROPERTY.equals(property)) {
					scheduleJob();
					return;
				}
				if (IProcessAttachmentsWorkingCopy.UPDATE_EVENT_PROPERTY.equals(property)) {
					IProcessItemHandle[] newValues = (IProcessItemHandle[]) event.getNewValue();
					boolean affected= false;
					for (IProcessItemHandle newIcon: newValues) {
						if (fPrefixProvider.getProcessContainer().getProcessAttachments().getPath((IProcessAttachmentHandle)newIcon).startsWith(fPrefixProvider.getIconPrefix())) {
							affected= true;
							break;
						}
					}
					if (affected) {
						scheduleJob();
					}
					return;
				}
				
				if (IProcessAttachmentsWorkingCopy.ADD_EVENT_PROPERTY.equals(property)) {
					IProcessItemHandle[] newValues = (IProcessItemHandle[]) event.getNewValue();
					boolean affected= false;
					for (IProcessItemHandle newIcon: newValues) {
						if (!fPrefixProvider.getProcessContainer().getProcessAttachments().getPath((IProcessAttachmentHandle)newIcon).startsWith(fPrefixProvider.getIconPrefix())) {
							continue;
						}
						if (newIcon instanceof IProcessAttachmentHandle) {
							affected= true;
						}
					}
					if (affected) {
						scheduleJob();
					}
					return;
				}
				
				if (IProcessAttachmentsWorkingCopy.DELETE_EVENT_PROPERTY.equals(property)) {
					scheduleJob();
				}
			}
		}
	}
	
	public static interface IDialogValidator {
		public IStatus isValid();
	}
	
	public static class ModeledElementIdSelectionDialog extends ProcessAttachmentIconSelectionDialog {
		private Text fIdField;
		private String fId;
		private Text fNameField;
		private String fName;
		private boolean fHasName;
		private List<ModeledElement> fAllElements;
		private IAspectMessageProvider fMessageProvider;
		
		public ModeledElementIdSelectionDialog(Shell parentShell, String title, IAspectMessageProvider messageProvider, List<? extends ModeledElement> allElements, ResourceManager resourceManager) {
			this(parentShell, title, messageProvider, allElements, resourceManager, true);
		}
		
		public ModeledElementIdSelectionDialog(Shell parentShell, String title, IAspectMessageProvider messageProvider, List<? extends ModeledElement> allElements, ResourceManager resourceManager, boolean hasName) {
			super(parentShell, title, null, null, resourceManager);
			fHasName = hasName;
			fAllElements= new ArrayList<ModeledElement>(allElements);
			fMessageProvider= messageProvider;
			setValidator(new IDialogValidator() {
				public IStatus isValid() {
					if (fId== null || "".equals(fId)) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.ProcessAttachmentIconSelectionDialog_EMPTY_ID);
					} else if (fId.split("\\s").length > 1) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.ProcessAttachmentIconSelectionDialog_WHITESPACES);
					} else if (checkIdLength() && !AspectEditorUtil.isValidLength(fId, AspectEditorUtil.MAX_SPECIFICATION_STRING_LENGHT - getMaxPrefixLength() - 5)) { // 5 chars for the id number
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, NLS.bind(Messages.ProcessAttachmentIconSelectionDialog_ID_TOO_LONG, AspectEditorUtil.MAX_SPECIFICATION_STRING_LENGHT - getMaxPrefixLength() - 5));
					} else if (fId.contains("/")) {
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.ProcessAttachmentIconSelectionDialog_NO_FORWARD_SLASHES);
					}
					for (ModeledElement s: fAllElements) {
						if (fId.equals(s.getIdentifier())) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, fMessageProvider.getSameIdMessage());
						}
					}
					return Status.OK_STATUS;
				}
			});
		}
		
		protected boolean checkIdLength() {
			return false;
		}
		
		protected int getMaxPrefixLength() {
			return 0;
		}
		
		@Override
		protected void iconChanged(){
			validate();
		}
		
		public String getId() {
			return fId.trim();
		}
		
		public String getName() {
			return fName != null ? fName.trim() : null;
		}
		
		protected void setId(String id) {
			fId= id;
		}

		@Override
		protected void createUpperDialogPart(Composite parent) {
			Composite message= new Composite(parent, SWT.NONE);
			GridLayout layout= new GridLayout();
			layout.marginWidth= 0;
			message.setLayout(layout);
			message.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			Label msg= new Label(message, SWT.WRAP);
			msg.setText(fMessageProvider.getEnterIdMessage());
			msg.setLayoutData(new GridData());
			if (fHasName) {
				Label nameLabel= new Label(parent, SWT.NONE);
				nameLabel.setText(Messages.ProcessAttachmentIconSelectionDialog_NAME);
				nameLabel.setLayoutData(new GridData());
				fNameField= new Text(parent, SWT.BORDER);
				if (fName != null) {
					fNameField.setText(fName);
				}
				fNameField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
				fNameField.addModifyListener(new ModifyListener() {
					public void modifyText(ModifyEvent e) {
						fName= fNameField.getText().trim();
					}
				});
			}
			Label l= new Label(parent, SWT.NONE);
			l.setText(Messages.ProcessAttachmentIconSelectionDialog_ID);

			fIdField= new Text(parent, SWT.BORDER);
			if (fId != null) {
				fIdField.setText(fId);
			}
			fIdField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			fIdField.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					setId(fIdField.getText().trim());
					validate();
				}
			});
			l.setLayoutData(new GridData());
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
			// empty
		}
	}
}
