/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor;

import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.text.IDocument;

import com.ibm.team.process.client.IClientProcess;
import com.ibm.team.process.client.IProcessClientService;
import com.ibm.team.process.client.workingcopies.IProcessAreaWorkingCopy;
import com.ibm.team.process.client.workingcopies.IProcessContainerWorkingCopy;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessConfigurationData;
import com.ibm.team.process.ide.ui.AbstractDynamicActionProvider;
import com.ibm.team.process.internal.common.model.specification.ProcessSpecificationModel;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.client.IAuditableClient;
import com.ibm.team.workitem.client.IWorkItemClient;
import com.ibm.team.workitem.common.IAuditableCommon;
import com.ibm.team.workitem.common.internal.util.AbstractWorkItemDynamicActionProvider;
import com.ibm.team.workitem.common.internal.util.LocalizationContext;
import com.ibm.team.workitem.common.model.IAttribute;


@SuppressWarnings("restriction")
public class WorkItemDynamicActionProvider extends AbstractDynamicActionProvider {

	@Override
	public DynamicAction[] computeActions(IPath actionPath, IProgressMonitor monitor) throws TeamRepositoryException {

		ProcessSpecificationModel model= null;
		final IClientProcess clientProcess= getClientProcess(monitor);

		if (clientProcess == null) {
			model= findModel();
			if (model == null) {
				return new DynamicAction[0];
			}
		}
		
		List<IAttribute> attributes= findAttributes(monitor);

		IAuditableCommon auditableCommon= (IAuditableCommon)getTeamRepository().getClientLibrary(IAuditableClient.class);
		LocalizationContext context= LocalizationContext.createUserContext(auditableCommon, monitor);
		
		AbstractWorkItemDynamicActionProvider<DynamicAction> delegate= new AbstractWorkItemDynamicActionProvider<DynamicAction>(attributes, context, model) {
			@Override
			protected DynamicAction createDynamicAction(String id, String name, String description) {
				return new DynamicAction(id, name, description);
			}

			@Override
			protected IProcessConfigurationData getProjectConfigurationData(String configDataId, IProgressMonitor monitor) throws TeamRepositoryException {
				return clientProcess.getProjectConfigurationData(configDataId, monitor);
			}
		};
		List<DynamicAction> actions= delegate.computeActions(actionPath, null);
		return actions.toArray(new DynamicAction[actions.size()]);
	}
	
	private ProcessSpecificationModel findModel() {
		
		IDocument specification= getProcessContainer().getProcessSpecification();
		if (specification == null)
			return null;
		
		ProcessSpecificationModel model= new ProcessSpecificationModel();
		model.initialize(specification);
		
		return model;
	}
	
	private IClientProcess getClientProcess(IProgressMonitor monitor) throws TeamRepositoryException {
		IProcessContainerWorkingCopy container= getProcessContainer();
		if (container instanceof IProcessAreaWorkingCopy) {
			IProcessArea processArea= ((IProcessAreaWorkingCopy) container).getUnderlyingProcessArea();
			if (processArea != null) {
				IProcessClientService processClientService= (IProcessClientService) getTeamRepository().getClientLibrary(IProcessClientService.class);
				return processClientService.getClientProcess(processArea, monitor);
			}
		}
		return null;
	}
	
	private List<IAttribute> findAttributes(IProgressMonitor monitor) throws TeamRepositoryException {
		IProcessContainerWorkingCopy container= getProcessContainer();
		if (container instanceof IProcessAreaWorkingCopy) {
			IProcessArea processArea= ((IProcessAreaWorkingCopy) container).getUnderlyingProcessArea();
			if (processArea != null) {
				
				ITeamRepository teamRepository= getTeamRepository();
				IWorkItemClient workItemClient= (IWorkItemClient) teamRepository.getClientLibrary(IWorkItemClient.class);
				
				return workItemClient.findAttributes(processArea.getProjectArea(), monitor);
			}
		}
		return Collections.emptyList();
	}
	
}
