/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking;

import java.util.ArrayList;
import java.util.List;

import com.ibm.team.workitem.common.model.IApprovalState;
import com.ibm.team.workitem.common.model.IApprovalType;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ModeledElement;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow.WFAction;

public class ApprovalTracking extends ModeledElement {
	private String fId;
	private List<ApprovalTrackingChangeListener> fListeners= new ArrayList<ApprovalTrackingChangeListener>();
	private List<TrackingEntry> fEntries= new ArrayList<TrackingEntry>();
	private WFWorkflow fWorkflow;
	
	/*package*/ApprovalTracking(String id, WFWorkflow workflow) {
		fId= id;
		fWorkflow= workflow;
	}
	
	@Override
	public String getIdentifier() {
		return fId;
	}
	
	@Override
	public String getName() {
		if (fWorkflow != null) {
			return fWorkflow.getName();
		}
		return fId;
	}
	
	public WFWorkflow getWorkflow() {
		return fWorkflow;
	}
	
	public void addEntry(TrackingEntry entry) {
		fEntries.add(entry);
		notifyListeners(entry);
	}
	
	public void removeEntry(TrackingEntry entry) {
		fEntries.remove(entry);
		notifyListeners(entry);
	}
	
	public List<TrackingEntry> getEntries() {
		return fEntries;
	}
	
	public void notifyListeners(Object changed) {
		ApprovalTrackingChangeEvent event= new ApprovalTrackingChangeEvent(changed);
		for (ApprovalTrackingChangeListener listener: fListeners) {
			listener.approvalTrackingChanged(event);
		}
	}
	
	public void addListener(ApprovalTrackingChangeListener listener) {
		fListeners.add(listener);
	}
	
	public void removeListener(ApprovalTrackingChangeListener listener) {
		fListeners.remove(listener);
	}
	
	public class TrackingEntry {
		private IApprovalType fApprovalType;
		private IApprovalState fApprovalTarget;
		private WFAction fWFAction;
		
		/*package*/TrackingEntry(IApprovalType approvalType, IApprovalState approvalTarget) {
			fApprovalType= approvalType;
			fApprovalTarget= approvalTarget;
		}
		
		public void setWFAction(WFAction wfAction) {
			fWFAction= wfAction;
			notifyListeners(this);
		}

		public IApprovalType getApprovalType() {
			return fApprovalType;
		}

		public IApprovalState getApprovalTarget() {
			return fApprovalTarget;
		}

		public WFAction getWFAction() {
			return fWFAction;
		}
	}
	
	public static interface ApprovalTrackingChangeListener {
		public void approvalTrackingChanged(ApprovalTrackingChangeEvent e);
	}
	
	public static class ApprovalTrackingChangeEvent {
		Object fChanged;
		ApprovalTrackingChangeEvent(Object changed) {
			fChanged= changed;
		}
		
		public Object getChanged() {
			return fChanged;
		}
	}

}
