/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import com.ibm.team.ui.editor.TeamFormPart;
import com.ibm.team.ui.editor.TeamFormSectionPart;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import com.ibm.team.foundation.common.util.IMemento;

import com.ibm.team.process.common.ModelElement;

import com.ibm.team.workitem.ide.ui.internal.HelpContextIds;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AbstractWorkItemAspectEditor;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil.WorkflowComparator;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ModeledElement;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WorkflowManager;

public class ApprovalTrackingAspectEditor extends AbstractWorkItemAspectEditor {

	private ApprovalTrackingPart fApprovalTracking;
	private ManagedForm fManagedForm;
	List<WFWorkflow> fWorkflows;
	
	private static final IAspectMessageProvider MESSAGE_PROVIDER = new IAspectMessageProvider() {
		public String getChooseMessage() {
			return Messages.ApprovalTrackingAspectEditor_CHOSSE_APPROVAL_TRACKING;
		}
		public String getSameIdMessage() {
			return Messages.ApprovalTrackingAspectEditor_ID_EXISTS;
		}
		public String getEnterIdMessage() {
			return Messages.ApprovalTrackingAspectEditor_ENTER_ID;
		}
	};
	
	/*package*/ ApprovalTrackingAspectEditor(String configurationDataId) {
		super(configurationDataId);
		setHelpContextId(HelpContextIds.APPROVAL_TRACKING_ASPECT_EDITOR);
	}
	
	@Override
	protected void commit(List<? extends ModeledElement> elements, IMemento memento) {
		ApprovalTrackingManager.writeTrackings(memento, (List<ApprovalTracking>) elements);
	}

	@Override
	protected void createEditorArea(Composite parent, FormToolkit toolkit) {
		GridLayoutFactory.fillDefaults().applyTo(parent);
		fManagedForm = new ManagedForm(toolkit, toolkit.createScrolledForm(parent));
		ScrolledForm form= fManagedForm.getForm();
		GridDataFactory.fillDefaults().grab(true, true).hint(300, 200).applyTo(form);
		
		Composite body= form.getBody();
		FormLayout layout= new FormLayout();
		layout.spacing= 5;
		body.setLayout(layout);
		
		fApprovalTracking= new ApprovalTrackingPart(getProcessContainerWorkingCopy(), this);
		TeamFormSectionPart approvalTrackingSection = new TeamFormSectionPart(fManagedForm, ExpandableComposite.TITLE_BAR, Messages.ApprovalTrackingAspectEditor_APPROVAL_TRACKING, new TeamFormPart[]{fApprovalTracking} );
		FormData formData= new FormData();
		formData.left= new FormAttachment(0);
		formData.right= new FormAttachment(100);
		formData.top= new FormAttachment(0);
		approvalTrackingSection.getSection().setLayoutData(formData);
		fManagedForm.addPart(approvalTrackingSection);
	}

	@Override
	protected ModeledElement createNewElement() {
		return null;
	}
	
	@Override
	protected void doDispose() {
		if (fManagedForm != null) {
			fManagedForm.dispose();
			fManagedForm= null;
		}
	}
	
	@Override
	protected IAspectMessageProvider getMessageProvider() {
		return MESSAGE_PROVIDER;
	}

	@Override
	protected void inputChanged(ModeledElement newElement) {
		fManagedForm.setInput(newElement);
	}

	private List<WFWorkflow> readWorkflows() {
		ModelElement workflowElement = getSite().getConfigurationData(AspectEditorUtil.WORKFLOWS_CONFIGURATION_POINT);
		if (workflowElement != null) {
			return WorkflowManager.readWorkflows(workflowElement);
		}
		return new ArrayList<WFWorkflow>();
	}

	@Override
	protected List<? extends ModeledElement> readElements(ModelElement configurationElement) {
		fWorkflows= readWorkflows();
		List<ApprovalTracking> trackings = ApprovalTrackingManager.readTrackings(configurationElement, fWorkflows);
		for (WFWorkflow w: fWorkflows) {
			boolean found= false;
			for (ApprovalTracking t: trackings) {
				if (w == t.getWorkflow()) {
					found= true;
					break;
				}
			}
			if (!found) {
				trackings.add(new ApprovalTracking(w.getIdentifier(), w));
			}
		}
		final WorkflowComparator comp= new WorkflowComparator();
		Collections.sort(trackings, new Comparator<ApprovalTracking>() {
			public int compare(ApprovalTracking o1, ApprovalTracking o2) {
				return comp.compare(o1.getWorkflow(), o2.getWorkflow());
			}
		});
		return trackings;
	}
	
	@Override
	protected boolean canAdd() {
		return false;
	}

}
