/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking;

import java.util.ArrayList;
import java.util.List;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.workitem.common.model.WorkItemApprovals;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking.ApprovalTracking.TrackingEntry;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow.WFAction;

public class ApprovalTrackingManager {
	private static final String APPROVAL_TYPE= "approvalType"; //$NON-NLS-1$
	private static final String APPROVAL_TARGET= "approvalTarget"; //$NON-NLS-1$
	private static final String WORKFLOW_ACTION= "workflowAction"; //$NON-NLS-1$
	private static final String WORKFLOW_ID= "workflowId"; //$NON-NLS-1$
	private static final String APPROVAL_TRACKING= "approvalTracking"; //$NON-NLS-1$
	
	public static List<ApprovalTracking> readTrackings(ModelElement configurationElement, List<WFWorkflow> workflows) {
		List<ApprovalTracking> trackings= new ArrayList<ApprovalTracking>();
		if (configurationElement != null) {
			for (ModelElement element: (List<ModelElement>)configurationElement.getChildElements()) {
				String id= element.getAttribute(WORKFLOW_ID);
				ApprovalTracking tracking= null;
				for (ApprovalTracking t: trackings) {
					if (t.getWorkflow().getId().equals(id)) {
						tracking= t;
						break;
					}
				}
				WFWorkflow wf= null;
				for (WFWorkflow workflow: workflows) {
					if (workflow.getId().equals(id)) {
						wf= workflow;
						break;
					}
				}
				if (wf == null) {
					wf= new WFWorkflow(id, id, null, null);
					workflows.add(wf);
				}
				if (tracking == null) {
					tracking= new ApprovalTracking(id, wf);
					trackings.add(tracking);
				}
				String type= element.getAttribute(APPROVAL_TYPE);
				String target= element.getAttribute(APPROVAL_TARGET);
				String action= element.getAttribute(WORKFLOW_ACTION);
				WFAction act= null;
				for (WFAction a: wf.getActions()) {
					if (a.getId().equals(action)) {
						act= a;
						break;
					}
 				}
				if (act == null) {
					act= new WFAction(wf, action, action, null, null);
				}
				TrackingEntry entry= tracking.new TrackingEntry(WorkItemApprovals.getType(type), WorkItemApprovals.getState(target));
				entry.setWFAction(act);
				tracking.addEntry(entry);
			}
		}
		return trackings;
	}

	private static void writeTracking(IMemento parentMemento, ApprovalTracking tracking) {
		for (TrackingEntry entry: tracking.getEntries()) {
			String type= null;
			if (entry.getApprovalType() != null) {
				type= entry.getApprovalType().getIdentifier();
			}
			String target= null;
			if (entry.getApprovalTarget() != null) {
				target= entry.getApprovalTarget().getIdentifier();
			}
			String action= null;
			if (entry.getWFAction() != null) {
				action= entry.getWFAction().getId();
			}
			if (type == null || target == null || action == null) {
				continue;
			}
			IMemento memento= parentMemento.createChild(APPROVAL_TRACKING);
			memento.putString(WORKFLOW_ID, tracking.getWorkflow().getId());
			memento.putString(APPROVAL_TYPE, type);
			memento.putString(APPROVAL_TARGET, target);
			memento.putString(WORKFLOW_ACTION, action);
		}
	}
	
	public static void writeTrackings(IMemento memento, List<ApprovalTracking> trackings) {
		for (ApprovalTracking t: trackings) {
			writeTracking(memento, t);
		}
	}

}
