/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.accessibility.ACC;
import org.eclipse.swt.accessibility.AccessibleAdapter;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.process.client.workingcopies.IProcessContainerWorkingCopy;
import com.ibm.team.ui.editor.TeamFormPart;
import com.ibm.team.workitem.common.model.IApprovalState;
import com.ibm.team.workitem.common.model.IApprovalType;
import com.ibm.team.workitem.common.model.WorkItemApprovals;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.IDirtyStateTracker;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ProcessAttachmentIconSelectionDialog;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking.ApprovalTracking.ApprovalTrackingChangeEvent;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking.ApprovalTracking.ApprovalTrackingChangeListener;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.approvaltracking.ApprovalTracking.TrackingEntry;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow.WFAction;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedCombo;
import com.ibm.team.workitem.rcp.ui.internal.util.Utils;

public class ApprovalTrackingPart extends TeamFormPart {

	private TreeViewer fApprovalViewer;
	private IDirtyStateTracker fTracker;
	
	private ApprovalTracking fTracking;
	
	private static final String APPROVAL_LABEL= Messages.ApprovalTrackingPart_APPROVAL_LABEL;
	private static final String ACTION_LABEL= Messages.ApprovalTrackingPart_ACTION_LABEL;
	private static final String NONE= Messages.ApprovalTrackingPart_NONE;
	
	private ResourceManager fResourceManager= new LocalResourceManager(JFaceResources.getResources());
	private IProcessContainerWorkingCopy fContainer;
	
	private ApprovalTrackingChangeListener fListener= new ApprovalTrackingChangeListener() {
		public void approvalTrackingChanged(final ApprovalTrackingChangeEvent e) {
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					fApprovalViewer.refresh();
					if (e.getChanged() != null) {
						fApprovalViewer.setSelection(new StructuredSelection(e.getChanged()));
						fApprovalViewer.getTree().setFocus();
					}
				}
			});
		}
	};
	
	
	/*package*/ApprovalTrackingPart(IProcessContainerWorkingCopy processContainer, IDirtyStateTracker tracker) {
		fContainer= processContainer;
		fTracker= tracker;
	}
	
	private void setDirty() {
		fTracker.setDirty();
	}
	
	@Override
	public void createContent(Composite parent) {
		GridLayout layout = new GridLayout(2, false);
		layout.marginWidth= 0;
		layout.verticalSpacing= 0;
		parent.setLayout(layout);
		
		Composite viewer= new Composite(parent, SWT.NONE);
		viewer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		Composite buttonBar= new Composite(parent, SWT.NONE);
		GridData data= new GridData(SWT.LEFT, SWT.BEGINNING, false, false);
		buttonBar.setLayoutData(data);
		createViewer(viewer, buttonBar);
	}

	private void createViewer(Composite viewer, Composite buttonBar) {
		FormToolkit toolkit= getSite().getToolkit();
		GridLayout layout= new GridLayout();
		layout.marginWidth= 0;
		viewer.setLayout(layout);
		Tree t= new Tree(viewer, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
		t.setHeaderVisible(true);
		t.setLinesVisible(true);
		GridData data= new GridData(SWT.FILL, SWT.FILL, true, true);
		data.minimumHeight= t.getItemHeight()*14;
		t.setLayoutData(data);
		TreeColumn c= new TreeColumn(t, SWT.NONE);
		c.setWidth(250);
		c.setResizable(true);
		c.setText(APPROVAL_LABEL);
		c= new TreeColumn(t, SWT.NONE);
		c.setWidth(180);
		c.setResizable(true);
		c.setText(ACTION_LABEL);
		t.getAccessible().addAccessibleListener(new AccessibleAdapter(){
			@Override
			public void getName(AccessibleEvent e) {
				if(e.childID == ACC.CHILDID_SELF){
					e.result = Messages.ApprovalTrackingAspectEditor_APPROVAL_TRACKING;
				}
			}
			
		});
		fApprovalViewer= new TreeViewer(t);
		fApprovalViewer.setColumnProperties(new String[]{APPROVAL_LABEL, ACTION_LABEL});
		fApprovalViewer.setLabelProvider(new CellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				Object element= cell.getElement();
				if (element instanceof TrackingEntry) {
					TrackingEntry entry= (TrackingEntry)element;
					if (cell.getColumnIndex() == 0) {
						cell.setText(entry.getApprovalTarget() != null? entry.getApprovalTarget().getDisplayName():null);
						cell.setImage(getImage(element, entry.getApprovalTarget().getIconURL().toString()));
					} else if (cell.getColumnIndex() == 1) {
						cell.setText(entry.getWFAction() != null?entry.getWFAction().getName():null);
						cell.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_LIST_FOREGROUND));
						if (entry.getWFAction() != null && entry.getWFAction().getIcon() != null) {
							cell.setImage(getImage(element, entry.getWFAction().getIcon()));
						}
					}
				}
				if (element instanceof StateEntry) {
					if (cell.getColumnIndex() == 0) {
						cell.setText(((StateEntry)element).getState().getDisplayName());
						cell.setImage(getImage(element, ((StateEntry)element).getState().getIconURL().toString()));
					} else if (cell.getColumnIndex() == 1) {
						cell.setText(Messages.ApprovalTrackingPart_NONE_CELL_ENTRY);
						cell.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW));
						cell.setImage(null);
					}
				}
				if (element instanceof IApprovalType) {
					if (cell.getColumnIndex() == 0) {
						cell.setText(((IApprovalType)element).getDisplayName());
						cell.setImage(getImage(element, ((IApprovalType)element).getIconURL().toString()));
					}
				}
			}
			
			private Image getImage(final Object element, String iconPath) {
				return AspectEditorUtil.getImage(fContainer, iconPath, fResourceManager, new Runnable() {
					public void run() {
						fireChanged(element);
					}
				});
			}
			
			private void fireChanged(Object element) {
				fireLabelProviderChanged(new LabelProviderChangedEvent(this, element));
			}
			
		});
		fApprovalViewer.setContentProvider(new ITreeContentProvider() {
			private TrackingEntry getEntry(IApprovalType type, IApprovalState state) {
				for (TrackingEntry e: fTracking.getEntries()) {
					if (e.getApprovalType().getIdentifier().equals(type.getIdentifier()) && e.getApprovalTarget().getIdentifier().equals(state.getIdentifier())) {
						return e;
					}
				}
				return null;
			}
			public Object[] getChildren(Object parentElement) {
				if (parentElement instanceof IApprovalType) {
					List<Object> result= new ArrayList<Object>();
					for (IApprovalState state: WorkItemApprovals.getStates()) {
						TrackingEntry e= getEntry((IApprovalType)parentElement, state);
						if (e != null) {
							result.add(e);
						} else {
							result.add(new StateEntry((IApprovalType)parentElement, state));
						}
					}
					return result.toArray();
				}
				return new Object[0];
			}

			public Object getParent(Object element) {
				if (element instanceof TrackingEntry) {
					return ((TrackingEntry)element).getApprovalType();
				}
				if (element instanceof StateEntry) {
					return ((StateEntry)element).getType();
				}
				return null;
			}

			public boolean hasChildren(Object element) {
				return getChildren(element).length > 0;
			}

			public Object[] getElements(Object inputElement) {
				return WorkItemApprovals.getTypes().toArray();
			}

			public void dispose() {
			}

			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		});
		
		layout= new GridLayout();
		layout.marginTop= fApprovalViewer.getTree().getHeaderHeight();
		buttonBar.setLayout(layout);
		
		final Button editEnumeration = toolkit.createButton(buttonBar, Messages.ApprovalTrackingPart_EDIT, SWT.FLAT);
		GridData btnData = new GridData(SWT.FILL, SWT.CENTER, false, false);
		btnData.minimumWidth = 50;
		editEnumeration.setLayoutData(btnData);
		editEnumeration.setEnabled(false);
		editEnumeration.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				editEntry();
			}
		});
		
		final Button removeEntry= toolkit.createButton(buttonBar, Messages.ApprovalTrackingPart_CLEAR, SWT.FLAT);
		btnData = new GridData(SWT.FILL, SWT.CENTER, false, false);
		btnData.minimumWidth = 50;
		removeEntry.setLayoutData(btnData);
		removeEntry.setEnabled(false);
		removeEntry.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				removeEntry();
			}
		});

		
		fApprovalViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				removeEntry.setEnabled(validRemoveSelection());
				editEnumeration.setEnabled(validEditSelection());
			}
		});
		fApprovalViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (validEditSelection()) {
					editEntry();
				}
			}
			
		});
		updateEnablement();
		Utils.updateColumnWidths(fApprovalViewer.getTree(), -1, 1);
		createContextMenu();
	}
	
	private void createContextMenu()
	{
		final MenuManager contextMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		contextMenuMgr.setRemoveAllWhenShown(true);
		contextMenuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(final IMenuManager manager) {				
				if (validEditSelection())
				{
					final Action edit = new Action(Messages.ApprovalTrackingPart_EDIT){
						public void run() {
							editEntry();
						}
					};
					
					contextMenuMgr.add(edit);
				}
				
				if (validRemoveSelection())
				{
					final Action remove = new Action(Messages.ApprovalTrackingPart_CLEAR){
						public void run() {
							removeEntry();
						}
					};
					
					contextMenuMgr.add(remove);
				}
			}
		});
		
		fApprovalViewer.getTree().setMenu(contextMenuMgr.createContextMenu(fApprovalViewer.getControl()));
	}
	
	private boolean validRemoveSelection() {
		for (Object o: ((IStructuredSelection)fApprovalViewer.getSelection()).toList()) {
			if (o instanceof TrackingEntry) {
				return true;
			}
		}
		return false;
	}
	
	private boolean validEditSelection() {
		if (((IStructuredSelection)fApprovalViewer.getSelection()).size() != 1) {
			return false;
		}
		return ((IStructuredSelection)fApprovalViewer.getSelection()).getFirstElement() instanceof TrackingEntry || ((IStructuredSelection)fApprovalViewer.getSelection()).getFirstElement() instanceof StateEntry;
	}
	
	private void removeEntry()
	{
		boolean removedAny= false;
		for (Object o: ((IStructuredSelection)fApprovalViewer.getSelection()).toList()) {
			if (o instanceof TrackingEntry) {
				fTracking.removeEntry((TrackingEntry)o);
				removedAny= true;
			}
		}
		if (removedAny) {
			setDirty();
		}
	}
	
	private void editEntry() {
		Object selected= ((IStructuredSelection)fApprovalViewer.getSelection()).getFirstElement();
		IApprovalType type= null;
		IApprovalState state= null;
		WFAction action= null;
		if (selected instanceof StateEntry) {
			type= ((StateEntry)selected).getType();
			state= ((StateEntry)selected).getState();
		}
		TrackingEntry entry= null;
		if (selected instanceof TrackingEntry) {
			entry= (TrackingEntry)selected;
			type= entry.getApprovalType();
			state= entry.getApprovalTarget();
			action= entry.getWFAction();
		}
		EditEntryDialog d= new EditEntryDialog(Display.getCurrent().getActiveShell(), Messages.ApprovalTrackingPart_EDIT_TRACKING,type, state, action, fTracking.getWorkflow().getActions(), fResourceManager);
		if (d.open() == Window.OK) {
			WFAction act= d.getAction();
			if (act == null && entry != null) {
				fTracking.removeEntry(entry);
				setDirty();
				return;
			}
			if (act != null && entry != null) {
				if (!act.equals(entry.getWFAction())) {
					entry.setWFAction(act);
					setDirty();
				}
				return;
			}
			if (entry == null && act != null) {
				entry= fTracking.new TrackingEntry(type, state);
				entry.setWFAction(act);
				fTracking.addEntry(entry);
				setDirty();
			}
		}
	}
	
	private void updateEnablement() {
		if (fApprovalViewer != null && !fApprovalViewer.getControl().isDisposed()) {
			fApprovalViewer.getControl().setEnabled(fTracking != null);
		}
	}
	
	@Override
	public boolean stretchHorizontally() {
		return true;
	}
	
	@Override
	public boolean stretchVertically() {
		return true;
	}
	
	@Override
	public void setInput(Object input) {
		if (fTracking != null) {
			fTracking.removeListener(fListener);
		}
		if (input instanceof ApprovalTracking) {
			fTracking= (ApprovalTracking)input;
			fApprovalViewer.setInput(fTracking);
			fTracking.addListener(fListener);
		} else {
			fTracking= null;
			fApprovalViewer.setInput(null);
		}
		updateEnablement();
		fApprovalViewer.expandAll();
	}
	
	@Override
	public void dispose() {
		if (fTracking != null) {
			fTracking.removeListener(fListener);
		}
		if (fResourceManager != null) {
			fResourceManager.dispose();
			fResourceManager= null;
		}
		super.dispose();
	}
	
	private static class StateEntry {
		IApprovalType fType;
		IApprovalState fState;
		public StateEntry(IApprovalType type, IApprovalState state) {
			fType= type;
			fState= state;
		}
		public IApprovalType getType() {
			return fType;
		}
		public IApprovalState getState() {
			return fState;
		}
	}
	
	private static class EditEntryDialog extends ProcessAttachmentIconSelectionDialog {
		private IApprovalType fType;
		private IApprovalState fTarget;
		private WFAction fAction;
		private List<WFAction> fActions;
		private DecoratedCombo fActionsCombo;
			
		protected EditEntryDialog(Shell parentShell, String dialogTitle, IApprovalType type, IApprovalState target, WFAction action, List<WFAction> actions, ResourceManager resourceManager) {
			super(parentShell, dialogTitle, null, null, resourceManager);
			fType= type;
			fTarget= target;
			fAction= action;
			if (actions != null) {
				fActions= new ArrayList<WFAction>(actions);
			} else {
				fActions= new ArrayList<WFAction>();
			}
		}
		
		public WFAction getAction() {
			return fAction;
		}
		
		@Override
		protected void createUpperDialogPart(Composite parent) {
			Label l= new Label(parent, SWT.NONE);
			l.setText(Messages.ApprovalTrackingPart_APPROVAL_TYPE);
			l.setLayoutData(new GridData());
			String t= fType.getDisplayName();
			if (t == null) {
				t= NONE;
			}
			Label typeField= new Label(parent, SWT.BORDER);
			typeField.setText(t);
			typeField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			
			l= new Label(parent, SWT.NONE);
			l.setText(Messages.ApprovalTrackingPart_APPROVAL_TARGET);
			l.setLayoutData(new GridData());
			String s= fTarget.getDisplayName();
			if (s == null) {
				s= NONE;
			}
			Label stateField= new Label(parent, SWT.BORDER);
			stateField.setText(s);
			stateField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			
			l= new Label(parent, SWT.NONE);
			l.setText(Messages.ApprovalTrackingPart_WORKFLOW_ACTION);
			l.setLayoutData(new GridData());
			
			fActionsCombo= new DecoratedCombo(parent, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
			AspectEditorUtil.adaptCombo(fActionsCombo.getCombo());
			fActionsCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			fActionsCombo.setLabelProvider(new LabelProvider() {
				@Override
				public String getText(Object o) {
					if (o instanceof String) {
						return (String)o;
					}
					if (o instanceof WFAction) {
						return ((WFAction)o).getName();
					}
					return null;
				}
			});
			Object[] values= new Object[fActions.size()+1];
			values[0]= NONE;
			for (int i= 1; i <= fActions.size(); i++) {
				values[i]= fActions.get(i-1);
			}
			fActionsCombo.setValueSet(values);
			if (fAction == null) {
				fActionsCombo.setValue(NONE);
			} else {
				fActionsCombo.setValue(fAction);
			}
			fActionsCombo.addSelectionChangedListener(new ISelectionChangedListener() {
				public void selectionChanged(SelectionChangedEvent event) {
					Object o= fActionsCombo.getValue();
					if (NONE.equals(o)) {
						fAction= null;
					} else if (o instanceof WFAction) {
						fAction= (WFAction)o;
					}
				}
			});
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
			// empty
		}
	}
}
