/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;

import com.ibm.icu.text.Collator;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.common.internal.presentations.AttributeTypePresentationIdBindingManager;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.ide.ui.internal.ImagePool;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ProcessAttachmentIconSelectionDialog;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorPresentation.Attribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.configurationlet.Configurationlet;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.configurationlet.FallbackConfigurationlet;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.valueprovider.IProcessConfigurationDataProvider;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedCombo;
import com.ibm.team.workitem.ide.ui.internal.editor.presentations.KindAttributeDescriptor;
import com.ibm.team.workitem.ide.ui.internal.editor.presentations.PresentationsManager;

public class AddEditPresentationDialog extends ProcessAttachmentIconSelectionDialog {
	private Text fIdField;
	private DecoratedCombo fKindsCombo;
	private DecoratedCombo fAKindsCombo;
	private DecoratedCombo fAttributeCombo;
	private String fId;
	private AddEditPresentationDialog.Kind fKind;
	private AddEditPresentationDialog.Kind fAKind;
	private String fLabel;
	private String fDescription;
	private boolean fIsAttribute= true;
	private Attribute fAttribute;
	private List<AddEditPresentationDialog.Kind> fKinds;
	private List<Attribute> fAllAttributes;
	private Map<String, String> fProperties;
	private List<Attribute> fIdenticallyNamedAttributes;
	private List<Attribute> fNotAlreadyExistingAttributes;
	private boolean fShowArchived= false;
	private boolean fShowAlreadyConfigured= false;
	private ToolBarManager fToolbarManager;
	private Composite fConfiguration;
	private IProcessConfigurationDataProvider fProcessProvider;
	private Configurationlet fConfig = null;
	private EditorPresentationAspectPart fPart = null;
	
	static Comparator<Attribute> attributeComparator = new Comparator<Attribute>() {
		public int compare(Attribute o1, Attribute o2) {
			if (o1 == null) {
				return 1;
			}
			if (o2 == null) {
				return -1;
			}
			return Collator.getInstance().compare(o1.getDisplayName(), o2.getDisplayName());
		}
	};
	
	public AddEditPresentationDialog(Shell parentShell, String dialogTitle, String id, Attribute attribute, String kind, String label, String description, Map<String, String> properties, List<String> kinds, List<Attribute> notAlreadyExistingAttributes, List<Attribute> allAttributes, IProcessConfigurationDataProvider processProvider, ResourceManager resourceManager, EditorPresentationAspectPart editorPresentationAspectPart) {
		super(parentShell, dialogTitle, null, null, resourceManager);
		fPart = editorPresentationAspectPart;
		fId= id;
		if (kind != null) {
			fKind= new Kind(kind, PresentationsManager.getDisplayName(kind));
		}
		fKinds= new ArrayList<AddEditPresentationDialog.Kind>();
		if (kinds != null) {
			for (String k: kinds) {
				fKinds.add(new Kind(k, PresentationsManager.getDisplayName(k)));
			}
		}
		Collections.sort(fKinds, new Comparator<AddEditPresentationDialog.Kind>() {
			public int compare(AddEditPresentationDialog.Kind o1, AddEditPresentationDialog.Kind o2) {
				return Collator.getInstance().compare(o1.getName(), o2.getName());
			}
		});
		fLabel= label;
		fDescription= description;
		fAttribute= attribute;
		if (attribute == null && kind != null) {
			fIsAttribute= false;
		}

		if (notAlreadyExistingAttributes != null) {
			fNotAlreadyExistingAttributes= new ArrayList<Attribute>(notAlreadyExistingAttributes);
		} else {
			fNotAlreadyExistingAttributes= new ArrayList<Attribute>();
		}
		if (fAttribute != null) {
			fNotAlreadyExistingAttributes.add(fAttribute);
		}
		Collections.sort(fNotAlreadyExistingAttributes, attributeComparator);
		if (allAttributes != null) {
			fAllAttributes= new ArrayList<Attribute>(allAttributes);
			Collections.sort(fAllAttributes, attributeComparator);
		} else {
			fAllAttributes= new ArrayList<Attribute>();
		}
		fIdenticallyNamedAttributes = findIdenticallyNamedAttributes(fAllAttributes);
		if (properties == null) {
			fProperties= new HashMap<String, String>();
		} else {
			fProperties= new HashMap<String, String>(properties);
		}
		fProcessProvider= processProvider;
		
		setValidator();
	}
	
	private void setValidator (){
		setValidator(new IDialogValidator() {
			public IStatus isValid() {
				if (fIsAttribute && fAttribute == null) {
					return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.AddEditPresentationDialog_SELECT_A_ATTRIBUTE);
				} else if (fIsAttribute) {
					if (fAKind == null) {
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.AddEditPresentationDialog_SELECT_KIND);
					}
					//Better way to do this?  Worried about caching this if process spec changes.
					Map<String, KindAttributeDescriptor> kinds = PresentationsManager.getPossibleKinds(fAttribute.getAttributeType());
					KindAttributeDescriptor desc = kinds.get(fAKind.getId());
					if(desc != null && desc.isDeprecated()){
						return new Status(IStatus.WARNING, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.AddEditPresentationDialog_KIND_DEPRECATED);
					}
				} else if (!fIsAttribute && fKind == null){
					return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.AddEditPresentationDialog_SELECT_KIND);
				}
				if (fConfig instanceof Configurationlet){
					return fConfig.isValid();
				}
				return Status.OK_STATUS;
			}
		});
	}
	
	private List<Attribute> findIdenticallyNamedAttributes(List<Attribute> attributes) {
		List<Attribute> identicallyNamed= new ArrayList<Attribute>();
		for (int i= 0; i < attributes.size() -1; i++) {
			if (attributes.get(i).getDisplayName().equals(attributes.get(i+1).getDisplayName())) {
				identicallyNamed.add(attributes.get(i));
				identicallyNamed.add(attributes.get(i+1));
			}
		}
		return identicallyNamed;
	}

	public String getId() {
		if ("".equals(fId)) {  //$NON-NLS-1$
			return null;
		}
		return fId;
	}
	
	public Attribute getAttribute() {
		if (fIsAttribute) {
			return fAttribute;
		}
		return null;
	}
	
	public String getKind() {
		if (fIsAttribute) {
			if (fAKind == null) {
				return null;
			}
			return fAKind.getId();
		}
		if (fKind == null) {
			return null;
		}
		return fKind.getId();
	}
	
	public String getLabel() {
		if ("".equals(fLabel)) {  //$NON-NLS-1$
			return null;
		}
		return fLabel;
	}
	
	public String getDescription() {
		if ("".equals(fDescription)) {  //$NON-NLS-1$
			return null;
		}
		return fDescription;
	}
	
	public Map<String, String> getProperties() {
		return fProperties;
	}
	
	@Override
	protected void createUpperDialogPart(Composite parent) {
		
		final List<Control> attControls= new ArrayList<Control>();
		final List<Control> kControls= new ArrayList<Control>();
		final Button attr= new Button(parent, SWT.RADIO);
		attr.setText(Messages.AddEditPresentationDialog_ATTRIBUTE_BASED_PRESENTATION);
		attr.setSelection(fIsAttribute);
		attr.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				fIsAttribute= true;
				updateEnablement(attControls, kControls);
				updateConfigurationUI();
				validate();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		attr.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
		Label l= new Label(parent, SWT.NONE);
		attControls.add(l);
		l.setText(Messages.AddEditPresentationDialog_ATTRIBUTE);
		GridData data = new GridData();
		data.horizontalIndent= IDialogConstants.INDENT;
		l.setLayoutData(data);
		
		Composite filteredCombo= new Composite(parent, SWT.NONE);
		filteredCombo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		GridLayout layout = new GridLayout(2, false);
		layout.marginHeight= 0;
		layout.marginWidth= 0;
		filteredCombo.setLayout(layout);
		
		fAttributeCombo= new DecoratedCombo(filteredCombo, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
		attControls.add(fAttributeCombo.getCombo());
		AspectEditorUtil.adaptCombo(fAttributeCombo.getCombo());
		fAttributeCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		fAttributeCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object o) {
				if (o instanceof String) {
					return (String)o;
				}
				if (o instanceof Attribute) {
					if (fIdenticallyNamedAttributes.contains(o)) {
						return NLS.bind(Messages.AddEditPresentationDialog_ATTRIBUTE_NAME_ID, ((Attribute)o).getDisplayName(), ((Attribute) o).getIdentifier().getStringIdentifier());
					} else {
						return ((Attribute)o).getDisplayName();
					}
				}
				return null;
			}
		});
		fAttributeCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			private Object oldValue = EditorPresentationAspectPart.NONE;
			public void selectionChanged(SelectionChangedEvent event) {
				Object attribute= fAttributeCombo.getValue();
				if (EditorPresentationAspectPart.NONE.equals(attribute)) {
					fAttribute= null;
				} else if (Messages.AddEditPresentationDialog_NEW_CUSTOM_ATTRIBUTE.equals(attribute)){
					Attribute newAttr = fPart.openAddAttributeDialog();
					if(newAttr == null){
						fAttributeCombo.setValue(oldValue);
						return;
					} else {
						/**
						 * Update the UI to include the new Attribute and select it.
						 */
						fNotAlreadyExistingAttributes.add(newAttr);
						Collections.sort(fNotAlreadyExistingAttributes, attributeComparator);
						fAllAttributes.add(newAttr);
						Collections.sort(fAllAttributes, attributeComparator);
						fAttribute = newAttr;
						updateAttributeValueSet();
					}
				} else {
					fAttribute= (Attribute)attribute;
				}
				updateAttributeKindValueSet();
				updateConfigurationUI();
				validate();
				oldValue = attribute;
			}
		});
		
		final Action archived= new Action(Messages.AddEditPresentationDialog_SHOW_ARCHIVED, Action.AS_CHECK_BOX) {
			@Override
			public void run() {
				fShowArchived= !fShowArchived;
				updateAttributeValueSet();
			}
		};
		final Action alreadyConfigured= new Action(Messages.AddEditPresentationDialog_SHOW_CONFIGURED, Action.AS_CHECK_BOX) {
			@Override
			public void run() {
				fShowAlreadyConfigured= !fShowAlreadyConfigured;
				updateAttributeValueSet();
			}
		};
		final Menu menu= new Menu(filteredCombo);
		new ActionContributionItem(alreadyConfigured).fill(menu, -1);
		new ActionContributionItem(archived).fill(menu, -1);
		
		fToolbarManager = new ToolBarManager(SWT.FLAT);
		final IMenuCreator creator= new IMenuCreator() {
			public Menu getMenu(Control parent) {
				return menu;
			}
			public Menu getMenu(Menu parent) {
				return null;
			}
			public void dispose() {
				menu.dispose();
			}
		};
		Action button= new Action(Messages.AddEditPresentationDialog_FILTERS, IAction.AS_DROP_DOWN_MENU) {
			@Override
			public IMenuCreator getMenuCreator() {
				return creator;
			}
			@Override
			public ImageDescriptor getImageDescriptor() {
				return ImagePool.FILTER_ICON;
			}
		};
		fToolbarManager.add(button);
		ToolBar toolbar= fToolbarManager.createControl(filteredCombo);
		toolbar.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		
		
		l= new Label(parent, SWT.NONE);
		attControls.add(l);
		l.setText(Messages.AddEditPresentationDialog_KIND_LABEL);
		data = new GridData();
		data.horizontalIndent= IDialogConstants.INDENT;
		l.setLayoutData(data);
		
		fAKindsCombo= new DecoratedCombo(parent, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
		attControls.add(fAKindsCombo.getCombo());
		AspectEditorUtil.adaptCombo(fAKindsCombo.getCombo());
		fAKindsCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		fAKindsCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object o) {
				if (o instanceof String) {
					return (String)o;
				}
				if (o instanceof AddEditPresentationDialog.Kind) {
					return ((AddEditPresentationDialog.Kind)o).getName();
				}
				return null;
			}
		});
		fAKindsCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				Object kind= fAKindsCombo.getValue();
				if (EditorPresentationAspectPart.NONE.equals(kind)) {
					fAKind= null;
				} else {
					fAKind= (AddEditPresentationDialog.Kind)kind;
				}
				updateConfigurationUI();
				validate();
			}
		});
		
		Object[] valueSet= new Object[fKinds.size()+1];
		valueSet[0]= EditorPresentationAspectPart.NONE;
		for (int i= 1; i <= fKinds.size(); i++) {
			valueSet[i]= fKinds.get(i-1);
		}
		fAKindsCombo.setValueSet(valueSet);
		if (fKind == null || !fIsAttribute) {
			fAKindsCombo.setValue(EditorPresentationAspectPart.NONE);
		} else {
			fAKindsCombo.setValue(fKind);
		}
		
		updateAttributeValueSet();
		
		final Button kind= new Button(parent, SWT.RADIO);
		kind.setText(Messages.AddEditPresentationDialog_NON_ATTRIBUTE_BASED);
		kind.setSelection(!fIsAttribute);
		kind.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
		kind.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				fIsAttribute= false;
				updateEnablement(attControls, kControls);
				updateConfigurationUI();
				validate();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		
		l= new Label(parent, SWT.NONE);
		kControls.add(l);
		l.setText(Messages.AddEditPresentationDialog_KIND_LABEL);
		data = new GridData();
		data.horizontalIndent= IDialogConstants.INDENT;
		l.setLayoutData(data);
		fKindsCombo= new DecoratedCombo(parent, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
		kControls.add(fKindsCombo.getCombo());
		AspectEditorUtil.adaptCombo(fKindsCombo.getCombo());
		fKindsCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		fKindsCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object o) {
				if (o instanceof String) {
					return (String)o;
				}
				if (o instanceof AddEditPresentationDialog.Kind) {
					return ((AddEditPresentationDialog.Kind)o).getName();
				}
				return null;
			}
		});
		fKindsCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				Object kind= fKindsCombo.getValue();
				if (EditorPresentationAspectPart.NONE.equals(kind)) {
					fKind= null;
				} else {
					fKind= (AddEditPresentationDialog.Kind)kind;
				}
				updateConfigurationUI();
				validate();
			}
		});
		
		List<AddEditPresentationDialog.Kind> values= new ArrayList<AddEditPresentationDialog.Kind>();
		for (AddEditPresentationDialog.Kind k: fKinds) {
			if (!PresentationsManager.needsAttribute(k.getId())) {
				values.add(k);
			}
		}
		valueSet= new Object[values.size()+1];
		valueSet[0]= EditorPresentationAspectPart.NONE;
		int i= 1;
		for (AddEditPresentationDialog.Kind k: values) {
			valueSet[i]= k;
			i++;
		}
		fKindsCombo.setValueSet(valueSet);
		if (fKind == null || fIsAttribute) {
			fKindsCombo.setValue(EditorPresentationAspectPart.NONE);
		} else {
			fKindsCombo.setValue(fKind);
		}
		for (Control c: kControls) {
			c.setEnabled(false);
		}
		
		Composite spacer= new Composite(parent, SWT.NONE);
		data= new GridData(SWT.FILL, SWT.FILL, false, false, 2, 1);
		data.heightHint= 10;
		spacer.setLayoutData(data);
		
		l= new Label(parent, SWT.NONE);
		l.setText(Messages.AddEditPresentationDialog_LABEL);
		final Text label= new Text(parent, SWT.BORDER);
		if (fLabel != null) {
			label.setText(fLabel);
		}
		label.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		label.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				fLabel= label.getText().trim();
			}
		});
		
		l= new Label(parent, SWT.NONE);
		l.setText(Messages.AddEditPresentationDialog_DESCRIPTION);
		final Text description= new Text(parent, SWT.BORDER);
		if (fDescription != null) {
			description.setText(fDescription);
		}
		description.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		description.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				fDescription= description.getText().trim();
			}
		});		

		l= new Label(parent, SWT.NONE);
		l.setText(Messages.AddEditPresentationDialog_ID);
		data = new GridData();
		l.setLayoutData(data);
		fIdField= new Text(parent, SWT.BORDER);
		if (fId != null) {
			fIdField.setText(fId);
		}
		fIdField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		fIdField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				fId= fIdField.getText().trim();
			}
		});
		fConfiguration= new Composite(parent, SWT.NONE);
		fConfiguration.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		fConfiguration.getShell().addShellListener(new ShellAdapter(){
			public void shellActivated(ShellEvent e) {
				//Have to repack after the button bar gets created, so we will wait until the Shell is actually activated.
				fConfiguration.getShell().pack(true);
			}
		});

		updateConfigurationUI();
		updateEnablement(attControls, kControls);
	}
	
	
	
	private void updateConfigurationUI() {
		if (fConfiguration != null && ! fConfiguration.isDisposed()) {
			for (Control child: fConfiguration.getChildren()) {
				child.dispose();
			}
			String kindId= fIsAttribute ? fAKind != null ? fAKind.getId() : null : fKind != null ? fKind.getId() : null;
			fConfig= getConfigurationlet(kindId);
			fConfig.configure(fProperties, fProcessProvider, fAttribute);
			fConfig.createContent(fConfiguration);
			fConfiguration.layout(true);

			if (fConfiguration.getShell().isVisible()) {
				//Repack the shell because the inner contents have changed.
				fConfiguration.getShell().pack(true);
			}
			
			fConfig.setupValidation(this);
		}
	}
	
	private void updateAttributeValueSet() {
		List<Attribute> aValues= new ArrayList<Attribute>();
		List<Attribute> attributes;
		if (fShowAlreadyConfigured) {
			attributes= fAllAttributes;
		} else {
			attributes= fNotAlreadyExistingAttributes;
		}
		for (Attribute a: attributes) {
			if (!PresentationsManager.getPossibleKinds(a.getAttributeType()).isEmpty() || AttributeTypePresentationIdBindingManager.getAttributeTypePresentationId(a.getAttributeType()) != null) {
				if (!fShowArchived && a.isCustomAttribute() && !a.isConfiguredInSpec()) {
					continue;
				}
				aValues.add(a);
			}
		}
		Object[] valueSet= new Object[aValues.size()+2];
		valueSet[0]= EditorPresentationAspectPart.NONE;
		valueSet[valueSet.length-1]= Messages.AddEditPresentationDialog_NEW_CUSTOM_ATTRIBUTE;
		int i = 1;
		for (Attribute a: aValues) {
			valueSet[i]= a;
			i++;
		}
		fAttributeCombo.setValueSet(valueSet);
		if (fAttribute == null) {
			fAttributeCombo.setValue(EditorPresentationAspectPart.NONE);
		} else {
			fAttributeCombo.setValue(fAttribute);
		}
	}
	
	private void updateEnablement(List<Control> attControls, List<Control> kControls) {
		for (Control c: attControls) {
			c.setEnabled(fIsAttribute);
		}
		for (Control c: kControls) {
			c.setEnabled(!fIsAttribute);
		}
		if (fIsAttribute) {
			updateAttributeKindValueSet();
		}
	}
	
	private void updateAttributeKindValueSet() {
		List<AddEditPresentationDialog.Kind> values= new ArrayList<AddEditPresentationDialog.Kind>();
		if (fAttribute != null) {
			Map<String,KindAttributeDescriptor> kinds = PresentationsManager.getPossibleKinds(fAttribute.getAttributeType());
			String defaultK= AttributeTypePresentationIdBindingManager.getAttributeTypePresentationId(fAttribute.getAttributeType());
			AddEditPresentationDialog.Kind defaultKind= null;
			Set<String> kindsSet = kinds.keySet(); 
			for (AddEditPresentationDialog.Kind k: fKinds) {
				if (kindsSet.contains(k.getId())) {
					if(!kinds.get(k.getId()).isDeprecated()){
						values.add(k);
					}
					if (defaultK != null && defaultK.equals(k.getId())) {
						defaultKind= k;
					}
				}
			}
			values.add(new Kind(EditorPresentationManager.DO_NOT_SHOW_KIND, PresentationsManager.getDisplayName(EditorPresentationManager.DO_NOT_SHOW_KIND)));
			fAKindsCombo.setValueSet(values.toArray(new AddEditPresentationDialog.Kind[values.size()]));
			if (!values.contains(fAKindsCombo.getValue())) {
				Object val = fAKindsCombo.getValue();
				if(val instanceof AddEditPresentationDialog.Kind){
					AddEditPresentationDialog.Kind kind = (AddEditPresentationDialog.Kind)val;
					KindAttributeDescriptor desc = kinds.get(kind.getId());
					if(desc == null || !desc.isDeprecated()){
						if (defaultKind != null) {
							fAKindsCombo.setValue(defaultKind);
						} else {
							fAKindsCombo.setValue(EditorPresentationAspectPart.NONE);
						}
					}
				}
			}
			fAKindsCombo.getCombo().setEnabled(!values.isEmpty());
		} else {
			fAKindsCombo.getCombo().setEnabled(false);
			fAKindsCombo.setValue(EditorPresentationAspectPart.NONE);
		}
	}
	
	@Override
	protected void createIconDialogPart(Composite parent) {
		// empty
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		Control area = super.createDialogArea(parent);
		if (fKind != null || fAttribute != null || fId != null) {
			validate();
		}
		return area;
	}
	
	@Override
	public boolean close() {
		fToolbarManager.dispose();
		return super.close();
	}
	
	public static class Kind {
		private String fId;
		private String fName;
		public Kind(String id, String name) {
			fId= id;
			fName= name;
		}
		public String getId() {
			return fId;
		}
		public String getName() {
			return fName;
		}
		public boolean equals(Object obj) {
			if (obj == null || obj.getClass() != getClass())
				return false;
			return fId.equals(((Kind)obj).fId);
		}
		public int hashCode() {
			return fId.hashCode();
		}
	}
	
	private static final String CONFIGURATIONLET_EXTENSION_POINT_ID = "com.ibm.team.workitem.ide.ui.presentationConfigurationlet";  //$NON-NLS-1$
	private static final String CLASS= "class";  //$NON-NLS-1$
	
	public static Configurationlet getConfigurationlet(String kindId) {
		if (kindId != null) {
			IExtensionRegistry registry = Platform.getExtensionRegistry();
			IConfigurationElement[] elements= registry.getConfigurationElementsFor(CONFIGURATIONLET_EXTENSION_POINT_ID);
			try {
				for (IConfigurationElement element: elements) {
					if (element.getAttribute(EditorPresentationManager.KIND).equals(kindId)) {
						return (Configurationlet) element.createExecutableExtension(CLASS);
					}
				}
			} catch (Exception e) {
				WorkItemIDEUIPlugin.getDefault().log(e.getMessage(), e);
			}
		}
		return new FallbackConfigurationlet();
	}
}