/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedControl;

/**
 * Button decorated with a status icon.
 * Automatically leaves space for the icon whether it's visible or not.
 * You must set the text of the button after this is created via
 * {@link #getButton()}.
 */
public class DecoratedButton extends DecoratedControl {

	public DecoratedButton(Composite parent, int style) {
		super(parent, style, DecoratedControl.NO_OPTION);
		fStatusDecoration.setMarginWidth(1);
	}

	private Button fButton;
	
	/* (non-Javadoc)
	 * @see com.ibm.team.workitem.ide.ui.internal.editor.DecoratedControl#createControl(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	protected Control createControl(Composite parent, int style) {
		// Create a composite just to leave space for the decoration if there is one.
		final Composite container= new Composite(parent, SWT.NONE);
		final GridLayout layout= new GridLayout(1, false);
		layout.marginHeight= 0;
		layout.marginWidth= 0;
		layout.marginLeft= DecoratedControl.DECORATION_WIDTH;
		container.setLayout(layout);
		container.setBackgroundMode(SWT.INHERIT_FORCE);
		fButton = new Button(container, style);
		return container;
	}
	
	@Override
	protected Control getDecoratedControl() {
		return fButton;
	}

	/**
	 * Gets the underlying button control.
	 * @return
	 * 	the button.
	 */
	public Button getButton() {
		return fButton;
	}
	
	@Override
	protected boolean isSet() {
		return fButton.getText().length() > 0 || fButton.getImage() != null;
	}

	/**
	 * Not used.
	 */
	public ISelection getSelection() {
		throw new UnsupportedOperationException();
	}

	/**
	 * Not used.
	 */
	public void setSelection(ISelection selection) {
		throw new UnsupportedOperationException();
	}

}
