/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import java.util.Collections;
import java.util.List;

import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ModeledElement;

public class EditorIdBinding extends ModeledElement {

	private ReadOnlyType fType;
	private String fEditorId;
	private String fTarget;
	
	public EditorIdBinding(String target, ReadOnlyType type, String editorId) {
		fType= type;
		fEditorId= editorId;
		fTarget= target;
	}
	
	@Override
	public String getIdentifier() {
		return fType != null? fType.getId():null;
	}
	
	@Override
	public String getName() {
		return fType != null? fType.getName():super.getName();
	}
	
	public String getTarget() {
		return fTarget;
	}
	
	public void setEditorId(String editorId) {
		fEditorId= editorId;
	}

	public ReadOnlyType getType() {
		return fType;
	}
	
	public String getEditorId() {
		return fEditorId;
	}
	
	/**
	 * Represents a type associated with an editor binding, but is immutable since this
	 * is used for reference only, but is never saved back to the process XML.
	 */
	public static class ReadOnlyType
		extends ModeledElement {
		final String fId;
		final String fName;
		final String fIconUrl;
		final List<String> fAliases;
		
		public ReadOnlyType(String id, String name) {
			this(id, name, null, Collections.EMPTY_LIST);
		}
		
		public ReadOnlyType(String id, String name, String iconUrl, List<String> aliases) {
			super();
			fId= id;
			fName= name;
			fIconUrl= iconUrl;
			fAliases= aliases;
		}

		public List<String> getAliases() {
			return fAliases;
		}

		public String getId() {
			return fId;
		}
		
		@Override
		public String getIdentifier() {
			return getId();
		}
		
		@Override
		public String getName() {
			return fName;
		}

		public String getIconUrl() {
			return fIconUrl;
		}
	}
}
