/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.client.workingcopies.IProcessContainerWorkingCopy;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessDefinition;
import com.ibm.team.process.common.IProcessItem;
import com.ibm.team.process.common.IProcessItemHandle;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.process.ide.ui.ProcessAspect;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.ui.editor.TeamFormPart;
import com.ibm.team.ui.editor.TeamFormSectionPart;
import com.ibm.team.workitem.common.internal.model.WorkItemAttributes;
import com.ibm.team.workitem.common.internal.presentations.AbstractPresentationDescriptor;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.common.internal.presentations.SectionDescriptor;
import com.ibm.team.workitem.common.internal.presentations.TabDescriptor;
import com.ibm.team.workitem.common.model.AttributeDescriptor;
import com.ibm.team.workitem.common.model.AttributeIdentifiers;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.Identifier;
import com.ibm.team.workitem.ide.ui.internal.HelpContextIds;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AbstractWorkItemAspectEditor;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ModeledElement;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ProcessAttachmentIconSelectionDialog.ModeledElementIdSelectionDialog;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorIdBinding.ReadOnlyType;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorPresentation.Attribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.AttributesUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeCategory;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeCategory.CustomAttribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeManager;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.valueprovider.IProcessConfigurationDataProvider;

public class EditorPresentationAspectEditor extends AbstractWorkItemAspectEditor implements IProcessConfigurationDataProvider {

	private static final String EDITOR_ID_BINDINGS= "com.ibm.team.workitem.editor.configuration.workitemTypeEditorIdBinding"; //$NON-NLS-1$
	private static final IAspectMessageProvider MESSAGE_PROVIDER = new IAspectMessageProvider() {
		public String getChooseMessage() {
			return Messages.EditorPresentationAspectEditor_CHOOSE_EDITOR_PRESENTATION;
		}
		public String getSameIdMessage() {
			return Messages.EditorPresentationAspectEditor_ID_EXISTS;
		}
		public String getEnterIdMessage() {
			return Messages.EditorPresentationAspectEditor_ENTER_PRESENTATION_ID;
		}
	};
	private EditorPresentationAspectPart fEPart;
	
	private Map<String, List<AbstractPresentationDescriptor>> fPresentationsMap;
	private Map<String, String> fTabLayouts;
	private List<Attribute> fAttributes;
	private List<String> fSectionIds;
	private List<String> fTabIds;
	private List<String> fEditorIds;

	private List<EditorIdBinding> fBindings;

	private ManagedForm fManagedForm;

	public EditorPresentationAspectEditor(String dataId) {
		super(dataId);
		setHelpContextId(HelpContextIds.EDITOR_PRESENTATION_ASPECT_EDITOR);
	}

	@Override
	protected void commit(List<? extends ModeledElement> elements, IMemento memento) {
		EditorPresentationAspectManager.writeEditorPresentations(memento, (List<EditorPresentation>)elements, fPresentationsMap, fTabLayouts, fTabIds, fSectionIds);
	}

	@Override
	protected void createEditorArea(Composite parent, FormToolkit toolkit) {
		GridLayoutFactory.fillDefaults().applyTo(parent);
		fManagedForm = new ManagedForm(toolkit, toolkit.createScrolledForm(parent));
		ScrolledForm form= fManagedForm.getForm();
		GridDataFactory.fillDefaults().grab(true, true).hint(300, 200).applyTo(form);
		
		Composite body= form.getBody();
		FormLayout layout= new FormLayout();
		layout.spacing= 5;
		body.setLayout(layout);
		
		fEPart = new EditorPresentationAspectPart(this, this);
		TeamFormSectionPart editorPresentationSection = new TeamFormSectionPart(fManagedForm, ExpandableComposite.TITLE_BAR, Messages.EditorPresentationAspectEditor_EDITOR_PRESENTATION, new TeamFormPart[]{fEPart} );
		FormData formData= new FormData();
		formData.left= new FormAttachment(0);
		formData.right= new FormAttachment(100);
		formData.top= new FormAttachment(0);
		formData.bottom= new FormAttachment(100);
		editorPresentationSection.getSection().setLayoutData(formData);
		fManagedForm.addPart(editorPresentationSection);
		fEPart.setRequiredListsAndMaps(fPresentationsMap, fEditorIds, fTabIds, fTabLayouts, fSectionIds, fAttributes);
	}
	
	public ModelElement getConfigurationData(String dataId) {
		return getSite().getConfigurationData(dataId);
	}

	@Override
	protected ModeledElement createNewElement() {
		ModeledElementIdSelectionDialog d= new ModeledElementIdSelectionDialog(Display.getDefault().getActiveShell(), Messages.EditorPresentationAspectEditor_CREATE_NEW_EDITOR_DIALOG_TITLE, getMessageProvider(), getAllElements(), getResourceManager(),false);
		if (d.open() != Window.OK) {
			return null;
		}
		return new EditorPresentation(d.getId());
	}
	
	@Override
	protected boolean canDuplicate() {
		return true;
	}
	
	@Override
	protected IAspectMessageProvider getMessageProvider() {
		return MESSAGE_PROVIDER;
	}
	
	@Override
	protected ModeledElement duplicateElement(ModeledElement original) {
		ModeledElementIdSelectionDialog d= new ModeledElementIdSelectionDialog(Display.getDefault().getActiveShell(), Messages.EditorPresentationAspectEditor_DUPLICATE_EDITOR_DIALOG_TITLE, getMessageProvider(), getAllElements(), getResourceManager(), false);
		if (d.open() != Window.OK) {
			return null;
		}
		EditorPresentation pres= new EditorPresentation(d.getId());
		fEditorIds.add(pres.getIdentifier());
		if (fPresentationsMap.get(pres.getIdentifier()) == null) {
			fPresentationsMap.put(pres.getIdentifier(), new ArrayList<AbstractPresentationDescriptor>());
		}
		List<AbstractPresentationDescriptor> toDuplicate= fPresentationsMap.get(original.getIdentifier());
		List<AbstractPresentationDescriptor> toFill= fPresentationsMap.get(pres.getIdentifier());
		if (toDuplicate != null) {
			for (AbstractPresentationDescriptor desc: toDuplicate) {
				if (desc instanceof TabDescriptor) {
					toFill.add(desc);
				}
			}
		}
		fEPart.initializeEditorDependency(pres.getIdentifier());
		return pres;
	}
	
	@Override
	protected boolean canRemove() {
		return true;
	}
	
	@Override
	protected boolean removeElement(ModeledElement element) {
		String types= ""; //$NON-NLS-1$
		for (EditorIdBinding binding: fBindings) {
			if (binding.getEditorId().equals(element.getIdentifier())) {
				types = NLS.bind(Messages.EditorPresentationAspectEditor_APPEND_TYPE_TO_LIST, types, binding.getType().getName());
			}
		}
		String message;
		if (types.length() == 0) {
			if (EditorPresentationManager.DEFAULT_EDITOR.equals(element.getIdentifier())) {
				message= Messages.EditorPresentationAspectEditor_REMOVE_DEFAULT_EDITOR;
			} else {
				message= NLS.bind(Messages.EditorPresentationAspectEditor_REMOVE_EDITOR, element.getName());
			}
		} else {
			if (EditorPresentationManager.DEFAULT_EDITOR.equals(element.getIdentifier())) {
				message= NLS.bind(Messages.EditorPresentationAspectEditor_REMOVING_REFERENCED_DEFAULT_EDITOR, element.getName(), types);
			} else {
				message= NLS.bind(Messages.EditorPresentationAspectEditor_REMOVING_REFERENCED_EDITOR, element.getName(), types);
			}
		}
		if (!MessageDialog.openConfirm(Display.getDefault().getActiveShell(), Messages.EditorPresentationAspectEditor_REMOVE, message)) {
			return false;
		}
		fEPart.cleanUpEditorDependency(element.getIdentifier());
		List<AbstractPresentationDescriptor> toCheck= fPresentationsMap.get(element.getIdentifier());
		if (toCheck != null) {
			for (Iterator<AbstractPresentationDescriptor> it= toCheck.iterator(); it.hasNext();) {
				AbstractPresentationDescriptor a= it.next();
				if (a instanceof SectionDescriptor) {
					it.remove();
				}
			}
		}
		fEditorIds.remove(element.getIdentifier());
		return true;
	}

	@Override
	protected void doDispose() {
		if (fManagedForm != null) {
			fManagedForm.dispose();
			fManagedForm= null;
		}
	}

	/**
	 * Set selection in Editor Presentation viewer
	 * @param ids
	 */
	public void selectInTree(TreeSelection selection){
		if(fEPart != null){
			fEPart.select(selection);
		}
	}
	
	
	public Map<String, Set<String>> getEditorDependencyCache() {
		if(fEPart != null){
			return fEPart.getEditorDependencyCache();
		}
		return null;
	}
	
	public Map<String, Set<TabDescriptor>> getSectionDependencyCache() {
		if(fEPart != null){
			return fEPart.getSectionDependencyCache();
		}
		return null;
	}
	
	public Map<String, List<AbstractPresentationDescriptor>> getPresentationsMap() {
		if(fEPart != null){
			return fEPart.getPresentationsMap();
		}
		return null;
	}

	@Override
	protected void inputChanged(ModeledElement newElement) {
		fManagedForm.setInput(newElement);
	}
	
	private List<CustomAttribute> readCustomAttributes() {
		List<TypeCategory> categories= TypeManager.readTypeCategories(getSite().getConfigurationData(TypeManager.CONFIG_ID));
		List<CustomAttribute> attributes= new ArrayList<CustomAttribute>();
		for (TypeCategory cat: categories) {
			attributes.addAll(cat.getCustomAttributes());
		}
		return attributes;
	}
	
	private List<EditorIdBinding> readBindings() {
		List<ReadOnlyType> types= WorkitemTypeEditorIdBindingManager.readWorkItemTypes(getSite().getConfigurationData(TypeManager.CONFIG_ID));
		return WorkitemTypeEditorIdBindingManager.readEditorIdBindings(getSite().getConfigurationData(EDITOR_ID_BINDINGS), types);
	}

	@Override
	protected List<? extends ModeledElement> readElements(final ModelElement configurationElement) {
		Map<String/*id*/, List<AbstractPresentationDescriptor>> definitions = new HashMap<String, List<AbstractPresentationDescriptor>>();
		Map<String/*tab id*/, String /*tab layout*/> tabLayouts= new HashMap<String, String>();
		List<String> editorIds= new ArrayList<String>();
		List<String> tabIds= new ArrayList<String>();
		List<String> sectionIds= new ArrayList<String>();
		if (configurationElement != null) {
			EditorPresentationManager.readPresentations(configurationElement.getChildElements(), definitions, tabLayouts, editorIds, tabIds, sectionIds);
		}
		fPresentationsMap= definitions;
		fTabLayouts= tabLayouts;
		fSectionIds= sectionIds;
		fTabIds= tabIds;
		fEditorIds= editorIds;
		final IProcessItem processItem = getProcessContainerWorkingCopy().getUnderlyingProcessItem();
		if (processItem instanceof IProcessArea) {
			final Set<CustomAttribute> repoAttrs = AttributesUtil.computeRepositoryAttributes(getAspect());
			fAttributes= new ArrayList<Attribute>();
			Set<Identifier<IAttribute>> attributeIdentifiers= new HashSet<Identifier<IAttribute>>();
			for (CustomAttribute attribute: repoAttrs) {
				if (attribute.isInternal()) {
					continue;
				}
				Identifier<IAttribute> ident = WorkItemAttributes.getPropertyIdentifier(attribute.getId());
				fAttributes.add(new Attribute(ident, attribute.getAttributeType(), attribute.getName(), attribute.isCustomAttribute(), false));
				attributeIdentifiers.add(ident);
				
			}
			// read in custom attributes created in type aspect editor
			List<CustomAttribute> customAttributes= readCustomAttributes();
			for (CustomAttribute a: customAttributes) {
				Identifier<IAttribute> ident= WorkItemAttributes.getPropertyIdentifier(a.getId());
				if (!attributeIdentifiers.contains(ident)) {
					fAttributes.add(new Attribute(ident, a.getAttributeType(), a.getName(), true, true));
				} else {
					for (Attribute attr: fAttributes) {
						if (attr.getIdentifier().equals(ident)) {
							attr.setConfiguredInSpec(true);
							break;
						}
					}
				}
			}
		} else if (processItem instanceof IProcessDefinition) {
			fAttributes= new ArrayList<Attribute>();
			for (AttributeDescriptor desc: AttributeIdentifiers.ATTRIBUTE_DESCRIPTORS) {
				if (desc.isInternal()) {
					continue;
				}
				fAttributes.add(new Attribute(WorkItemAttributes.getPropertyIdentifier(desc.getIdentifier()), desc.getAttributeType(), desc.getDisplayName(), false, false));
			}
			List<CustomAttribute> customAttributes= readCustomAttributes();
			for (CustomAttribute a: customAttributes) {
				fAttributes.add(new Attribute(WorkItemAttributes.getPropertyIdentifier(a.getId()), a.getAttributeType(), a.getName(), true, true));
			}
		} else {
			throw new UnsupportedOperationException(Messages.EditorPresentationAspectEditor_NEITHER_AREA_NOR_DEFINITION);
		}
		List<EditorPresentation> elements= new ArrayList<EditorPresentation>();
		for (String id: editorIds) {
			elements.add(new EditorPresentation(id));
		}
		if (fEPart != null) {
			fEPart.setRequiredListsAndMaps(fPresentationsMap, fEditorIds, fTabIds, fTabLayouts, fSectionIds, fAttributes);
		}
		fBindings = readBindings();
		return elements;
	}

	public ProcessAspect getProcessAspect() {
		return getAspect();
	}

	public IProcessItemHandle getProcessItem() {
		ProcessAspect aspect= getAspect();
		if (aspect != null) {
			IProcessContainerWorkingCopy processContainer= aspect.getProcessContainerWorkingCopy();
			if (processContainer != null)
				return processContainer.getUnderlyingProcessItem();
		}
		return null;
	}
}