/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import java.util.List;
import java.util.Map;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.workitem.common.internal.presentations.AbstractPresentationDescriptor;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.common.internal.presentations.PresentationDescriptor;
import com.ibm.team.workitem.common.internal.presentations.SectionDescriptor;
import com.ibm.team.workitem.common.internal.presentations.TabDescriptor;

public class EditorPresentationAspectManager {

	public static void writeEditorPresentations(IMemento memento, List<EditorPresentation> elements, Map<String, List<AbstractPresentationDescriptor>> map, Map<String, String> tabLayouts, List<String> tabIds, List<String> sectionIds) {
		// write editors with tab references
		for (EditorPresentation p: elements) {
			writeEditor(memento, p, map);
		}
		// for each key in the map: write the declaration with the list of references
		for (String key: map.keySet()) {
			List<AbstractPresentationDescriptor> descriptors= map.get(key);
			if (sectionIds.contains(key)) {
				writeSection(memento, key, descriptors);
			}
			if (tabIds.contains(key)) {
				writeTab(memento, key, descriptors, tabLayouts);
			}
		}
	}
	
	private static void writeEditor(IMemento memento, EditorPresentation p, Map<String, List<AbstractPresentationDescriptor>> map) {
		IMemento editor= memento.createChild(EditorPresentationManager.EDITOR);
		editor.putString(EditorPresentationManager.ID, p.getIdentifier());
		List<AbstractPresentationDescriptor> descriptors= map.get(p.getIdentifier());
		if (descriptors != null) {
			for (AbstractPresentationDescriptor d: descriptors) {
				if (d instanceof TabDescriptor) {
					TabDescriptor desc= (TabDescriptor)d;
					IMemento tab= editor.createChild(EditorPresentationManager.TAB);
					tab.putString(EditorPresentationManager.TAB_ID, desc.getElementId());
					tab.putString(EditorPresentationManager.TITLE, desc.getTitle());
					writeProperties(desc, tab);
				}
			}
		}
	}
	
	private static void writeTab(IMemento memento, String tabId, List<AbstractPresentationDescriptor> descriptors, Map<String, String> tabLayouts) {
		IMemento tab= memento.createChild(EditorPresentationManager.TAB);
		tab.putString(EditorPresentationManager.ID, tabId);
		tab.putString(EditorPresentationManager.LAYOUT, tabLayouts.get(tabId));
		if (descriptors != null) {
			for (AbstractPresentationDescriptor d: descriptors) {
				if (d instanceof SectionDescriptor) {
					SectionDescriptor desc= (SectionDescriptor)d;
					IMemento sec= tab.createChild(EditorPresentationManager.SECTION);
					sec.putString(EditorPresentationManager.SECTION_ID, desc.getElementId());
					sec.putString(EditorPresentationManager.TITLE, desc.getTitle());
					sec.putString(EditorPresentationManager.SLOT, desc.getSlot());
					writeProperties(desc, sec);
				}
			}
		}
	}
	
	private static void writeSection(IMemento memento, String sectionId, List<AbstractPresentationDescriptor> descriptors) {
		IMemento section= memento.createChild(EditorPresentationManager.SECTION);
		section.putString(EditorPresentationManager.ID, sectionId);
		if (descriptors != null) {
			for (AbstractPresentationDescriptor d: descriptors) {
				if (d instanceof PresentationDescriptor) {
					PresentationDescriptor desc= (PresentationDescriptor)d;
					IMemento pres= section.createChild(EditorPresentationManager.PRESENTATION);
					if (desc.getId() != null) {
						pres.putString(EditorPresentationManager.ID, desc.getId());
					}
					if (desc.getAttributeId() != null) {
						pres.putString(EditorPresentationManager.ATTRIBUTE_ID, desc.getAttributeId().getStringIdentifier());
					}
					if (desc.getKind() != null) {
						pres.putString(EditorPresentationManager.KIND, desc.getKind());
					}
					if (desc.getLabel() != null) {
						pres.putString(EditorPresentationManager.LABEL, desc.getLabel());
					}
					if (desc.getDescription() != null) {
						pres.putString(EditorPresentationManager.DESCRIPTION, desc.getDescription());
					}
					writeProperties(desc, pres);
				}
			}
		}
	}
	
	private static void writeProperties(AbstractPresentationDescriptor desc, IMemento memento) {
		Map<String, String> properties= desc.getProperties();
		if (properties == null || properties.isEmpty()) {
			return;
		}
		for (String key: properties.keySet()) {
			IMemento prop= memento.createChild(EditorPresentationManager.PROPERTY);
			prop.putString(EditorPresentationManager.KEY, key);
			prop.putString(EditorPresentationManager.VALUE, properties.get(key));
		}
	}
}
