/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2008, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.SameShellProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.accessibility.ACC;
import org.eclipse.swt.accessibility.AccessibleAdapter;
import org.eclipse.swt.accessibility.AccessibleEvent;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.jface.JazzResources;
import com.ibm.team.jface.OverlayIcon;
import com.ibm.team.jface.tooltip.TooltipSupport;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.ui.editor.TeamFormPart;
import com.ibm.team.workitem.common.internal.model.WorkItem;
import com.ibm.team.workitem.common.internal.model.WorkItemAttributes;
import com.ibm.team.workitem.common.internal.presentations.AbstractPresentationDescriptor;
import com.ibm.team.workitem.common.internal.presentations.AttributeTypePresentationIdBindingManager;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.common.internal.presentations.PresentationDescriptor;
import com.ibm.team.workitem.common.internal.presentations.QuickInformationConfigurationManager;
import com.ibm.team.workitem.common.internal.presentations.SectionDescriptor;
import com.ibm.team.workitem.common.internal.presentations.TabDescriptor;
import com.ibm.team.workitem.common.model.IAttribute;
import com.ibm.team.workitem.common.model.Identifier;
import com.ibm.team.workitem.ide.ui.internal.ImagePool;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.IDirtyStateTracker;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ProcessAttachmentIconSelectionDialog;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorIdBinding.ReadOnlyType;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorPresentation.Attribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.AddCustomAttributeWizard;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeCategory.CustomAttribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeManager;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.valueprovider.IProcessConfigurationDataProvider;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedCombo;
import com.ibm.team.workitem.ide.ui.internal.editor.WorkItemEditorTabs;
import com.ibm.team.workitem.ide.ui.internal.editor.presentations.KindAttributeDescriptor;
import com.ibm.team.workitem.ide.ui.internal.editor.presentations.PresentationPart;
import com.ibm.team.workitem.ide.ui.internal.editor.presentations.PresentationsManager;
import com.ibm.team.workitem.rcp.ui.internal.util.Utils;

public class EditorPresentationAspectPart extends TeamFormPart {

	private static final String SHAREDUNLOCKED_COLOR = "com.ibm.team.workitem.ide.ui.SHAREDUNLOCKED"; //$NON-NLS-1$
	private Map<String , List<AbstractPresentationDescriptor>> fPresentationsMap;
	/**
	 * Map&lt;sectionId, Set&lt;TabDescriptor&gt;&gt;
	 */
	private Map<String/*sectionId*/, Set<TabDescriptor>> fSectionDependencyCache= new HashMap<String, Set<TabDescriptor>>();
	/**
	 * Map&lt;tabId, Set&lt;editorId&gt;&gt;
	 */
	private Map<String/*tabId*/, Set<String/*editorId*/>> fEditorDependencyCache= new HashMap<String, Set<String>>();
	/**
	 * Set&lt;sectionId&gt;
	 */
	private Set<String/*sectionId*/> fUnlockedPresentations= new HashSet<String>();
	/**
	 * Set&lt;tabId&gt;
	 */
	private Set<String/*tabId*/> fUnlockedSections= new HashSet<String>();
	private TreeViewer fPresentationViewer;
	protected EditorPresentation fEPresentation;
	private IDirtyStateTracker fTracker;

	/**
	 * Map < SectionDescriptor, TabDescriptor ><br/>
	 * SectionDescriptor -> TabDescriptor
	 */
	protected Map<SectionDescriptor,TabDescriptor> fSectionParents= new HashMap< SectionDescriptor,TabDescriptor>();
	
	/**
	 * Map < PresentationDescriptor, SectionDescriptor ><br/>
	 * PresentationDescriptor -> SectionDescriptor
	 */
	private Map<PresentationDescriptor,SectionDescriptor> fPresentationParents= new HashMap< PresentationDescriptor,SectionDescriptor>();
	
	static final String NONE= Messages.EditorPresentationAspectPart_NONE;
	
	private static final String KEY_LABEL= Messages.EditorPresentationAspectPart_KEY_LABEL;
	private static final String VALUE_LABEL= Messages.EditorPresentationAspectPart_VALUE_LABEL;
	
	private ResourceManager fResourceManager= new LocalResourceManager(JFaceResources.getResources());
	
	private List<Attribute> fAttributes;
	private List<String> fSectionIds;
	
	/**
	 * A list of TabDescriptor ids.
	 */
	private List<String> fTabIds;
	
	/**
	 * Map < String, String > <br/>
	 * TabDescriptor id -> TabLayout
	 */
	private Map<String, String> fTabLayouts;
	private Button fAddTab;
	private List<String> fEditorIds;
	
	private boolean fShowIds;
	private boolean fIdToggle;
	private Button fIdToobleButton;
	private Button fAddHeader;
	private IProcessConfigurationDataProvider fProcessProvider;
	
	private int fEventDetail = DND.DROP_MOVE;
	
	private TabDescriptor getParentTabDescriptor(SectionDescriptor descriptor) {
		return fSectionParents.get(descriptor);
	}

	private SectionDescriptor getParentSectionDescriptor(PresentationDescriptor descriptor) {
		return fPresentationParents.get(descriptor);
	}

	private AbstractPresentationDescriptor getParentDescriptor(AbstractPresentationDescriptor childDescriptor) {
		if (childDescriptor instanceof SectionDescriptor) {
			return getParentTabDescriptor((SectionDescriptor)childDescriptor);
		}
		if (childDescriptor instanceof PresentationDescriptor) {
			return getParentSectionDescriptor((PresentationDescriptor)childDescriptor);
		}
		return null;
	}

	private Set<String> getContainingEditorPresentationIds(TabDescriptor tabDescriptor) {
		String tabDescriptorId= tabDescriptor.getElementId();
		Set<String> editorIds= fEditorDependencyCache.get(tabDescriptorId);
		if (editorIds == null) {
			editorIds= new HashSet<String>();
			fEditorDependencyCache.put(tabDescriptorId, editorIds);
		}
		return editorIds;
	}
	
	private Set<TabDescriptor> getContainingTabDescriptors(SectionDescriptor sectionDescriptor){
		String sectionDescriptorId= sectionDescriptor.getElementId();
		Set<TabDescriptor> containingTabs= fSectionDependencyCache.get(sectionDescriptorId);
		if (containingTabs == null) {
			containingTabs= new HashSet<TabDescriptor>();
			fSectionDependencyCache.put(sectionDescriptorId, containingTabs);
		}
		return containingTabs;
	}
	

	protected EditorPresentationAspectPart(IDirtyStateTracker tracker, IProcessConfigurationDataProvider processProvider) {
		fTracker= tracker;
		fProcessProvider= processProvider;
	}
	
	private void setDirty() {
		fTracker.setDirty();
	}
	
	public Map<String, Set<String>> getEditorDependencyCache() {
		return fEditorDependencyCache;
	}
	
	public Map<String, Set<TabDescriptor>> getSectionDependencyCache() {
		return fSectionDependencyCache;
	}
	
	public Map<String, List<AbstractPresentationDescriptor>> getPresentationsMap() {
		return fPresentationsMap;
	}
	
	@Override
	public void createContent(Composite parent) {
		GridLayout layout = new GridLayout(2, false);
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		layout.verticalSpacing= 0;
		parent.setLayout(layout);
		FormToolkit toolkit= getSite().getToolkit();
		Composite viewer= toolkit.createComposite(parent, SWT.NONE);
		viewer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		Composite buttonBar= toolkit.createComposite(parent, SWT.NONE);
		GridData data= new GridData(SWT.LEFT, SWT.BEGINNING, false, false);
		buttonBar.setLayoutData(data);
		createViewer(viewer, buttonBar);
		addDragAndDropSupport();
		addMouseListenerSupport();
		addKeyListenerSupport();
	}
	
	private static IStatus isValid(String kind, Attribute attribute) {
		if (kind != null && attribute != null) {
			List<KindAttributeDescriptor> supportedTypeDesc= PresentationsManager.getAttributeTypes(kind);
			if (!supportedTypeDesc.isEmpty()) {
				boolean supported= false;
				boolean deprecated= false;
				for (KindAttributeDescriptor desc: supportedTypeDesc) {
					if (PresentationsManager.isValidForAttribute(desc.getAttributeType(), attribute.getAttributeType())) {
						supported= true;
						deprecated= desc.isDeprecated();
					}
				}
				if (!supported || deprecated) {
					String attributeTypeNames= getSupportedTypeNames(supportedTypeDesc);
					String name= PresentationsManager.getDisplayName(kind);
					String message= NLS.bind(Messages.EditorPresentationAspectPart_KIND_DOES_NOT_SUPPORT_ATTRIBUTE, name, attribute.getDisplayName(), attribute.getAttributeType(), attributeTypeNames);
					return new Status(!supported ? IStatus.ERROR : IStatus.WARNING, WorkItemIDEUIPlugin.PLUGIN_ID, message);
				}
				
			}
			if (!PresentationsManager.needsAttribute(kind)) {
				String name= PresentationsManager.getDisplayName(kind);
				return new Status(IStatus.INFO, WorkItemIDEUIPlugin.PLUGIN_ID, NLS.bind(Messages.EditorPresentationAspectPart_KIND_DOES_NOT_NEED_ATTRIBUTE, name));
			}
		}
		if (kind == null && attribute != null) {
			String aKind= AttributeTypePresentationIdBindingManager.getAttributeTypePresentationId(attribute.getAttributeType());
			if (aKind != null) {
				String name= PresentationsManager.getDisplayName(aKind);
				return new Status(IStatus.INFO, WorkItemIDEUIPlugin.PLUGIN_ID, NLS.bind(Messages.EditorPresentationAspectPart_ATTRIBUTE_USES_DEFAULT_KIND, attribute.getDisplayName(), name));
			} else {
				Map<String,KindAttributeDescriptor> possibleKinds= PresentationsManager.getPossibleKinds(attribute.getAttributeType());
				String kindNames= ""; //$NON-NLS-1$
				for (KindAttributeDescriptor k: possibleKinds.values()) {
					if(!kindNames.equals("")){ //$NON-NLS-1$
						kindNames= NLS.bind(Messages.EditorPresentationAspectPart_LIST_POSSIBLE_KINDS, kindNames, k.getPresentationName());
					} else {
						kindNames = k.getPresentationName();
					}
				}
				String message= NLS.bind(Messages.EditorPresentationAspectPart_ATTRIBUTE_NO_DEFAULT_KIND, attribute.getDisplayName(), kindNames);
				return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, message);
			}
		}
		if (kind != null && attribute == null) {
			if (PresentationsManager.needsAttribute(kind)) {
				String name= PresentationsManager.getDisplayName(kind);
				List<KindAttributeDescriptor> supportedTypeDesc= PresentationsManager.getAttributeTypes(kind);
				String attributeTypeNames= getSupportedTypeNames(supportedTypeDesc);
				String message= NLS.bind(Messages.EditorPresentationAspectPart_KIND_NEEDS_ATTRIBUTE_OF_TYPE, name, attributeTypeNames);
				return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, message);
			}
		}
		return Status.OK_STATUS;
	}
	
	private static String getSupportedTypeNames(Collection<KindAttributeDescriptor> supportedTypeDesc) {
		String attributeTypeNames= ""; //$NON-NLS-1$
		for (KindAttributeDescriptor desc: supportedTypeDesc) {
			if (!desc.isDeprecated()) {
				if(!attributeTypeNames.equals("")){ //$NON-NLS-1$
					attributeTypeNames = NLS.bind(Messages.EditorPresentationAspectPart_LIST_ATTRIBUTE_NAMES, attributeTypeNames, desc.getAttributeType());
				} else {
					attributeTypeNames = desc.getAttributeType();
				}
			}
		}
		return attributeTypeNames;
	}
	
	/**
	 * Set selection in Editor Presentation viewer
	 * @param selection
	 */
	public void select(TreeSelection selection){
		if(fPresentationViewer != null){
			fPresentationViewer.setSelection(selection, true);
		}	
	}

	private void createViewer(Composite viewer, Composite buttonBar) {
		FormToolkit toolkit= getSite().getToolkit();
		GridLayout layout= new GridLayout();
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		viewer.setLayout(layout);
		Tree t= new Tree(viewer, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
		GridData data= new GridData(SWT.FILL, SWT.FILL, true, true);
		data.minimumHeight= t.getItemHeight()*6;
		data.heightHint= t.getItemHeight()*12;
		t.setLayoutData(data);
		t.getAccessible().addAccessibleListener(new AccessibleAdapter() {
			@Override
			public void getName(AccessibleEvent e) {
				if(e.childID == ACC.CHILDID_SELF){
					e.result = Messages.EditorPresentationAspectEditor_EDITOR_PRESENTATION; 
				}
			}
		});
		fPresentationViewer= new TreeViewer(t);
		fPresentationViewer.setLabelProvider(new CellLabelProvider() {
			@Override
			public void update(ViewerCell cell) {
				Object element= cell.getElement();
				if (element instanceof TabDescriptor) {
					TabDescriptor desc= (TabDescriptor)element;
					if (isHeaderTab(desc)) {
						cell.setText(Messages.EditorPresentationAspectPart_EDITOR_HEADER_LABEL);
						cell.setImage(JazzResources.getImage(fResourceManager, ImagePool.HEADER, null));
						cell.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW));
						return;
					}
					String text= desc.getTitle();
					String id= ""; //$NON-NLS-1$
					if (showIds()) {
						id= NLS.bind(Messages.EditorPresentationAspectPart_ELEMENT_ID, desc.getElementId());
					}
					String shared= ""; //$NON-NLS-1$
					if (isAShared(desc)) {
						shared= NLS.bind(Messages.EditorPresentationAspectPart_CONTENT_SHARED_SUFFIX, Messages.EditorPresentationAspectPart_CONTENT_SHARED);
					}
					cell.setText(NLS.bind(Messages.EditorPresentationAspectPart_TEXT_ID_SHARED, text, id, shared));
					cell.setImage(JazzResources.getImage(fResourceManager, ImagePool.TAB, null));
					cell.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_LIST_FOREGROUND));
				}
				if (element instanceof SectionDescriptor) {
					SectionDescriptor desc= (SectionDescriptor)element;
					TabDescriptor tab= getParentTabDescriptor(desc);
					boolean isHeader= isHeaderTab(tab);
					String title;
					if (isHeader) {
						title= Messages.EditorPresentationAspectPart_HEADER_SECTION_LABEL;
					} else {
						title= desc.getTitle();
					}
					String descr= ""; //$NON-NLS-1$
					if (!isHeader) {
						if (showIds()) {
							descr= NLS.bind(Messages.EditorPresentationAspectPart_DESCRIPTION_LIST_ENTRY, Messages.EditorPresentationAspectPart_LAYOUT_SLOT, (desc.getSlot()!= null?WorkItemEditorTabs.getSlotName(desc.getSlot()):Messages.EditorPresentationAspectPart_NONE_ENTRY), desc.getElementId());
						} else {
							descr= NLS.bind(Messages.EditorPresentationAspectPart_SLOT_NAME, Messages.EditorPresentationAspectPart_LAYOUT_SLOT, (desc.getSlot()!= null?WorkItemEditorTabs.getSlotName(desc.getSlot()):Messages.EditorPresentationAspectPart_NONE_ENTRY));
						}
					} else {
						if (showIds()) {
							descr= NLS.bind(Messages.EditorPresentationAspectPart_ELEMENT_ID_LIST_ENTRY, desc.getElementId());
						}
					}
					String shared= ""; //$NON-NLS-1$
					if (isAShared(desc)) {
						shared= NLS.bind(Messages.EditorPresentationAspectPart_CONTENT_SHARED_SUFFIX, Messages.EditorPresentationAspectPart_CONTENT_SHARED);
					}
					if ("".equals(descr) && !"".equals(shared)) { //$NON-NLS-1$ //$NON-NLS-2$
						shared= shared.substring(1);
					}
					cell.setText(NLS.bind(Messages.EditorPresentationAspectPart_TITLE_DESCRIPTION_SHARED, title, descr, shared));
					cell.setImage(JazzResources.getImage(fResourceManager, ImagePool.SECTION, null));
					if (isHeader) {
						cell.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW));
					} else {
						cell.setForeground(getCellForeground(desc));
					}
				}
				
				if (element instanceof PresentationDescriptor) {
					PresentationDescriptor desc= (PresentationDescriptor)element;
					String kind= desc.getKind();
					Attribute attribute = findAttribute(desc);
					if (kind == null) {
						if (attribute != null) {
							kind= AttributeTypePresentationIdBindingManager.getAttributeTypePresentationId(attribute.getAttributeType());
							desc.setKind(kind);
						}
					}
					String attrName= ""; //$NON-NLS-1$
					String kindName= ""; //$NON-NLS-1$
					String label= ""; //$NON-NLS-1$
					String endPointName = ""; //$NON-NLS-1$
					if (desc.getLabel() != null) {
						label= NLS.bind(Messages.EditorPresentationAspectPart_LABEL, desc.getLabel());
					}
					if (attribute != null) {
						attrName= attribute.getDisplayName();
						if (kind != null) {
							kindName= NLS.bind(Messages.EditorPresentationAspectPart_DISPLAY_NAME, PresentationsManager.getDisplayName(kind));
						}
						
					} else if (kind != null) {
						kindName= PresentationsManager.getDisplayName(kind);
					} else {
						kindName= Messages.EditorPresentationAspectPart_UNKNOWN_ENTRY;
					}

					if (kindName.length() > 0) {
						// If we have a kind name, then also show the end point name. This is only relevant
						// is the kind is a reference (4/16/10 jgillibr-I'm assuming only reference have an endpointId otherwise
						// this needs to test against the name of the kind, but that info comes from the plugin.xml and
						// I'm not sure how to leverage that here).
						String endPointXml = desc.getProperties().get(QuickInformationConfigurationManager.ENDPOINTID);
						if (endPointXml != null) {
							final String endPointDisplayName = AspectEditorUtil.getEndPointXMLStringsToDisplayNames().get(endPointXml);
							if (endPointDisplayName != null) {
								endPointName = NLS.bind(Messages.EditorPresentationAspectPart_DISPLAY_NAME, endPointDisplayName);
							}
						}
					}
					
					String id= ""; //$NON-NLS-1$
					if (showIds() && desc.getId() != null) {
						id= NLS.bind(Messages.EditorPresentationAspectPart_PART_ID, Messages.EditorPresentationAspectPart_ID, desc.getId());
					}
					String descr= ""; //$NON-NLS-1$
					if (desc.getDescription() != null) {
						descr= NLS.bind(Messages.EditorPresentationAspectPart_DESCRIPTION, desc.getDescription());
					}
					cell.setText(NLS.bind(Messages.EditorPresentationAspectPart_LABEL_NAME_KIND_ENDPOINT_ID_DESCRIPTION, label, attrName, kindName, endPointName, id, descr));
					if (desc.getKind() != null && (desc.getKind().equals(EditorPresentationManager.SEPARATOR_KIND) || desc.getKind().equals(EditorPresentationManager.LINE_SEPARATOR_KIND))) {
						cell.setImage(JazzResources.getImage(fResourceManager, ImagePool.SEPARATOR, null));
					} else {
						IStatus stat= isValid(desc.getKind(), attribute);
						if (stat.getSeverity() == IStatus.ERROR) {
							ImageDescriptor overlayed= new OverlayIcon(ImagePool.PRESENTATION, new ImageDescriptor[]{null, ImagePool.ERROR_OVR}, SWT.BOTTOM | SWT.LEFT);
							cell.setImage(JazzResources.getImage(fResourceManager, overlayed, null));
						} else if (stat.getSeverity() == IStatus.WARNING) {
							ImageDescriptor overlayed= new OverlayIcon(ImagePool.PRESENTATION, new ImageDescriptor[]{null, ImagePool.WARNING_OVR}, SWT.BOTTOM | SWT.LEFT);
							cell.setImage(JazzResources.getImage(fResourceManager, overlayed, null));
						} else {
							cell.setImage(JazzResources.getImage(fResourceManager, ImagePool.PRESENTATION, null));
						}
					}
					cell.setForeground(getCellForeground(desc));
				}
			}
		});
		fPresentationViewer.setContentProvider(new ITreeContentProvider() {
			public Object[] getChildren(Object parentElement) {
				if (parentElement instanceof TabDescriptor) {
					TabDescriptor desc= (TabDescriptor)parentElement;
					List<SectionDescriptor> children= new ArrayList<SectionDescriptor>();
					List<AbstractPresentationDescriptor> list = fPresentationsMap.get(desc.getElementId());
					if (list != null) {
						for (AbstractPresentationDescriptor d: list) {
							if (d instanceof SectionDescriptor) {
								children.add((SectionDescriptor)d);
								fSectionParents.put((SectionDescriptor)d, desc); // SectionDescriptor -> TabDescriptor
							}
						}
					}
					return children.toArray();
				}
				if (parentElement instanceof SectionDescriptor) {
					SectionDescriptor desc= (SectionDescriptor)parentElement;
					List<PresentationDescriptor> children= new ArrayList<PresentationDescriptor>();
					List<AbstractPresentationDescriptor> list = fPresentationsMap.get(desc.getElementId());
					if (list != null) {
						for (AbstractPresentationDescriptor d: list) {
							if (d instanceof PresentationDescriptor) {
								children.add((PresentationDescriptor)d);
								fPresentationParents.put((PresentationDescriptor)d, desc); // PresentationDescriptor -> SectionDescriptor
							}
						}
					}
					return children.toArray();
				}
				return new Object[0];
			}

			public Object getParent(Object element) {
				
				return null;
			}

			public boolean hasChildren(Object element) {
				return getChildren(element).length > 0;
			}

			public Object[] getElements(Object inputElement) {
				List<TabDescriptor> elements= new ArrayList<TabDescriptor>();
				List<AbstractPresentationDescriptor> list = fPresentationsMap.get(fEPresentation.getIdentifier());
				if (list != null) {
					for (AbstractPresentationDescriptor d: list) {
						if (d instanceof TabDescriptor) {
							elements.add((TabDescriptor)d);
						}
					}
				}
				if(elements.size() == 0){
					//Set focus to Add Header button when Presentation is empty to help users get started.
					Display.getDefault().asyncExec(new Runnable(){
						public void run() {
							fAddHeader.setFocus();
						}
					});
					
				}
				return elements.toArray();
			}

			public void dispose() {
			}

			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		});
		new TooltipSupport(fPresentationViewer.getControl(), false, false) {
			@Override
			protected Object mapElement(int x, int y) {
				TreeItem item = fPresentationViewer.getTree().getItem(new Point(x, y));
				if (item != null) {
					Object data = item.getData();
					if (data instanceof PresentationDescriptor) {
						return data;
					}
				}
				return null;
			}
			
			@Override
			protected String getMarkup(Object element, boolean isFocus) {
				if (element instanceof PresentationDescriptor) {
					IStatus status= isValid(((PresentationDescriptor) element).getKind(), findAttribute(((PresentationDescriptor) element)));
					if (status.getSeverity() == IStatus.ERROR || status.getSeverity() == IStatus.WARNING) {
						return status.getMessage();
					}
				}
				return null;
			}
		};
		layout= new GridLayout();
		buttonBar.setLayout(layout);

		final Button editEntry = toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_EDIT, SWT.FLAT);
		editEntry.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		editEntry.setEnabled(false);
		editEntry.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				editEntry();
			}
		});
		AspectEditorUtil.createFiller(buttonBar, toolkit, 0);
		final Button addPresentation = toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_ADD_PRESENTATION, SWT.FLAT);
		addPresentation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		addPresentation.setEnabled(false);
		addPresentation.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				addPresentation();
			}
		});
		final Button addSection = toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_ADD_SECTION, SWT.FLAT);
		addSection.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		addSection.setEnabled(false);
		addSection.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				addSection();
			}
		});
		fAddTab = toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_ADD_TAB, SWT.FLAT);
		fAddTab.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		fAddTab.setEnabled(false);
		fAddTab.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				addTab();
			}
		});
		fAddHeader = toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_ADD_HEADER, SWT.FLAT);
		fAddHeader.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		enableSetHeaderButton(false);
		fAddHeader.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				addHeader();
			}
		});
		AspectEditorUtil.createFiller(buttonBar, toolkit, 0);
		final Button duplicateEntry= toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_DUPLICATE, SWT.FLAT);
		duplicateEntry.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		duplicateEntry.setEnabled(false);
		duplicateEntry.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				duplicateEntry(false);
			}
		});
		final Button removeEntry= toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_REMOVE, SWT.FLAT);
		removeEntry.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		removeEntry.setEnabled(false);
		removeEntry.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				removeEntry();
			}
		});
		
		AspectEditorUtil.createFiller(buttonBar, toolkit, 6);
		final Button moveUpButton= toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_MOVE_UP, SWT.FLAT);
		moveUpButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		moveUpButton.setEnabled(false);
		final Button moveDownButton= toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_MOVE_DOWN, SWT.FLAT);
		moveDownButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		moveDownButton.setEnabled(false);

		moveUpButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (fPresentationViewer.getSelection() instanceof IStructuredSelection) {
					Object o= ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
					if (o instanceof AbstractPresentationDescriptor) {
						moveSelectedItemsUpOrDownOne(true);
						setDirty();
						updateMoveButtons(moveUpButton, moveDownButton);
					}
				}
			}
		});
		moveDownButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (fPresentationViewer.getSelection() instanceof IStructuredSelection) {
					Object o= ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
					if (o instanceof AbstractPresentationDescriptor) {
						moveSelectedItemsUpOrDownOne(false);
						setDirty();
						updateMoveButtons(moveUpButton, moveDownButton);
					}
				}
			}
		});
		
		fIdToobleButton = toolkit.createButton(buttonBar, Messages.EditorPresentationAspectPart_SHOW_IDS, SWT.CHECK);
		fIdToobleButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		fIdToobleButton.setEnabled(false);
		fIdToobleButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fIdToggle= fIdToobleButton.getSelection();
				fPresentationViewer.refresh();
			}
		});
		fPresentationViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				removeEntry.setEnabled(!fPresentationViewer.getSelection().isEmpty());
				boolean isHeaderTab= false;
				if (((IStructuredSelection)fPresentationViewer.getSelection()).size() == 1 && ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement() instanceof TabDescriptor) {
					if (isHeaderTab((TabDescriptor) ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement())) {
						isHeaderTab= true;
					}
				}
				editEntry.setEnabled(!isHeaderTab && ((IStructuredSelection)fPresentationViewer.getSelection()).size() == 1);
				addPresentation.setEnabled(validAddPresentationSelection());
				boolean isHeaderSection= false;
				if (((IStructuredSelection)fPresentationViewer.getSelection()).size() == 1 && ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement() instanceof SectionDescriptor) {
					if (isHeaderTab(getParentTabDescriptor((SectionDescriptor)(((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement())))) {
						isHeaderSection= true;
					}
				}
				addSection.setEnabled(validAddSectionSelection());
				duplicateEntry.setEnabled((!isHeaderSection || isAShared((AbstractPresentationDescriptor) ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement())) && !isHeaderTab && validDuplicateSelection());
				updateMoveButtons(moveUpButton, moveDownButton);
			}
		});
		fPresentationViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (((IStructuredSelection)fPresentationViewer.getSelection()).size() == 1) {
					if (isHeaderTab((AbstractPresentationDescriptor)((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement())) {
						return;
					}
					editEntry();
				}
			}
			
		});
		updateEnablement();
		showIds(false);
		createContextMenu();
	}
	
	
	private void createContextMenu()
	{
		final MenuManager contextMenuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
		contextMenuMgr.setRemoveAllWhenShown(true);
		contextMenuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(final IMenuManager manager) {
				final IStructuredSelection selection = (IStructuredSelection) fPresentationViewer.getSelection();
				boolean isHeaderTab= false;
				if (selection.size() == 1 && selection.getFirstElement() instanceof TabDescriptor) {
					if (isHeaderTab((AbstractPresentationDescriptor) selection.getFirstElement())) {
						isHeaderTab= true;
					}
				}
				
				if (!isHeaderTab && selection.size() == 1)
				{
					final Action edit = new Action(Messages.EditorPresentationAspectPart_EDIT){
						public void run() {
							editEntry();
						}
					};
					
					contextMenuMgr.add(edit);
					contextMenuMgr.add(new Separator());
				}
				
				if(validAddPresentationSelection())
				{
					final Action addPresentation = new Action(Messages.EditorPresentationAspectPart_ADD_PRESENTATION){
						public void run() {
							addPresentation();
						}
					};
					
					contextMenuMgr.add(addPresentation);
				}
				
				if (validAddSectionSelection())
				{
					final Action addSection = new Action(Messages.EditorPresentationAspectPart_ADD_SECTION){
						public void run() {
							addSection();
						}
					};
					
					contextMenuMgr.add(addSection);
				}
				
				final Action addTab = new Action(Messages.EditorPresentationAspectPart_ADD_TAB){
					public void run() {
						addTab();
					}
				};
				
				contextMenuMgr.add(addTab);
				
				boolean hasHeader= false;
				if (fEPresentation != null && fPresentationsMap.get(fEPresentation.getIdentifier()) != null) {
					for (AbstractPresentationDescriptor desc: fPresentationsMap.get(fEPresentation.getIdentifier())) {
						if (isHeaderTab(desc)) {
							hasHeader= true;
							break;
						}
					}
				}
				
				if (!hasHeader)
				{
					final Action addHeader = new Action(Messages.EditorPresentationAspectPart_ADD_HEADER){
						public void run() {
							addHeader();
						}
					};
					
					contextMenuMgr.add(addHeader);
				}
				
				contextMenuMgr.add(new Separator());
				
				boolean isHeaderSection= false;
				if (selection.size() == 1 && selection.getFirstElement() instanceof SectionDescriptor) {
					if (isHeaderTab(getParentTabDescriptor((SectionDescriptor)selection.getFirstElement()))) {
						isHeaderSection= true;
					}
				}
				
				if ((!isHeaderSection || isAShared((AbstractPresentationDescriptor) selection.getFirstElement())) && !isHeaderTab && validDuplicateSelection())
				{
					final Action duplicate = new Action(Messages.EditorPresentationAspectPart_DUPLICATE){
						public void run() {
							duplicateEntry(false);
						}
					};
					
					contextMenuMgr.add(duplicate);
				}
				
				if (!selection.isEmpty())
				{
					final Action remove = new Action(Messages.EditorPresentationAspectPart_REMOVE){
						public void run() {
							removeEntry();
						}
					};
					
					contextMenuMgr.add(remove);
				}
			}
		});
		
		fPresentationViewer.getTree().setMenu(contextMenuMgr.createContextMenu(fPresentationViewer.getControl()));
	}
	
		
	private void showIds(boolean show) {
		fShowIds= show;
		fIdToobleButton.setSelection(showIds());
		fPresentationViewer.refresh();
	}
	
	private boolean showIds() {
		return fShowIds || fIdToggle;
	}
	
	private boolean isHeaderTab(AbstractPresentationDescriptor descriptor) {
		return descriptor instanceof TabDescriptor && EditorPresentationManager.INTERNAL_HEADER.equals(fTabLayouts.get(descriptor.getElementId()));
	}
	
	private boolean isHeaderTabOrSectionOfHeaderTab(AbstractPresentationDescriptor descriptor) {
		if (descriptor instanceof TabDescriptor) {
			if (isHeaderTab(descriptor)) {
				return true;
			}
		}
		
		if (descriptor instanceof SectionDescriptor) {
			if (isHeaderTab(getParentTabDescriptor((SectionDescriptor)descriptor))) {
				return true;
			}
		}
		
		return false;
	}
	
	private boolean validDuplicateSelection() {
		if (((IStructuredSelection)fPresentationViewer.getSelection()).size() != 1) {
			return false;
		}
		Object entry= ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		return entry instanceof SectionDescriptor || entry instanceof TabDescriptor;
	}
	
	private boolean validAddPresentationSelection() {
		if (((IStructuredSelection)fPresentationViewer.getSelection()).size() != 1) {
			return false;
		}
		Object entry= ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		return entry instanceof PresentationDescriptor || entry instanceof SectionDescriptor;
	}
	
	private boolean validAddSectionSelection() {
		if (((IStructuredSelection)fPresentationViewer.getSelection()).size() != 1) {
			return false;
		}
		Object entry= ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		return (entry instanceof TabDescriptor) && (!isHeaderTab((AbstractPresentationDescriptor) entry) || (fPresentationsMap.get(((TabDescriptor)entry).getElementId())==null) || fPresentationsMap.get(((TabDescriptor)entry).getElementId()).isEmpty());
	}
	
	private boolean isAShared(AbstractPresentationDescriptor desc) {
		if (desc instanceof SectionDescriptor) {
			return getContainingTabDescriptors((SectionDescriptor)desc).size() > 1;
		} else if (desc instanceof TabDescriptor) {
			return getContainingEditorPresentationIds((TabDescriptor)desc).size() > 1;
		}
		return false;
	}
	
	private boolean isInShared(AbstractPresentationDescriptor desc) {
		if (desc instanceof PresentationDescriptor) {
			SectionDescriptor section= getParentSectionDescriptor((PresentationDescriptor)desc);
			if (getContainingTabDescriptors(section).size() > 1) {
				return true;
			}
			desc= section;
		}
		if (desc instanceof SectionDescriptor) {
			TabDescriptor presentationDescriptor= getParentTabDescriptor((SectionDescriptor)desc);
			return getContainingEditorPresentationIds(presentationDescriptor).size() > 1;
		}
		return false;
	}
	
	private boolean isInUnlocked(AbstractPresentationDescriptor desc) {
		if (desc instanceof PresentationDescriptor) {
			return fUnlockedPresentations.contains(getParentSectionDescriptor((PresentationDescriptor)desc).getElementId());
		} else if (desc instanceof SectionDescriptor) {
			return fUnlockedSections.contains(getParentTabDescriptor((SectionDescriptor)desc).getElementId());
		}
		return false;
	}
	
	private boolean isAUnlocked(AbstractPresentationDescriptor desc) {
		if (desc instanceof SectionDescriptor) {
			return fUnlockedPresentations.contains(desc.getElementId());
		} else if (desc instanceof TabDescriptor) {
			return fUnlockedSections.contains(desc.getElementId());
		}
		return false;
	}

	
	private IStructuredSelection getSelection() {
		if (fPresentationViewer.getSelection() instanceof IStructuredSelection) {
			return (IStructuredSelection) fPresentationViewer.getSelection();
		}
		
		return null;
	}
	
	
	/**
	 * Adds keyboard listener support to move Tabs, sections and presentations
	 * up and down using the keyboard, the shortcuts to do this are Ctrl+Alt+Up
	 * and Ctrl+Alt+Down.
	 */
	private void addKeyListenerSupport() {
		fPresentationViewer.getTree().addListener(SWT.KeyUp, new Listener() {
			public void handleEvent(Event event) {
				// Check that CTRl and ALT are both pressed, then check if the Up or Down key was pressed.
				if (((event.stateMask & SWT.CTRL) != 0) && ((event.stateMask & SWT.ALT) != 0)) {
					if (event.keyCode == SWT.ARROW_UP) {
						moveSelectedItemsUpOrDownOne(true);
					}
					else if (event.keyCode == SWT.ARROW_DOWN) {
						moveSelectedItemsUpOrDownOne(false);
					}
				}
			}
		});
	}
	
	
	/**
	 * Determines what type of move the selected item needs to take and hands
	 * off to another method to handle the actual moving.
	 * 
	 * @param isMoveUp
	 *            True if the operation is an up operation, false for a down op.
	 */
	private synchronized void moveSelectedItemsUpOrDownOne(boolean isMoveUp) {
		final IStructuredSelection selection = getSelection();
		
		if ((selection == null) || (selection.isEmpty() || selection.size() != 1)) {
			return;
		}
		
		// Get the selected item from the selection, the selected items sibling entries,  the selected items current location
		// in regards to it's siblings and then the position that the selected item would be moving to.
		final AbstractPresentationDescriptor selected = (AbstractPresentationDescriptor) selection.getFirstElement();
		final List<AbstractPresentationDescriptor> entries = getSiblingEntries(selected);
		final int currentIndex = entries.indexOf(selected);
		final int moveToIndex = isMoveUp ? currentIndex - 1 : currentIndex + 1;
						
		if (selected instanceof TabDescriptor) {
			if (((moveToIndex == 0) || (currentIndex == 0)) && (isHeaderTabOrSectionOfHeaderTab(entries.get(0)))) {
				// Don't allow header to be moved or replaced.
				return;
			}
			
			if (moveToIndex == entries.size()) {
				// There is no where down to move to.
				return;
			}
			
			// Tabs can only be moved up and down with regards to other tabs.
			moveItemWithinSiblings(selected, entries, moveToIndex);
		}
		else if (selected instanceof SectionDescriptor) {
			if ((moveToIndex == -1) || (moveToIndex == entries.size())) {
				// If a section is at it's extremes the only place to go is to another tab.
				moveItemBetweenParents(selected, entries, isMoveUp);
			}
			else {
				// Not at the extremes so just move within the current tab.
				moveItemWithinSiblings(selected,entries, moveToIndex);
			}
		}
		else if (selected instanceof PresentationDescriptor) {
			if ((moveToIndex == -1) || (moveToIndex == entries.size())) {
				// If a presentation is at it's extremes the only place to go is to another section potentially in another tab.
				movePresentationBetweenSectionsAndTabs(selected,entries, isMoveUp);
			}
			else {
				// Not at the extremes so just move within the current section.
				moveItemWithinSiblings(selected,entries, moveToIndex);
			}
		}
	}
	
	/**
	 * Moves a section or a presentation between parents. For a section this
	 * means moving it between tabs and for a presentation it means moving it
	 * between sections that are all within a given tab.
	 * 
	 * @param selected
	 *            The item that is selected.
	 * @param selectedsSiblings
	 *            The selected items siblings.
	 * @param isMoveUp
	 *            True if the operation is an up operation, false for a down op.
	 */
	private void moveItemBetweenParents(AbstractPresentationDescriptor selected, 
			List<AbstractPresentationDescriptor> selectedsSiblings, boolean isMoveUp) {
		if (!canMoveItemBetweenParents(selected, selectedsSiblings, isMoveUp)) {
			return;
		}
		
		// Get the selected items parent, the parents siblings and the parents position within it's siblings.
		final AbstractPresentationDescriptor parent = getParentDescriptor(selected);
		final List<AbstractPresentationDescriptor> parentsSiblings = getSiblingEntries(parent);
		final int parentIndex = parentsSiblings.indexOf(parent);
		final int siblingIndexForMove = isMoveUp ? parentIndex - 1 : parentIndex + 1;
		
		selected = handleItemDependencyChecks(selected);
		
		if (selected == null) {
			//if it's null, the user canceled the dialog to unlock it.
			return;	
		}
				
		AbstractPresentationDescriptor parentSibling = parentsSiblings.get(siblingIndexForMove);
		List<AbstractPresentationDescriptor> relatedsChildren = fPresentationsMap.get(parentSibling.getElementId());
		
		// Try to perform the best unlocking we can, this locking/unlocking stuff is confusing.
		if (relatedsChildren != null && !relatedsChildren.isEmpty()) {
			AbstractPresentationDescriptor child = relatedsChildren.get(0);
			
			child = handleItemDependencyChecks(child);
			
			if (child == null) {
				//if it's null, the user canceled the dialog to unlock it.
				return;	
			}
		}
		else {
			parentSibling = handleItemDependencyChecks(parentSibling);
			
			if (parentSibling == null) {
				//if it's null, the user canceled the dialog to unlock it.
				return;	
			}
		}
		
		selectedsSiblings.remove(selected);
		
		// Add the item to the list of children or create a new list if it doesn't exist.
		if (relatedsChildren == null) {
			relatedsChildren = new ArrayList<AbstractPresentationDescriptor>();
			relatedsChildren.add(selected);
			fPresentationsMap.put(parentSibling.getElementId(), relatedsChildren);
		}
		else {
			if (isMoveUp) {
				relatedsChildren.add(selected);
			}
			else {
				relatedsChildren.add(0, selected);
			}
		}
		
		// Set dirty, update treeviewer and expand tree to correct level.
		setDirty();
		fPresentationViewer.refresh();
		
		if (selected instanceof PresentationDescriptor) {
			fPresentationViewer.expandToLevel(parentSibling, 3);
		}
		else if (selected instanceof SectionDescriptor) {
			fPresentationViewer.expandToLevel(parentSibling, 1);
		}
		
		fPresentationViewer.setSelection(new StructuredSelection(selected), true);
	}
	
	
	/**
	 * Moves a presentation between sections of different tabs. If the
	 * presentation is going down and is at the last section of a tab it goes to
	 * the first section of the next tab if one exists. If going up then goes to
	 * the end of the last section in the next higher tab if one exits.
	 * 
	 * @param selected
	 *            The item that is selected.
	 * @param selectedsSiblings
	 *            The selected items siblings.
	 * @param isMoveUp
	 *            True if the operation is an up operation, false for a down op.
	 */
	private void movePresentationBetweenSectionsAndTabs(AbstractPresentationDescriptor selected,
			List<AbstractPresentationDescriptor> selectedsSiblings, boolean isMoveUp) {
		if (!canMovePresentationBetweenSectionsAndTabs(selected, selectedsSiblings, isMoveUp)) {
			return;
		}
		
		// Get the parent section and the parents siblings and finally the current index of our parent.
		final AbstractPresentationDescriptor parentSection = getParentDescriptor(selected);
		final List<AbstractPresentationDescriptor> parentsSiblings = getSiblingEntries(parentSection);
		final int parentIndex = parentsSiblings.indexOf(parentSection);
		
		// If this is an up operation and the parent index is currently 0 we have to try and jump up to a 
		// new tab, or if we are moving down and we're at the last section we also need to jump to a new tab.
		if (((isMoveUp) && (parentIndex == 0)) || ((!isMoveUp) && (parentIndex == parentsSiblings.size() - 1))) {
			// move between sections on different tabs, get the tab which contains the section which contains 
			// the presentation to move. Also get the tabs siblings and the index of the current tab we are in.
			final AbstractPresentationDescriptor grandParentTab = getParentDescriptor(parentSection);
			final List<AbstractPresentationDescriptor> grandParentsSiblings = getSiblingEntries(grandParentTab);
			final int grandParentIndex = grandParentsSiblings.indexOf(grandParentTab);
			final int grandParentIndexForMove = isMoveUp ? grandParentIndex - 1 : grandParentIndex + 1;
			
			//Need to move out of the current section and into the correct section in the next tab, either higher or lower.
			final AbstractPresentationDescriptor moveToTab = grandParentsSiblings.get(grandParentIndexForMove);
			final List<AbstractPresentationDescriptor> moveToTabsChildren = fPresentationsMap.get(moveToTab.getElementId());
			
			// Get the section we are to move to, if up the last section in the next tab and if down the first section in
			// the next tab down. Then get the children for that section.
			AbstractPresentationDescriptor moveToSection = isMoveUp ? moveToTabsChildren.get(moveToTabsChildren.size() - 1)
					: moveToTabsChildren.get(0); 
			List<AbstractPresentationDescriptor> moveToSectionsChildren = fPresentationsMap.get(moveToSection.getElementId());
			
			// Try to perform the best unlocking we can, this locking/unlocking stuff is confusing.
			selected = handleItemDependencyChecks(selected);
			
			if (selected == null) {
				//if it's null, the user canceled the dialog to unlock it.
				return;	
			}
			
			if (moveToSectionsChildren != null && !moveToSectionsChildren.isEmpty()) {
				AbstractPresentationDescriptor child = moveToSectionsChildren.get(0);
				
				child = handleItemDependencyChecks(child);
				
				if (child == null) {
					//if it's null, the user canceled the dialog to unlock it.
					return;	
				}
			}
			else {
				moveToSection = handleItemDependencyChecks(moveToSection);
				
				if (moveToSection == null) {
					//if it's null, the user canceled the dialog to unlock it.
					return;	
				}
			}
			
			selectedsSiblings.remove(selected);
			
			if(moveToSectionsChildren == null) {
				moveToSectionsChildren = new ArrayList<AbstractPresentationDescriptor>();
				moveToSectionsChildren.add(selected);
				fPresentationsMap.put(moveToSection.getElementId(), moveToSectionsChildren);
			}
			else {
				final int moveToIndex = isMoveUp ? moveToSectionsChildren.size() : 0;
				
				if (isMoveUp) {
					moveToSectionsChildren.add(moveToIndex, selected);
				}
				else {					
					moveToSectionsChildren.add(0, selected);
				}
			}
			
			setDirty();
			fPresentationViewer.refresh();
			//Since we are moving tabs first expand the tab and then expand the section.
			fPresentationViewer.expandToLevel(moveToTab, 1);
			fPresentationViewer.expandToLevel(moveToSection, 2);
			fPresentationViewer.setSelection(new StructuredSelection(selected), true);
		}
		else {
			// move between sections in current tab.
			moveItemBetweenParents(selected, selectedsSiblings, isMoveUp);
		}
	}
	
	
	/**
	 * Simply moves an element within the given list
	 * 
	 * @param selectedThe
	 *            item that is selected.
	 * @param entries
	 *            The selected items siblings.
	 * @param moveToLocation
	 *            The location to move the selected item to.
	 */
	private void moveItemWithinSiblings(AbstractPresentationDescriptor selected, List<AbstractPresentationDescriptor> entries, int moveToLocation) {
		selected = handleInDependency(selected);
		
		if ((selected == null) || (moveToLocation == -1) || (moveToLocation == entries.size())) {
			//if it's null, the user canceled the dialog or at the end so don't do anything.
			return;
		}
		
		entries.remove(selected);
		entries.add(moveToLocation, selected);
		
		setDirty();
		fPresentationViewer.refresh();
		fPresentationViewer.setSelection(new StructuredSelection(selected), true);
	}
	
	
	/**
	 * Top level check to see if an item (TabDescriptor, SectionDescriptor or
	 * PresentationDescriptor) can be moved from its current position. The
	 * header tab is the only tab that cannot move, thus all other tabs can move
	 * up or down unless they reach the bottom and there is no where to go or
	 * they would replace the header tab, which means they are at the top. A
	 * section can move up or down within their own tabs no matter what, a
	 * section can also jump tabs either up or down unless the jump up would be
	 * to the header tab or it is the last section in the last tab. A
	 * presentation can move up or down inside of it's own section and can also
	 * jump sections. A presentation can go from the header all the way to the
	 * last item in the last tab/section.
	 * <p>
	 * This method is called by the code that actually performs the move so we
	 * need to pass over the current selection to make sure we're dealing with
	 * the same object.
	 * 
	 * @param selection
	 *            The current selection.
	 * @param isMoveUp
	 *            If the move request is a move up then true, false means a move
	 *            down.
	 * @return True if the move can happen, false otherwise.
	 */
	private boolean canSelectedItemsBeMoved(IStructuredSelection selection, boolean isMoveUp) {
		if ((selection == null) || (selection.isEmpty() || selection.size() != 1)) {
			return false;
		}
		
		// Get the selected item from the selection, the selected items sibling entries,  the selected items current location
		// in regards to it's siblings and then the position that the selected item would be moving to.
		final AbstractPresentationDescriptor selected = (AbstractPresentationDescriptor) selection.getFirstElement();
		final List<AbstractPresentationDescriptor> entries = getSiblingEntries(selected);
		final int currentIndex = entries.indexOf(selected);
		final int moveToIndex = isMoveUp ? currentIndex - 1 : currentIndex + 1;
		
		if (selected instanceof TabDescriptor) {
			if (((moveToIndex == 0) || (currentIndex == 0)) && (isHeaderTabOrSectionOfHeaderTab(entries.get(0)))) {
				// Don't allow header to be moved or replaced.
				return false;
			}
			
			if (moveToIndex == entries.size()) {
				// There is no where down to move to.
				return false;
			}
			
			// Tabs can only be moved up and down with regards to other tabs which is always allowed.
			return true;
		} 
		else if (selected instanceof SectionDescriptor) {
			if ((moveToIndex == -1) || (moveToIndex == entries.size())) {
				// If a section is at it's extremes the only place to go is to another tab, check to see if that is allowed.
				return canMoveItemBetweenParents(selected, entries, isMoveUp);
			}
			else {
				// Not at the extremes so just move within the current tab which is always allowed.
				return true;
			}
		}
		else if (selected instanceof PresentationDescriptor) {
			if ((moveToIndex == -1) || (moveToIndex == entries.size())) {
				// If a presentation is at it's extremes the only place to go is to another section potentially in another tab.
				return canMovePresentationBetweenSectionsAndTabs(selected,entries, isMoveUp);
			}
			else {
				// Not at the extremes so just move within the current section which is always allowed.
				return true;
			}
		}
		
		return false;
	}
	
	
	/**
	 * This is a secondary check that is called by canSelectedItemsBeMoved and
	 * determines if a section or presentation can jump to a new parent item.
	 * 
	 * @param selected
	 *            The currently selected item, should be a SectionDescriptor or
	 *            a PresentationDescriptor.
	 * @param selectedsSiblings
	 *            The siblings of the currently selected item.
	 * @param isMoveUp
	 *            If the move request is a move up then true, false means a move
	 *            down.
	 * @return True if the move can happen, meaning the selected item can jump
	 *         to a new parent, false otherwise.
	 */
	private boolean canMoveItemBetweenParents(AbstractPresentationDescriptor selected, 
		List<AbstractPresentationDescriptor> selectedsSiblings, boolean isMoveUp) {
		if ((selected instanceof SectionDescriptor) && (isHeaderTabOrSectionOfHeaderTab(selected))) {
			// can't move the header section up or down.
			return false;
		}
		
		// Get the selected items parent, the parents siblings and the parents position within it's siblings.
		final AbstractPresentationDescriptor parent = getParentDescriptor(selected);
		final List<AbstractPresentationDescriptor> parentsSiblings = getSiblingEntries(parent);
		final int parentIndex = parentsSiblings.indexOf(parent);
		final int siblingIndexForMove = isMoveUp ? parentIndex - 1 : parentIndex + 1;
		boolean canMove = false;
		
		if (isMoveUp){
			if (selected instanceof PresentationDescriptor) {
				if (siblingIndexForMove != -1) {
					// A presentation can move up until there is no where up to go.
					canMove = true;
				}
			}
			else if (selected instanceof SectionDescriptor) {
				if ((siblingIndexForMove != -1) && (!isHeaderTabOrSectionOfHeaderTab(parentsSiblings.get(siblingIndexForMove)))) {
					// A section can be moved up as long as it doesn't go into the header section and there is some place to go.
					canMove = true;
				}
			}
		}
		else {
			if (selected instanceof PresentationDescriptor) {
				if (siblingIndexForMove != parentsSiblings.size()) {
					// A presentation can move down until there is no where else to go.
					canMove = true;
				}
			}
			else if (selected instanceof SectionDescriptor) {
				if (siblingIndexForMove != parentsSiblings.size()) {
					canMove = true;
				}
			}
		}
		
		return canMove;
	}
	
	
	/**
	 * Checks whether a presentation can jump out of the current section in the
	 * current tab to another section contained with a different tab. For
	 * example we are the last presentation in the last section of a tab can we
	 * move down to the first section of the next tab.
	 * 
	 * @param selected
	 *            The currently selected item, should be a
	 *            PresentationDescriptor
	 * @param selectedsSiblings
	 *            The siblings of the currently selected item.
	 * @param isMoveUp
	 *            If the move request is a move up then true, false means a move
	 *            down.
	 * @return True if the presentation can be moved to a new tab/section, false
	 *         otherwise.
	 */
	private boolean canMovePresentationBetweenSectionsAndTabs(AbstractPresentationDescriptor selected,
			List<AbstractPresentationDescriptor> selectedsSiblings, boolean isMoveUp) {
		
		// Get the parent section and the parents siblings and finally the current index of our parent.
		final AbstractPresentationDescriptor parentSection = getParentDescriptor(selected);
		final List<AbstractPresentationDescriptor> parentsSiblings = getSiblingEntries(parentSection);
		final int parentIndex = parentsSiblings.indexOf(parentSection);
		
		// If this is an up operation and the parent index is currently 0 we have to try and jump up to a 
		// new tab, or if we are moving down and we're at the last section we also need to jump to a new tab.
		if (((isMoveUp) && (parentIndex == 0)) || ((!isMoveUp) && (parentIndex == parentsSiblings.size() - 1))) {
			// move between sections on different tabs, get the tab which contains the section which contains 
			// the presentation to move. Also get the tabs siblings and the index of the current tab we are in.
			final AbstractPresentationDescriptor grandParentTab = getParentDescriptor(parentSection);
			final List<AbstractPresentationDescriptor> grandParentsSiblings = getSiblingEntries(grandParentTab);
			final int grandParentIndex = grandParentsSiblings.indexOf(grandParentTab);
			final int grandParentIndexForMove = isMoveUp ? grandParentIndex - 1 : grandParentIndex + 1;
			
			if (((isMoveUp) && (grandParentIndexForMove == -1)) || ((!isMoveUp) && (grandParentIndexForMove == (grandParentsSiblings.size())))) {
				// no where to go, we are at the extremes.
				return false;
			}
			
			//Need to move out of the current section and into the correct section in the next tab, either higher or lower.
			final AbstractPresentationDescriptor moveToTab = grandParentsSiblings.get(grandParentIndexForMove);
			final List<AbstractPresentationDescriptor> moveToTabsChildren = fPresentationsMap.get(moveToTab.getElementId());
			
			if (moveToTabsChildren == null || moveToTabsChildren.isEmpty()) {
				// There is no section available in this tab, should check if we can move down or up even more?
				return false;
			}
		}
		
		return true;
	}
	
	
	private void addMouseListenerSupport() {
		fPresentationViewer.getTree().addListener(SWT.MouseMove, new Listener() {
			public void handleEvent(Event event) {
				updateMouseCursor(event);
			}
		});
		
		fPresentationViewer.getTree().addListener(SWT.MouseUp, new Listener() {
			public void handleEvent(Event event) {
				updateMouseCursor(event);
			}
		});
	}
	
	
	private void updateMouseCursor(Event event) {		
		final Point point = new Point(event.x, event.y);
		final Tree tree = (Tree) event.widget;
		final TreeItem item = tree.getItem(point);
		
		if (item == null)
			return;
		
		final TreeItem[] selectedItems = tree.getSelection();
		boolean itemIsSelected = false;
		
		// Determine if the item the user is hovering over or clicked down on
		// is one of the selected items.
		for (TreeItem selectedItem : selectedItems) {
			if (selectedItem.equals(item)) {
				itemIsSelected = true;
				break;
			}
		}
		
		if (itemIsSelected) {
			final AbstractPresentationDescriptor descriptor = (AbstractPresentationDescriptor) item.getData();
			
			if (isHeaderTabOrSectionOfHeaderTab(descriptor)) {
				tree.setCursor(tree.getDisplay().getSystemCursor(SWT.CURSOR_ARROW));
			}
			else {
				tree.setCursor(tree.getDisplay().getSystemCursor(SWT.CURSOR_HAND));
			}
		}
		else {
			tree.setCursor(tree.getDisplay().getSystemCursor(SWT.CURSOR_ARROW));
		}
	}
	
	
	private void addDragAndDropSupport() {
		final Transfer[] transfers = new Transfer[] { LocalSelectionTransfer.getTransfer() };
		addDragSupport(transfers);
		addDropSupport(transfers);
	}
	
	
	private void addDragSupport(Transfer[] transfers) {
		final int OPERATIONS = DND.DROP_MOVE | DND.DROP_COPY;
		fPresentationViewer.addDragSupport(OPERATIONS, transfers, new DragSourceAdapter() {
			public void dragStart(DragSourceEvent event) {
				final IStructuredSelection selection = getSelection();
				
				if (selection == null || selection.isEmpty() || selection.size() != 1) {
					event.doit = false;	
				}
				else {
					final AbstractPresentationDescriptor descriptor = (AbstractPresentationDescriptor) selection.getFirstElement();
					final AbstractPresentationDescriptor parent = getParentDescriptor(descriptor);
					
					// Not allowed to move header or section contained in header.
					if ((isHeaderTab(descriptor)) || (parent != null && isHeaderTab(parent))) {
						event.doit = false;
					}
					else {
						LocalSelectionTransfer.getTransfer().setSelection(selection);
						event.doit = true;
					}
				}
			}
			
			
			public void dragFinished(DragSourceEvent event) {
				// since the same tree is handling all events then 
				// don't bother doing anything here, the drop code can handle it all.
			}
			
			
			public void dragSetData(DragSourceEvent event) {
				event.data = LocalSelectionTransfer.getTransfer().getSelection();
			}
		});
	}
	
	private void addDropSupport(Transfer[] transfers) {
		final int OPERATIONS = DND.DROP_MOVE | DND.DROP_COPY;
		fPresentationViewer.addDropSupport(OPERATIONS, transfers, new DropTargetAdapter() {
			public void drop(DropTargetEvent event) {
				if (!(LocalSelectionTransfer.getTransfer().isSupportedType(event.currentDataType))) {
					return;
				}
				
				if (!(event.data instanceof IStructuredSelection)) {
					return;
				}
				
				final IStructuredSelection selection = (IStructuredSelection) event.data;
				
				if (selection == null || selection.isEmpty() || selection.size() != 1) {
					// nothing to do if there is no selection.
					return;
				}
				
				if ((event.item == null) || (event.item.getData() == null) || (!(event.item.getData() instanceof AbstractPresentationDescriptor))) {
					// Drop over nothing then don't do anything.
					return;
				}
				
				// Get both the selected item and the mouse over item and make sure both are editable.
				AbstractPresentationDescriptor mouseOverDescriptor = (AbstractPresentationDescriptor) event.item.getData();
				AbstractPresentationDescriptor selectedDescriptor = (AbstractPresentationDescriptor) selection.getFirstElement();
				
				if (selectedDescriptor instanceof TabDescriptor) {
					if (mouseOverDescriptor instanceof TabDescriptor) {
						handleTabDescriptionDrop((TabDescriptor) selectedDescriptor, (TabDescriptor) mouseOverDescriptor, event);
					}
					else {
						// Do nothing when a tab is dropped on a section or presentation.
						return;
					}
				}
				else if (selectedDescriptor instanceof SectionDescriptor) {
					if (mouseOverDescriptor instanceof PresentationDescriptor) {
						// Do nothing when a section is dropped on a presentation.
						return;
					}
					else if ((mouseOverDescriptor instanceof SectionDescriptor) || (mouseOverDescriptor instanceof TabDescriptor)) {
						handleSectionAndPresentationDrop(selectedDescriptor, mouseOverDescriptor, event);
					}
				
				}
				else if (selectedDescriptor instanceof PresentationDescriptor) {
					if (mouseOverDescriptor instanceof TabDescriptor) {
						// Nothing to do here.
						return;
					}
					else if ((mouseOverDescriptor instanceof SectionDescriptor) || (mouseOverDescriptor instanceof PresentationDescriptor)) {
						handleSectionAndPresentationDrop(selectedDescriptor, mouseOverDescriptor, event);
					}
				}
				
				//Reset the event detail, make the user click ctrl again while dragging.
				fEventDetail = DND.DROP_MOVE;
			}
			
			public void dragOperationChanged(DropTargetEvent event) {
					fEventDetail = event.detail;
			}
			
			public void dragOver(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_NONE;
				if (LocalSelectionTransfer.getTransfer().isSupportedType(event.currentDataType)) {
					final IStructuredSelection selection = getSelection();
					
					if (selection == null || selection.isEmpty() || selection.size() != 1) {
						// If nothing is selected in our table there is no feedback to give.
						event.feedback = DND.FEEDBACK_NONE;
						event.detail = DND.DROP_NONE;
						return;
					}
					
					// If there is no item being hovered over then display nothing.
					if ((event.item == null) 
						|| (event.item.getData() == null) 
						|| (!(event.item.getData() instanceof AbstractPresentationDescriptor))) {
						event.feedback = DND.FEEDBACK_NONE;
						event.detail = DND.DROP_NONE;
					}
					else {
						final AbstractPresentationDescriptor mouseOverDescriptor = (AbstractPresentationDescriptor) event.item.getData();
						final AbstractPresentationDescriptor selectedDescriptor = (AbstractPresentationDescriptor) selection.getFirstElement();
						
						if (selectedDescriptor instanceof TabDescriptor) {
							//Tab's can only be dropped on other tabs and the header tab cannot be moved or replaced.
							if ((mouseOverDescriptor instanceof TabDescriptor)
								&& (!mouseOverDescriptor.getElementId().equals(selectedDescriptor.getElementId()))) {
								event.feedback = DND.FEEDBACK_INSERT_AFTER | DND.FEEDBACK_SCROLL;
								event.detail = fEventDetail;
							}
							else {
								event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;
								event.detail = DND.DROP_NONE;
							}
						}
						else if (selectedDescriptor instanceof SectionDescriptor) {
							if ((mouseOverDescriptor instanceof SectionDescriptor) 
								&& (!mouseOverDescriptor.getElementId().equals(selectedDescriptor.getElementId()))
								&& (!isHeaderTabOrSectionOfHeaderTab(mouseOverDescriptor))) {
								event.feedback = DND.FEEDBACK_INSERT_AFTER | DND.FEEDBACK_SCROLL;
								event.detail = fEventDetail;
							}
							else if ((mouseOverDescriptor instanceof TabDescriptor)
								&& (!isHeaderTab(mouseOverDescriptor))) {
								event.feedback = DND.FEEDBACK_INSERT_AFTER | DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;
								event.detail = fEventDetail;
							}
							else {
								// over a presentation or the tab header or section header or ourselves.
								event.feedback = DND.FEEDBACK_SCROLL;
								event.detail = DND.DROP_NONE;
							}
						}
						else if (selectedDescriptor instanceof PresentationDescriptor) {
							if (((mouseOverDescriptor instanceof PresentationDescriptor) || (mouseOverDescriptor instanceof SectionDescriptor)) 
								&& ((!mouseOverDescriptor.getElementId().equals(selectedDescriptor.getElementId()))))	{
								event.feedback = DND.FEEDBACK_INSERT_AFTER | DND.FEEDBACK_SCROLL | DND.FEEDBACK_EXPAND;
								event.detail = fEventDetail;
							}
							else {
								// over a tab
								event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;
								event.detail = DND.DROP_NONE;
							}
						}
					}
				}
			}
				
			public void dragEnter(DropTargetEvent event) {
				// Allow dropping LocalSelectionTransfer only.
				for (TransferData transferData : event.dataTypes) {
					if (LocalSelectionTransfer.getTransfer().isSupportedType(transferData)) {
						event.currentDataType = transferData;
					}
				}
			}
		});
	}
	
	
	private void handleTabDescriptionDrop(TabDescriptor selectedDescriptor, TabDescriptor mouseOverDescriptor, DropTargetEvent event) {
		final List<AbstractPresentationDescriptor> siblings = getSiblingEntries(selectedDescriptor);
		if (event.detail == DND.DROP_MOVE) {
			siblings.remove(selectedDescriptor);
			int index = siblings.indexOf(mouseOverDescriptor);
			if ((event.feedback == DND.FEEDBACK_INSERT_AFTER) || (event.feedback == DND.FEEDBACK_SELECT)) {
				index++;
			}
			if (index > siblings.size()) {
				siblings.add(selectedDescriptor);
			}
			else {
				siblings.add(index, selectedDescriptor);
			}
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(selectedDescriptor), true);
		}
		else if (event.detail == DND.DROP_COPY) {
			if (!duplicateEntry(false)) {
				return;
			}
			final int indexOfDup = siblings.indexOf(selectedDescriptor) + 1;
			final AbstractPresentationDescriptor duplicate = siblings.get(indexOfDup);
			siblings.remove(duplicate);
			int index = siblings.indexOf(mouseOverDescriptor);
			if ((event.feedback == DND.FEEDBACK_INSERT_AFTER) || (event.feedback == DND.FEEDBACK_SELECT)) {
				index++;
			}
			if (index > siblings.size()) {
				siblings.add(duplicate);
			}
			else {
				siblings.add(index, duplicate);
			}
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(duplicate), true);
		}
	}
	
	
	private void handleSectionAndPresentationDrop(AbstractPresentationDescriptor selectedDescriptor, AbstractPresentationDescriptor mouseOverDescriptor, DropTargetEvent event) {		
		mouseOverDescriptor = handleItemDependencyChecks(mouseOverDescriptor);
		
		if (mouseOverDescriptor == null) {
			//if it's null, the user canceled the dialog to unlock it.
			return;	
		}
		
		if (event.detail == DND.DROP_MOVE) {	
			selectedDescriptor = handleInDependency(selectedDescriptor);
			
			if (selectedDescriptor == null) {
				//if it's null, the user canceled the dialog to unlock it.
				return;	
			}
		}
		
		final AbstractPresentationDescriptor selectParent = getParentDescriptor(selectedDescriptor);
		
		// The necessary areas have been unlocked.
		if (((mouseOverDescriptor instanceof TabDescriptor) && (selectedDescriptor instanceof SectionDescriptor)) 
				|| ((mouseOverDescriptor instanceof SectionDescriptor) && (selectedDescriptor instanceof PresentationDescriptor))) {
			if (selectParent.equals(mouseOverDescriptor)) {
				handleSelectionDropOnParentItem(selectedDescriptor, mouseOverDescriptor, event);
			}
			else {
				// We are dropping on a different parent item so need to handle unlocking.
				final List<AbstractPresentationDescriptor> children = fPresentationsMap.get(mouseOverDescriptor.getElementId());
				
				if (children != null && children.size() != 0) {
					AbstractPresentationDescriptor childDescriptor = children.get(0);
					childDescriptor = handleInDependency(childDescriptor);
					
					if (childDescriptor == null) {
						//if it's null, the user canceled the dialog to unlock it.
						return;	
					}
				}

				handleSelectionDropOnAnotherPerentItem(selectedDescriptor, mouseOverDescriptor, event);
			}
		}
		else if (((mouseOverDescriptor instanceof SectionDescriptor) && (selectedDescriptor instanceof SectionDescriptor))
			|| ((mouseOverDescriptor instanceof PresentationDescriptor) && (selectedDescriptor instanceof PresentationDescriptor))) {
			final AbstractPresentationDescriptor mouseOverParent = getParentDescriptor(mouseOverDescriptor);
				
			if (mouseOverParent.equals(selectParent)) { // we are in the same section, just reorder.
				handleSelectionDropWithinSameParent(selectedDescriptor, mouseOverDescriptor, event);
			}
			else {
				handleSelectionDropWithinDifferentParent(selectedDescriptor, mouseOverDescriptor, event);
			}
		}
	}
	
	private void handleSelectionDropOnParentItem(AbstractPresentationDescriptor selectedDescriptor, AbstractPresentationDescriptor mouseOverDescriptor, DropTargetEvent event) {
		final List<AbstractPresentationDescriptor> siblings = getSiblingEntries(selectedDescriptor);
		if ((event.detail == DND.DROP_MOVE) || ((event.detail == DND.DROP_COPY) && (selectedDescriptor instanceof PresentationDescriptor))) {
			siblings.remove(selectedDescriptor);
			//we dropped a section the parent tab, make it the first item in the list so it flows with the insert after.
			siblings.add(0, selectedDescriptor);
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(selectedDescriptor), true);
		}
		else if ((event.detail == DND.DROP_COPY) && (!(selectedDescriptor instanceof PresentationDescriptor))) {
			if (!duplicateEntry(false)) {
				return;
			}
			final int indexOfDup = siblings.indexOf(selectedDescriptor) + 1;
			final AbstractPresentationDescriptor duplicate = siblings.get(indexOfDup);
			siblings.remove(duplicate);
			//we dropped a section on the parent tab, make it the first item in the list so it flows with the insert after.
			siblings.add(0, duplicate);
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(duplicate), true);
		}
	}
	
	private void handleSelectionDropOnAnotherPerentItem(AbstractPresentationDescriptor selectedDescriptor, AbstractPresentationDescriptor mouseOverDescriptor, DropTargetEvent event) {
		final List<AbstractPresentationDescriptor> selectedItemSiblings = getSiblingEntries(selectedDescriptor);
		List<AbstractPresentationDescriptor> mousedOverItemSiblings;
		
		if ((mouseOverDescriptor instanceof TabDescriptor) || (mouseOverDescriptor instanceof SectionDescriptor)) {
			mousedOverItemSiblings = fPresentationsMap.get(mouseOverDescriptor.getElementId());
		}
		else {
			return;
		}
		
		if ((event.detail == DND.DROP_MOVE) || ((event.detail == DND.DROP_COPY) && (selectedDescriptor instanceof PresentationDescriptor))) {
			selectedItemSiblings.remove(selectedDescriptor);
			
			if (mousedOverItemSiblings == null) {
				mousedOverItemSiblings = new ArrayList<AbstractPresentationDescriptor>();
				mousedOverItemSiblings.add(selectedDescriptor);
				fPresentationsMap.put(mouseOverDescriptor.getElementId(), mousedOverItemSiblings);
			}
			else {
				//we dropped an item a parent, make it the first item in the list so it flows with the insert after.
				mousedOverItemSiblings.add(0, selectedDescriptor);
			}

			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(selectedDescriptor), true);
		}
		else if ((event.detail == DND.DROP_COPY) && (!(selectedDescriptor instanceof PresentationDescriptor))) {
			if (!duplicateEntry(false)) {
				return;
			}
			final int indexOfDup = selectedItemSiblings.indexOf(selectedDescriptor) + 1;
			final AbstractPresentationDescriptor duplicate = selectedItemSiblings.get(indexOfDup);
			selectedItemSiblings.remove(duplicate);
			//we dropped an item a parent, make it the first item in the list so it flows with the insert after.
			mousedOverItemSiblings.add(0, duplicate);
						
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(duplicate), true);
		}
	}
	
	private void handleSelectionDropWithinSameParent(AbstractPresentationDescriptor selectedDescriptor, AbstractPresentationDescriptor mouseOverDescriptor, DropTargetEvent event) {
		final List<AbstractPresentationDescriptor> siblings = getSiblingEntries(selectedDescriptor);
		if ((event.detail == DND.DROP_MOVE) || ((event.detail == DND.DROP_COPY) && (selectedDescriptor instanceof PresentationDescriptor))) {
			siblings.remove(selectedDescriptor);
			int index = siblings.indexOf(mouseOverDescriptor) + 1;
			if (index > siblings.size()) {
				siblings.add(selectedDescriptor);
			}
			else {
				siblings.add(index, selectedDescriptor);
			}
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(selectedDescriptor), true);
		}
		else if ((event.detail == DND.DROP_COPY) && (!(selectedDescriptor instanceof PresentationDescriptor))) {
			if (!duplicateEntry(false)) {
				return;
			}
			final int indexOfDup = siblings.indexOf(selectedDescriptor) + 1;
			final AbstractPresentationDescriptor duplicate = siblings.get(indexOfDup);
			siblings.remove(duplicate);
			int index = siblings.indexOf(mouseOverDescriptor) + 1;
			if (index > siblings.size()) {
				siblings.add(duplicate);
			}
			else {
				siblings.add(index, duplicate);
			}
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(duplicate), true);
		}
	}
	
	
	private void handleSelectionDropWithinDifferentParent(AbstractPresentationDescriptor selectedDescriptor, AbstractPresentationDescriptor mouseOverDescriptor, DropTargetEvent event) {
		final List<AbstractPresentationDescriptor> selectedItemSiblings = getSiblingEntries(selectedDescriptor);
		final List<AbstractPresentationDescriptor> mousedOverItemSiblings = getSiblingEntries(mouseOverDescriptor);
		
		if (event.detail == DND.DROP_MOVE) {
			selectedItemSiblings.remove(selectedDescriptor);
			int index = mousedOverItemSiblings.indexOf(mouseOverDescriptor) + 1;
			if (index > mousedOverItemSiblings.size()) {
				mousedOverItemSiblings.add(selectedDescriptor);
			}
			else {
				mousedOverItemSiblings.add(index, selectedDescriptor);
			}
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(selectedDescriptor), true);
		}
		else if (event.detail == DND.DROP_COPY) {
			if (!duplicateEntry(false)) {
				return;
			}
			
			//if has no items dup is at 0?
			final int indexOfDup = selectedItemSiblings.indexOf(selectedDescriptor) + 1;
			final AbstractPresentationDescriptor duplicate = selectedItemSiblings.get(indexOfDup);
			selectedItemSiblings.remove(duplicate);
			int index = mousedOverItemSiblings.indexOf(mouseOverDescriptor) + 1;
			if (index > mousedOverItemSiblings.size()) {
				mousedOverItemSiblings.add(duplicate);
			}
			else {
				mousedOverItemSiblings.add(index, duplicate);
			}
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.setSelection(new StructuredSelection(duplicate), true);
		}
	}
	
	private int getNextIndex(List<AbstractPresentationDescriptor> entries, AbstractPresentationDescriptor desc) {
		int current= entries.indexOf(desc);
		int next= current+1;
		while(next < entries.size() && !entries.get(next).getClass().equals(desc.getClass())) {
			next++;
		}
		return next;
	}
	
	
	private AbstractPresentationDescriptor handleItemDependencyChecks(AbstractPresentationDescriptor selected) {
		if (selected instanceof TabDescriptor) {
			return handleADependency(selected);
		}
		else {
			return handleInDependency(selected);
		}
	}
	
	private AbstractPresentationDescriptor handleInDependency(AbstractPresentationDescriptor selected){

		if (isInShared(selected) && !isInUnlocked(selected)) {	
			if (selected instanceof PresentationDescriptor) {
				SectionDescriptor section = getParentSectionDescriptor((PresentationDescriptor)selected);
				TabDescriptor tab = getParentTabDescriptor(section);
				boolean tabShared = isAShared(tab);
				String currentTabId = tab.getElementId();
				String currentSectionId = section.getElementId();
				
				SharedSectionDialog d= new SharedSectionDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_LOCKED_SECTION, fTabIds, fSectionIds, fResourceManager, tabShared, currentSectionId, currentTabId);
				if (d.open() == Window.OK) {
					if (d.shouldUnlockSharing()) {
						unlockASharing(section);
						return selected;
					}
					else if(!d.shouldUnlockSharing()){
						if(tabShared){
							duplicateEntry(tab,d.getTabId());
							removeDescriptor(tab);
							setDirty();
							fPresentationViewer.refresh();
						}
						duplicateEntry(section, d.getSectionId());
						removeDescriptor(section);
						setDirty();
						fPresentationViewer.refresh();
						return selected;
					}
				}
				else{
					return null;
				}
			} else if (selected instanceof SectionDescriptor) {
				showIds(true);
				TabDescriptor sharedTab = getParentTabDescriptor((SectionDescriptor)selected);
				String currentTabId = sharedTab.getElementId();
				
				SharedTabDialog d= new SharedTabDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_LOCKED_TAB, fTabIds, fResourceManager, currentTabId);
				if (d.open() == Window.OK) {
					if (d.shouldUnlockSharing()) {
						unlockInSharing(selected);
						return selected;
					}
					else if(!d.shouldUnlockSharing()){
						duplicateEntry(sharedTab,d.getTabId());
						removeDescriptor(sharedTab);
						setDirty();
						fPresentationViewer.refresh();
						return selected;
					}
				}
				else{
					return null;
				}
				showIds(false);
			} 
		}
		return selected;
	}
	
	private AbstractPresentationDescriptor handleADependency(AbstractPresentationDescriptor selected){

		if (isAShared(selected) && !isAUnlocked(selected)) {
			if (selected instanceof SectionDescriptor) {
				showIds(true);
				boolean tabShared = isInShared(selected);
				String currentSectionId = selected.getElementId();
				TabDescriptor sharedTab = getParentTabDescriptor((SectionDescriptor)selected);
				String currentTabId = sharedTab.getElementId();
				SharedSectionDialog d= new SharedSectionDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_LOCKED_SECTION, fTabIds, fSectionIds, fResourceManager, tabShared, currentSectionId, currentTabId);
				if (d.open() == Window.OK) {
					if (d.shouldUnlockSharing()) {
						unlockASharing(selected);
						return selected;
					}
					else if(!d.shouldUnlockSharing()){
						if(tabShared){
							duplicateEntry(sharedTab,d.getTabId());
							removeDescriptor(sharedTab);
							setDirty();
							fPresentationViewer.refresh();
						}
						AbstractPresentationDescriptor newSection = duplicateEntry(selected, d.getSectionId());
						removeDescriptor(selected);
						setDirty();
						fPresentationViewer.refresh();
						return newSection;
					}
				}
				else{
					return null;
				}
				showIds(false);
				
			} else if (selected instanceof TabDescriptor) {
				String currentTabId = selected.getElementId();
				
				SharedTabDialog d= new SharedTabDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_LOCKED_TAB, fTabIds, fResourceManager, currentTabId);
				if (d.open() == Window.OK) {
					if (d.shouldUnlockSharing()) {
						unlockInSharing(selected);
						return selected;
					}
					else if(!d.shouldUnlockSharing()){
						AbstractPresentationDescriptor newTab = duplicateEntry(selected, d.getTabId());
						removeDescriptor(selected);
						setDirty();
						fPresentationViewer.refresh();
						return newTab;
					}
				}
				else{
					return null;
				}
			}
		}
		return selected;
	}
	
	private void addPresentation() {
		AbstractPresentationDescriptor selected= (AbstractPresentationDescriptor)((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		AbstractPresentationDescriptor section;
		int index= -1;
		if (selected instanceof PresentationDescriptor) {
			section= getParentSectionDescriptor((PresentationDescriptor)selected);
			index= fPresentationsMap.get(section.getElementId()).indexOf(selected);
		} else {
			section= selected;
		}
		section = handleADependency(section);
		if (section==null){
			//if it's null, the user canceled the dialog
			return;
		}
		showIds(true);
		AddEditPresentationDialog d= new AddEditPresentationDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_ADD_PRESENTATION_TITLE, null, null, null, null, null, new HashMap<String, String>(), getKinds(), computeAlreadyExistingAttributes(), fAttributes, fProcessProvider, fResourceManager, this);
		if (d.open() == Window.OK) {
			String id= d.getId();
			Attribute att= d.getAttribute();
			String kind= d.getKind();
			String label= d.getLabel();
			String description= d.getDescription();
			Identifier<IAttribute> ident= null;
			if (att != null) {
				ident= att.getIdentifier();
			}
			PresentationDescriptor desc= new PresentationDescriptor(section.getElementId(), id, kind, label, description, ident, d.getProperties());
			if (fPresentationsMap.get(section.getElementId()) == null) {
				fPresentationsMap.put(section.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
				if (!fSectionIds.contains(section.getElementId())) {
					fSectionIds.add(section.getElementId());
				}
			}
			if (index > -1) {
				fPresentationsMap.get(section.getElementId()).add(index+1, desc);
			} else {
				fPresentationsMap.get(section.getElementId()).add(desc);
			}
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.expandToLevel(desc, 3);
			fPresentationViewer.setSelection(new StructuredSelection(desc), true);
		}
		showIds(false);
	}
	
	private List<Attribute> computeAlreadyExistingAttributes() {
		Set<Identifier<IAttribute>> configuredAttributeIds= new HashSet<Identifier<IAttribute>>();
		List<AbstractPresentationDescriptor> tabs = fPresentationsMap.get(fEPresentation.getIdentifier());
		if (tabs != null) {
			for (AbstractPresentationDescriptor tab: tabs) {
				if (tab instanceof TabDescriptor) {
					List<AbstractPresentationDescriptor> sections = fPresentationsMap.get(tab.getElementId());
					if (sections != null) {
						for (AbstractPresentationDescriptor section: sections) {
							if (section instanceof SectionDescriptor) {
								List<AbstractPresentationDescriptor> presentations = fPresentationsMap.get(section.getElementId());
								if (presentations != null) {
									for (AbstractPresentationDescriptor presentation: presentations) {
										if (presentation instanceof PresentationDescriptor) {
											Identifier<IAttribute> attributeId = ((PresentationDescriptor) presentation).getAttributeId();
											if (attributeId != null) {
												configuredAttributeIds.add(attributeId);
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		
		List<Attribute> preferred= new ArrayList<Attribute>();
		for (Attribute a: fAttributes) {
			if (configuredAttributeIds.contains(a.getIdentifier())) {
				continue;
			}
			preferred.add(a);
		}
		return preferred;
	}
	
	private boolean addSection() {
		AbstractPresentationDescriptor descriptor= null;
		Object object= ((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		if ((object instanceof AbstractPresentationDescriptor)) {
			descriptor= handleADependency((AbstractPresentationDescriptor)object);
		}
		if (!(descriptor instanceof TabDescriptor)) {
			//if it's null, the user canceled the dialog
			return false;
		}
		TabDescriptor tab= (TabDescriptor)descriptor;
		
		showIds(true);
		boolean isHeader = isHeaderTab(tab);
		AddEditExistingCreateNewSectionDialog d= new AddEditExistingCreateNewSectionDialog(Display.getCurrent().getActiveShell(), isHeader?Messages.EditorPresentationAspectPart_ADD_HEADER_SECTION:Messages.EditorPresentationAspectPart_ADD_SECTION_TITLE, null, null, null, null, fSectionIds, WorkItemEditorTabs.getLayoutSlots(fTabLayouts.get(tab.getElementId())), fResourceManager);
		if (isHeader) {
			d.setHeader(true);
		}
		boolean created= false;
		if (d.open() == Window.OK) {
			SectionDescriptor desc= new SectionDescriptor(d.getSectionId(), isHeader?"none":d.getTitel(), d.getSlot(), d.getProperties()); //$NON-NLS-1$
			if (d.isCreate()) {
				fSectionIds.add(desc.getElementId());
				if (fPresentationsMap.get(desc.getElementId()) == null) {
					fPresentationsMap.put(desc.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
				}
			} 
			if (fPresentationsMap.get(tab.getElementId()) == null) {
				fPresentationsMap.put(tab.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
				if (!fTabIds.contains(tab.getElementId())) {
					fTabIds.add(tab.getElementId());
				}
			}
			fPresentationsMap.get(tab.getElementId()).add(desc);
			getContainingTabDescriptors(desc).add(tab);
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.expandToLevel(desc, 2);
			fPresentationViewer.setSelection(new StructuredSelection(desc), true);
			created= true;
		}
		showIds(false);
		return created;
	}
	
	private void addHeader() {
		TabDescriptor header= new TabDescriptor(fEPresentation.getIdentifier()+"."+EditorPresentationManager.INTERNAL_HEADER, "none", new HashMap<String, String>()); //$NON-NLS-1$ //$NON-NLS-2$
		fTabLayouts.put(header.getElementId(), EditorPresentationManager.INTERNAL_HEADER);
		fTabIds.add(header.getElementId());
		if (fPresentationsMap.get(header.getElementId()) == null) {
			fPresentationsMap.put(header.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
		}
		if (fPresentationsMap.get(fEPresentation.getIdentifier()) == null) {
			fPresentationsMap.put(fEPresentation.getIdentifier(), new ArrayList<AbstractPresentationDescriptor>());
		}
		fPresentationsMap.get(fEPresentation.getIdentifier()).add(0, header);
		getContainingEditorPresentationIds(header).add(fEPresentation.getIdentifier());
		setDirty();
		enableSetHeaderButton(false);
		fPresentationViewer.refresh();
//		fPresentationViewer.expandToLevel(header, 1);
		fPresentationViewer.setSelection(new StructuredSelection(header), true);
		if (!addSection()) {
			removeEntry();
		}
	}
	
	private void addTab() {
		showIds(true);
		AddEditExistingCreateNewTabDialog d= new AddEditExistingCreateNewTabDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_ADD_TAB_TITLE, null, null, null, null, fTabIds, fResourceManager);
		if (d.open() == Window.OK) {
			TabDescriptor tab= new TabDescriptor(d.getTabId(), d.getTitle(), d.getProperties());
			fTabLayouts.put(tab.getElementId(), d.getTabLayout());
			if (d.isCreate()) {
				fTabIds.add(tab.getElementId());
				if (fPresentationsMap.get(tab.getElementId()) == null) {
					fPresentationsMap.put(tab.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
				}
			}
			if (fPresentationsMap.get(fEPresentation.getIdentifier()) == null) {
				fPresentationsMap.put(fEPresentation.getIdentifier(), new ArrayList<AbstractPresentationDescriptor>());
			}
			fPresentationsMap.get(fEPresentation.getIdentifier()).add(tab);
			getContainingEditorPresentationIds(tab).add(fEPresentation.getIdentifier());
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.expandToLevel(tab, 1);
			fPresentationViewer.setSelection(new StructuredSelection(tab), true);
		}
		showIds(false);
	}
	
	private void editEntry() {
		AbstractPresentationDescriptor selected= (AbstractPresentationDescriptor)((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		selected = handleInDependency(selected);
		if (selected==null){
			//if it's null, the user canceled the dialog
			return;
		}
		showIds(true);
		if (selected instanceof PresentationDescriptor) {
			PresentationDescriptor desc= (PresentationDescriptor)selected;
			AddEditPresentationDialog d= new AddEditPresentationDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_EDIT_PRESENTATION_TITLE, desc.getId(), findAttribute(desc), desc.getKind(), desc.getLabel(), desc.getDescription(), desc.getProperties(), getKinds(), computeAlreadyExistingAttributes(), fAttributes, fProcessProvider, fResourceManager, this);
			if (d.open() == Window.OK) {
				if (!AspectEditorUtil.equals(desc.getId(), d.getId())) {
					desc.setId(d.getId());
					setDirty();
				}
				Attribute att= d.getAttribute();
				Identifier<IAttribute> ident= null;
				if (att != null) {
					ident= att.getIdentifier();
				}
				if (!AspectEditorUtil.equals(desc.getAttributeId(), ident)) {
					desc.setAttributeId(ident);
					setDirty();
				}
				if (!AspectEditorUtil.equals(desc.getKind(), d.getKind())) {
					desc.setKind(d.getKind());
					setDirty();
				}
				if (!AspectEditorUtil.equals(desc.getLabel(), d.getLabel())) {
					desc.setLabel(d.getLabel());
					setDirty();
				}
				if (!AspectEditorUtil.equals(desc.getDescription(), d.getDescription())) {
					desc.setDescription(d.getDescription());
					setDirty();
				}
				Map<String, String> properties= d.getProperties();
				if (!AspectEditorUtil.equalMaps(properties, desc.getProperties())) {
					desc.setProperties(properties);
					setDirty();
				}
				fPresentationViewer.refresh();
			}
		} else if (selected instanceof SectionDescriptor) {
			SectionDescriptor desc= (SectionDescriptor)selected;
			TabDescriptor tab = getParentTabDescriptor((SectionDescriptor)selected);
			boolean isHeader = isHeaderTab(tab);
			AddEditExistingCreateNewSectionDialog d= new AddEditExistingCreateNewSectionDialog(Display.getCurrent().getActiveShell(), isHeader?Messages.EditorPresentationAspectPart_EDIT_HEADER_SECTION:Messages.EditorPresentationAspectPart_EDIT_SECTION_TITLE, desc.getElementId(), desc.getTitle(), desc.getSlot(), desc.getProperties(), fSectionIds, WorkItemEditorTabs.getLayoutSlots(fTabLayouts.get(tab.getElementId())), fResourceManager);
			if (isHeader) {
				d.setHeader(true);
			}
			if (d.open() == Window.OK) {
				if (!isHeader && !AspectEditorUtil.equals(d.getTitel(), desc.getTitle())){
					desc.setTitle(d.getTitel());
					setDirty();
				} else if (isHeader) {
					desc.setTitle("none"); //$NON-NLS-1$
				}
				if (!isHeader && !AspectEditorUtil.equals(d.getSlot(), desc.getSlot())){
					desc.setSlot(d.getSlot());
					setDirty();
				}
				Map<String, String> properties= d.getProperties();
				if (!AspectEditorUtil.equalMaps(properties, desc.getProperties())) {
					desc.setProperties(properties);
					setDirty();
				}
				fPresentationViewer.refresh();
			}
		} else if (selected instanceof TabDescriptor) {
			TabDescriptor desc= (TabDescriptor)selected;
			AddEditExistingCreateNewTabDialog d= new AddEditExistingCreateNewTabDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_EDIT_TAB_TITLE, desc.getElementId(), desc.getTitle(), fTabLayouts.get(desc.getElementId()), desc.getProperties(), fTabIds, fResourceManager);
			if (d.open() == Window.OK) {
				if (!AspectEditorUtil.equals(d.getTitle(), desc.getTitle())) {
					desc.setTitle(d.getTitle());
					setDirty();
				}
				if (!AspectEditorUtil.equals(fTabLayouts.get(desc.getElementId()), d.getTabLayout())) {
					fTabLayouts.put(desc.getElementId(), d.getTabLayout());
					setDirty();
				}
				Map<String, String> properties= d.getProperties();
				if (!AspectEditorUtil.equalMaps(properties, desc.getProperties())) {
					desc.setProperties(properties);
					setDirty();
				}
				fPresentationViewer.refresh();
			}
		}
		showIds(false);
	}
	
	private void removeEntry() {
		AbstractPresentationDescriptor single= null;
		AbstractPresentationDescriptor parent= null;
		if (((IStructuredSelection)fPresentationViewer.getSelection()).size() == 1) {
			single= (AbstractPresentationDescriptor)((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		}
		TabDescriptor affectedHeaderTab= null;
		@SuppressWarnings("unchecked")
		List<AbstractPresentationDescriptor> list = new ArrayList<AbstractPresentationDescriptor>(((IStructuredSelection)fPresentationViewer.getSelection()).toList());
		for (AbstractPresentationDescriptor selected: list) {

			if (isHeaderTab(selected)) {
				List<AbstractPresentationDescriptor> sList = new ArrayList<AbstractPresentationDescriptor>(fPresentationsMap.get(selected.getElementId()));
				if (sList != null && sList.size() > 0) {
					// expected and correct is that there is exactly one entry in the list
					// we however clean up everything that does not follow our requirements of correctness.
					for (AbstractPresentationDescriptor hs: sList) {
						removeDescriptor(hs);
					}
				}
			}
			else if (selected instanceof SectionDescriptor){
				TabDescriptor tab = getParentTabDescriptor((SectionDescriptor) selected);
				if (isHeaderTab(tab)) {
					affectedHeaderTab= tab;
				}
			}
			
			else if (handleInDependency(selected)==null){
				return;
			}
			if (single != null && getParentDescriptor(selected) != null) {
				parent= getParentDescriptor(selected);
			}
			removeDescriptor(selected);
		}
		if (affectedHeaderTab != null) {
			removeDescriptor(affectedHeaderTab);
		}
		
		setDirty();
		fPresentationViewer.refresh();
		if (parent != null) {
			fPresentationViewer.setSelection(new StructuredSelection(parent), true);
		}
	}

	protected void removeDescriptor(AbstractPresentationDescriptor descriptor) {
		List<AbstractPresentationDescriptor> entries= getSiblingEntries(descriptor);
		entries.remove(descriptor);
		String descriptorId= descriptor.getElementId();
		if (descriptor instanceof SectionDescriptor) {
			getContainingTabDescriptors((SectionDescriptor)descriptor).remove(getParentTabDescriptor((SectionDescriptor)descriptor));
		} else if (descriptor instanceof TabDescriptor) {
			TabDescriptor tab= (TabDescriptor)descriptor;

			Set<String> ePresentationIds= getContainingEditorPresentationIds(tab);
			ePresentationIds.remove(fEPresentation.getIdentifier());

			if (isHeaderTab(descriptor)) {
				if (ePresentationIds.isEmpty()) {
					// No more Editor presentations contain this tab
					fTabIds.remove(descriptorId);
					fTabLayouts.remove(descriptorId);
					fEditorDependencyCache.remove(descriptorId);
					
					//Remove references to/from contained sections
					List<AbstractPresentationDescriptor> containedSections= fPresentationsMap.get(descriptorId);
					if (containedSections != null) {
						for (AbstractPresentationDescriptor section : containedSections) {
							if (section instanceof SectionDescriptor) {
								getContainingTabDescriptors((SectionDescriptor)section).remove(descriptor);
							}
						}
						fPresentationsMap.remove(descriptorId);
					}
				}

				enableSetHeaderButton(true);
			}
		}
	}	

	private AbstractPresentationDescriptor duplicateEntry(AbstractPresentationDescriptor selected, String newID) {
		if (selected instanceof SectionDescriptor) {
			SectionDescriptor sec= (SectionDescriptor)selected;
			TabDescriptor tab = getParentTabDescriptor(sec);
			if (isHeaderTab(tab)) {
				SectionDescriptor desc= new SectionDescriptor(newID, sec.getTitle(), sec.getSlot(), sec.getProperties()!=null?new HashMap<String, String>(sec.getProperties()):new HashMap<String, String>());
				fSectionIds.add(newID);
				if (fPresentationsMap.get(desc.getElementId()) == null) {
					fPresentationsMap.put(desc.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
				}
				List<AbstractPresentationDescriptor> toDuplicate= fPresentationsMap.get(sec.getElementId());
				if (toDuplicate != null) {
					List<AbstractPresentationDescriptor> toFill= fPresentationsMap.get(desc.getElementId());
					for (AbstractPresentationDescriptor a: toDuplicate) {
						if (a instanceof PresentationDescriptor) {
							toFill.add(a);
						}
					}
				}
				int index= getNextIndex(fPresentationsMap.get(tab.getElementId()), sec);
				if (index > -1) {
					fPresentationsMap.get(tab.getElementId()).add(index, desc);
				} else {
					fPresentationsMap.get(tab.getElementId()).add(desc);
				}
				getContainingTabDescriptors(desc).add(tab);
				
				setDirty();
				List<AbstractPresentationDescriptor> entries = getSiblingEntries(sec);
				entries.remove(sec);
				getContainingTabDescriptors(sec).remove(tab);
				fPresentationViewer.refresh();
				fPresentationViewer.expandToLevel(desc, 2);
				fPresentationViewer.setSelection(new StructuredSelection(desc), true);
				return desc;
			} else {
					SectionDescriptor desc= new SectionDescriptor(newID, sec.getTitle(), sec.getSlot(), sec.getProperties()!=null?new HashMap<String, String>(sec.getProperties()):new HashMap<String, String>());
					fSectionIds.add(newID);
					if (fPresentationsMap.get(desc.getElementId()) == null) {
						fPresentationsMap.put(desc.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
					}
					List<AbstractPresentationDescriptor> toDuplicate= fPresentationsMap.get(sec.getElementId());
					if (toDuplicate != null) {
						List<AbstractPresentationDescriptor> toFill= fPresentationsMap.get(desc.getElementId());
						for (AbstractPresentationDescriptor a: toDuplicate) {
							if (a instanceof PresentationDescriptor) {
								toFill.add(a);
								
							}
						}
					}
					int index= getNextIndex(fPresentationsMap.get(tab.getElementId()), sec);
					if (index > -1) {
						fPresentationsMap.get(tab.getElementId()).add(index, desc);
					} else {
						fPresentationsMap.get(tab.getElementId()).add(desc);
					}
					getContainingTabDescriptors(desc).add(tab);
					setDirty();
					fPresentationViewer.refresh();
					fPresentationViewer.expandToLevel(desc, 2);
					fPresentationViewer.setSelection(new StructuredSelection(desc), true);
					return desc;
				}
		} else if (selected instanceof TabDescriptor) {
			TabDescriptor tab= (TabDescriptor) selected;
			TabDescriptor desc= new TabDescriptor(newID, tab.getTitle(), tab.getProperties()!=null?new HashMap<String, String>(tab.getProperties()):new HashMap<String, String>());
			fTabIds.add(newID);
			String tabLayout= fTabLayouts.get(tab.getElementId());
			if (tabLayout != null) {
				fTabLayouts.put(desc.getElementId(), tabLayout);
			}
			if (fPresentationsMap.get(desc.getElementId()) == null) {
				fPresentationsMap.put(newID, new ArrayList<AbstractPresentationDescriptor>());
			}
			List<AbstractPresentationDescriptor> toDuplicate= fPresentationsMap.get(tab.getElementId());
			if (toDuplicate != null) {
				List<AbstractPresentationDescriptor> toFill= fPresentationsMap.get(desc.getElementId());
				for (AbstractPresentationDescriptor a: toDuplicate) {
					if (a instanceof SectionDescriptor) {
						toFill.add(a);
						getContainingTabDescriptors((SectionDescriptor)a).add(desc);
					}
				}
			}
			int index= getNextIndex(fPresentationsMap.get(fEPresentation.getIdentifier()), tab);
			if (index > -1) {
				fPresentationsMap.get(fEPresentation.getIdentifier()).add(index, desc);
			} else {
				fPresentationsMap.get(fEPresentation.getIdentifier()).add(desc);
			}
			getContainingEditorPresentationIds(desc).add(fEPresentation.getIdentifier());
			
			setDirty();
			fPresentationViewer.refresh();
			fPresentationViewer.expandToLevel(desc, 1);
			fPresentationViewer.setSelection(new StructuredSelection(desc), true);
			return desc;
			}
			return null;
	}
	
	private boolean duplicateEntry(boolean performRefresh) {
		AbstractPresentationDescriptor selected= (AbstractPresentationDescriptor)((IStructuredSelection)fPresentationViewer.getSelection()).getFirstElement();
		selected = handleInDependency(selected);
		if (selected==null){
			//if it's null, the user canceled the dialog
			return false;
		}
		if (selected instanceof SectionDescriptor) {
			SectionDescriptor sec= (SectionDescriptor)selected;
			TabDescriptor tab= getParentTabDescriptor(sec);
			if (isHeaderTab(tab)) {
				showIds(true);
				DuplicateEntryIdSelectionDialog d= new DuplicateEntryIdSelectionDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_DUP_HEADER, Messages.EditorPresentationAspectPart_SECTION_ID_EXISTS, Messages.EditorPresentationAspectPart_ENTER_ID, fSectionIds, fResourceManager);
				if (d.open() == Window.OK) {
					String id= d.getId();
					SectionDescriptor desc= new SectionDescriptor(id, sec.getTitle(), sec.getSlot(), sec.getProperties()!=null?new HashMap<String, String>(sec.getProperties()):new HashMap<String, String>());
					fSectionIds.add(id);
					if (fPresentationsMap.get(desc.getElementId()) == null) {
						fPresentationsMap.put(desc.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
					}
					List<AbstractPresentationDescriptor> toDuplicate= fPresentationsMap.get(sec.getElementId());
					if (toDuplicate != null) {
						List<AbstractPresentationDescriptor> toFill= fPresentationsMap.get(desc.getElementId());
						for (AbstractPresentationDescriptor a: toDuplicate) {
							if (a instanceof PresentationDescriptor) {
								toFill.add(a);
							}
						}
					}
					int index= getNextIndex(fPresentationsMap.get(tab.getElementId()), sec);
					if (index > -1) {
						fPresentationsMap.get(tab.getElementId()).add(index, desc);
					} else {
						fPresentationsMap.get(tab.getElementId()).add(desc);
					}
					getContainingTabDescriptors(desc).add(tab);
					
					setDirty();
					List<AbstractPresentationDescriptor> entries = getSiblingEntries(sec);
					entries.remove(sec);
					getContainingTabDescriptors(sec).remove(tab);
					
					if (performRefresh) {
						fPresentationViewer.refresh();
						fPresentationViewer.expandToLevel(desc, 2);
						fPresentationViewer.setSelection(new StructuredSelection(desc), true);
					}
				}
				else {
					return false;
				}
				showIds(false);
			} else {
				showIds(true);
				DuplicateEntryIdSelectionDialog d= new DuplicateEntryIdSelectionDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_DUPLICATE_SECTION, Messages.EditorPresentationAspectPart_SECTION_ID_EXISTS, Messages.EditorPresentationAspectPart_ENTER_DUP_SECTION_ID, fSectionIds, fResourceManager);
				if (d.open() == Window.OK) {
					String id= d.getId();
					SectionDescriptor desc= new SectionDescriptor(id, sec.getTitle(), sec.getSlot(), sec.getProperties()!=null?new HashMap<String, String>(sec.getProperties()):new HashMap<String, String>());
					fSectionIds.add(id);
					if (fPresentationsMap.get(desc.getElementId()) == null) {
						fPresentationsMap.put(desc.getElementId(), new ArrayList<AbstractPresentationDescriptor>());
					}
					List<AbstractPresentationDescriptor> toDuplicate= fPresentationsMap.get(sec.getElementId());
					if (toDuplicate != null) {
						List<AbstractPresentationDescriptor> toFill= fPresentationsMap.get(desc.getElementId());
						for (AbstractPresentationDescriptor a: toDuplicate) {
							if (a instanceof PresentationDescriptor) {
								toFill.add(a);
								
							}
						}
					}
					int index= getNextIndex(fPresentationsMap.get(tab.getElementId()), sec);
					if (index > -1) {
						fPresentationsMap.get(tab.getElementId()).add(index, desc);
					} else {
						fPresentationsMap.get(tab.getElementId()).add(desc);
					}
					getContainingTabDescriptors(desc).add(tab);
					setDirty();
					
					if (performRefresh) {
						fPresentationViewer.refresh();
						fPresentationViewer.expandToLevel(desc, 2);
						fPresentationViewer.setSelection(new StructuredSelection(desc), true);
					}
				}
				else {
					return false;
				}
				showIds(false);
			}
		} else if (selected instanceof TabDescriptor) {
			TabDescriptor tab= (TabDescriptor) selected;
			showIds(true);
			DuplicateEntryIdSelectionDialog d= new DuplicateEntryIdSelectionDialog(Display.getCurrent().getActiveShell(), Messages.EditorPresentationAspectPart_DUPLICATE_TAB, Messages.EditorPresentationAspectPart_TAB_ID_EXISTS, Messages.EditorPresentationAspectPart_ENTER_DUP_TAB_ID,fTabIds, fResourceManager);
			if (d.open() == Window.OK) {
				String id= d.getId();
				TabDescriptor desc= new TabDescriptor(id, tab.getTitle(), tab.getProperties()!=null?new HashMap<String, String>(tab.getProperties()):new HashMap<String, String>());
				fTabIds.add(id);
				String tabLayout= fTabLayouts.get(tab.getElementId());
				if (tabLayout != null) {
					fTabLayouts.put(desc.getElementId(), tabLayout);
				}
				if (fPresentationsMap.get(desc.getElementId()) == null) {
					fPresentationsMap.put(id, new ArrayList<AbstractPresentationDescriptor>());
				}
				List<AbstractPresentationDescriptor> toDuplicate= fPresentationsMap.get(tab.getElementId());
				if (toDuplicate != null) {
					List<AbstractPresentationDescriptor> toFill= fPresentationsMap.get(desc.getElementId());
					for (AbstractPresentationDescriptor a: toDuplicate) {
						if (a instanceof SectionDescriptor) {
							toFill.add(a);
							getContainingTabDescriptors((SectionDescriptor)a).add(desc);
						}
					}
				}
				int index= getNextIndex(fPresentationsMap.get(fEPresentation.getIdentifier()), tab);
				if (index > -1) {
					fPresentationsMap.get(fEPresentation.getIdentifier()).add(index, desc);
				} else {
					fPresentationsMap.get(fEPresentation.getIdentifier()).add(desc);
				}
				getContainingEditorPresentationIds(desc).add(fEPresentation.getIdentifier());
				
				setDirty();
				
				if (performRefresh) {
					fPresentationViewer.refresh();
					fPresentationViewer.expandToLevel(desc, 1);
					fPresentationViewer.setSelection(new StructuredSelection(desc), true);
				}
			}
			else {
				return false;
			}
			showIds(false);
		}
		
		return true;
	}
	
	
	private void unlockInSharing(AbstractPresentationDescriptor desc) {
		if (desc instanceof PresentationDescriptor) {
			if (isInShared(desc)) {
				fUnlockedPresentations.add(getParentSectionDescriptor((PresentationDescriptor)desc).getElementId());
			}
		} else if (desc instanceof SectionDescriptor) {
			if (isInShared(desc)) {
				fUnlockedSections.add(getParentTabDescriptor((SectionDescriptor)desc).getElementId());
			}
		}
		fPresentationViewer.refresh();
	}
	
	private void unlockASharing(AbstractPresentationDescriptor desc) {
		if (desc instanceof SectionDescriptor) {
			if (isAShared(desc)) {
				fUnlockedPresentations.add(desc.getElementId());
			}
		} else if (desc instanceof TabDescriptor) {
			if (isAShared(desc)) {
				fUnlockedSections.add(desc.getElementId());
			}
		}
		fPresentationViewer.refresh();
	}
	
	private List<String> getKinds() {
		ITeamRepository repository= (ITeamRepository)fProcessProvider.getProcessItem().getOrigin();
		Set<String> presentations= PresentationsManager.getPresentations();
		List<String> result= new ArrayList<String>(presentations.size());
		for (String id : presentations) {
			PresentationPart part= PresentationsManager.getPresentation(id);
			if (part == null || part.isAvailable(repository)) {
				result.add(id);
			}
		}
		return result;
	}
	
	private void updateMoveButtons(Button moveUp, Button moveDown) {
		final IStructuredSelection selection = getSelection();
		
		moveUp.setEnabled(canSelectedItemsBeMoved(selection, true));
		moveDown.setEnabled(canSelectedItemsBeMoved(selection, false));
	}

	private List<AbstractPresentationDescriptor> getSiblingEntries(AbstractPresentationDescriptor desc) {
		List<AbstractPresentationDescriptor> entries;
		AbstractPresentationDescriptor p= getParentDescriptor(desc);
		if (p != null) {
			entries= fPresentationsMap.get(p.getElementId());
		} else {
			entries= fPresentationsMap.get(fEPresentation.getIdentifier());
		}
		return entries;
	}
	
	
	private void updateEnablement() {
		if (fPresentationViewer != null && !fPresentationViewer.getControl().isDisposed()) {
			fPresentationViewer.getControl().setEnabled(fEPresentation != null);
			fAddTab.setEnabled(fEPresentation != null);
			boolean hasHeader= false;
			if (fEPresentation != null && fPresentationsMap.get(fEPresentation.getIdentifier()) != null) {
				for (AbstractPresentationDescriptor desc: fPresentationsMap.get(fEPresentation.getIdentifier())) {
					if (isHeaderTab(desc)) {
						hasHeader= true;
						break;
					}
				}
			}
			enableSetHeaderButton(!hasHeader);
			fIdToobleButton.setEnabled(fEPresentation != null);
		}
	}
	
	protected void enableSetHeaderButton(boolean enable) {
		fAddHeader.setEnabled(enable);
	}

	private void initializeDependencyCache() {
		fSectionDependencyCache= new HashMap<String, Set<TabDescriptor>>();
		fEditorDependencyCache= new HashMap<String, Set<String>>();
		fUnlockedPresentations= new HashSet<String>();
		fUnlockedSections= new HashSet<String>();
		for (String editorId: fEditorIds) {
			initializeEditorDependency(editorId);
		}
	}
	
/* package */ void initializeEditorDependency(String editorId) {
		List<AbstractPresentationDescriptor> tabs= fPresentationsMap.get(editorId);
		if (tabs != null) {
			for (AbstractPresentationDescriptor tab : tabs) {
				if (tab instanceof TabDescriptor) {
					getContainingEditorPresentationIds((TabDescriptor)tab).add(editorId);
					List<AbstractPresentationDescriptor> sections= fPresentationsMap.get(tab.getElementId());
					if (sections != null) {
						for (AbstractPresentationDescriptor section : sections) {
							if (section instanceof SectionDescriptor) {
								getContainingTabDescriptors((SectionDescriptor)section).add((TabDescriptor)tab);
							}
						}
					}
				}
			}
		}
	}
	
	/*package*/ void cleanUpEditorDependency(String editorId) {
		for (Map.Entry<String, Set<String>> entry: fEditorDependencyCache.entrySet()) {
			if (entry.getValue().contains(editorId)) {
				entry.getValue().remove(editorId);
			}
		}
	}
	
	public void setRequiredListsAndMaps(Map<String, List<AbstractPresentationDescriptor>> presentationsMap, List<String> editorIds, List<String> tabIds, Map<String, String> tabLayouts, List<String> sectionIds, List<Attribute> attributes) {

		fPresentationsMap= presentationsMap;
		fEditorIds= editorIds;
		fTabIds= tabIds;
		fTabLayouts= tabLayouts;
		fSectionIds= sectionIds;
		fAttributes= new ArrayList<Attribute>();
		for (Attribute a: attributes) {
			if (WorkItemAttributes.STATE.equals(a.getIdentifier()) || WorkItemAttributes.RESOLUTION.equals(a.getIdentifier()) || WorkItem.START_DATE_PROPERTY.equals(a.getIdentifier().getStringIdentifier())) {
				continue;
			}
			fAttributes.add(a);
		}
		initializeDependencyCache();
	}
	
	@Override
	public boolean stretchHorizontally() {
		return true;
	}
	
	@Override
	public boolean stretchVertically() {
		return true;
	}
	
	@Override
	public void setInput(Object input) {
		if (input instanceof EditorPresentation) {
			fEPresentation= (EditorPresentation)input;
			fPresentationViewer.setInput(fEPresentation);
		} else {
			fEPresentation= null;
			fPresentationViewer.setInput(null);
		}
		updateEnablement();
		fPresentationViewer.expandToLevel(2);
	}
	
	@Override
	public void dispose() {
		if (fResourceManager != null) {
			fResourceManager.dispose();
			fResourceManager= null;
		}
		super.dispose();
	}

	private Attribute findAttribute(PresentationDescriptor desc) {
		Attribute attribute= null;
		if (desc.getAttributeId() != null) {
			for (Attribute a: fAttributes) {
				if (a.getIdentifier().equals(desc.getAttributeId())) {
					attribute= a;
					break;
				}
			}
		}
		return attribute;
	}

	/**
	 * Determine the color of the text for the foreground of the descriptor
	 * based on it's current state.
	 * @param desc
	 * @return {@link Color}
	 */
	private Color getCellForeground(AbstractPresentationDescriptor desc) {
		return isInShared(desc)? 
				isInUnlocked(desc)?
						PlatformUI.getWorkbench()
						.getThemeManager().getCurrentTheme()
						.getColorRegistry().get(SHAREDUNLOCKED_COLOR)
						:Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW) 
				: Display.getDefault().getSystemColor(SWT.COLOR_LIST_FOREGROUND);
	}

	/**
	 * Open dialog to create new Custom Attribute
	 */
	protected Attribute openAddAttributeDialog() {
		List<ReadOnlyType> types= WorkitemTypeEditorIdBindingManager.readWorkItemTypes(fProcessProvider.getConfigurationData(TypeManager.CONFIG_ID));
		List<EditorIdBinding> bindings = WorkitemTypeEditorIdBindingManager.readEditorIdBindings(fProcessProvider.getConfigurationData(WorkitemTypeEditorIdBindingManager.EDITOR_ID_BINDING), types);
		String type = null;
		for (EditorIdBinding editorIdBinding : bindings) {
			if(editorIdBinding.getEditorId().equals(fEPresentation.getIdentifier())){
				type = editorIdBinding.getType().getIdentifier();
				break;
			}
		}
		AddCustomAttributeWizard wiz = new AddCustomAttributeWizard(fProcessProvider, fResourceManager, type);
		WizardDialog wdlg = new WizardDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), wiz);
		wdlg.create();
		int rc = wdlg.open();
		if(rc == Window.OK){
			CustomAttribute customAttribute = wiz.getCreated();
			Identifier<IAttribute> ident= WorkItemAttributes.getPropertyIdentifier(customAttribute.getId());
			setDirty();
			Attribute attr = new Attribute(ident, customAttribute.getAttributeType(), customAttribute.getName(), true, true);
			fAttributes.add(attr);
			return attr;
		}
		return null;
	}


	private static class AddEditExistingCreateNewTabDialog extends ProcessAttachmentIconSelectionDialog {
		private DecoratedCombo fTabsCombo;
		private List<String> fTabs;
		private String fTitle;
		private String fLayout;
		private String fTabId;
		private String fCreateId;
		private Map<String, String> fProperties= new HashMap<String, String>();
		private boolean fCreate;
		private Text fIdField;
		private Button fExistingButton;
		private Button fNewButton;
		
		protected AddEditExistingCreateNewTabDialog(Shell parentShell, String title, String tabId, String tabTitle, String tabLayout, Map<String, String> properties, List<String> tabs, ResourceManager resourceManager) {
			super(parentShell, title, null, null, resourceManager);
			fTitle= tabTitle;
			fLayout= tabLayout;
			fTabId= tabId;
			if (tabs == null) {
				fTabs= new ArrayList<String>();
			} else {
				fTabs= new ArrayList<String>(tabs);
				Collections.sort(fTabs);
			}
			if (properties == null) {
				fProperties= new HashMap<String, String>();
			} else {
				fProperties= new HashMap<String, String>(properties);
			}
			setValidator(new IDialogValidator() {
				public IStatus isValid() {
					if (fTitle == null || "".equals(fTitle)) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_TITLE_EMPTY);
					}
					if (fLayout == null || "".equals(fLayout)) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_LAYOUT_EMPTY);
					}
					if (fCreate) {
						if (fCreateId == null || "".equals(fCreateId)) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_ID_EMPTY);
						}
						if (fCreateId.split("\\s").length > 1) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_WHITESPACES);
						}
						if (fTabs.contains(fCreateId)) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SAME_TAB_ID);
						}
					} else {
						if (fTabId == null) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_TAB);
						}
					}
					return Status.OK_STATUS;
				}
			});
		}
		
		public String getTabLayout() {
			return fLayout;
		}
		
		public String getTitle() {
			return fTitle;
		}
		
		public String getTabId() {
			if (fCreate) {
				return fCreateId;
			}
			return fTabId;
		}
		
		public boolean isCreate() {
			return fCreate;
		}
		
		public Map<String, String> getProperties() {
			return fProperties;
		}
		
		@Override
		protected void createUpperDialogPart(Composite parent) {
			Label l= new Label(parent, SWT.NONE);
			l.setText(Messages.EditorPresentationAspectPart_TITLE_LABEL);
			l.setLayoutData(new GridData());
			final Text titleField= new Text(parent, SWT.BORDER);
			if (fTitle != null) {
				titleField.setText(fTitle);
			}
			titleField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			titleField.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					fTitle= titleField.getText().trim();
					validate();
				}
			});
			
			l= new Label(parent, SWT.NONE);
			l.setText(Messages.EditorPresentationAspectPart_LAYOUT_LABEL);
			l.setLayoutData(new GridData());
			final DecoratedCombo layoutCombo= new DecoratedCombo(parent, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
			AspectEditorUtil.adaptCombo(layoutCombo.getCombo());
			layoutCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			layoutCombo.setLabelProvider(new LabelProvider() {
				@Override
				public String getText(Object o) {
					if (o instanceof String) {
						return WorkItemEditorTabs.getLayoutDisplayName((String)o);
					}
					return null;
				}
			});
			layoutCombo.addSelectionChangedListener(new ISelectionChangedListener() {
				public void selectionChanged(SelectionChangedEvent event) {
					Object layout= layoutCombo.getValue();
					if (NONE.equals(layout)) {
						fLayout= null;
					} else {
						fLayout= (String)layout;
					}
					validate();
				}
			});
			List<String> layoutIds= new ArrayList<String>(WorkItemEditorTabs.getTabLayouts());
			Collections.sort(layoutIds);
			Object[] valueSet= new Object[WorkItemEditorTabs.getTabLayouts().size()+1];
			valueSet[0]= NONE;
			for (int i= 1; i <= layoutIds.size(); i++) {
				valueSet[i]= layoutIds.get(i-1);
			}
			layoutCombo.setValueSet(valueSet);
			if (fLayout == null) {
				layoutCombo.setValue(NONE);
			} else {
				layoutCombo.setValue(fLayout);
			}
			
			if (fTabId != null) {
				l= new Label(parent, SWT.NONE);
				l.setText(Messages.EditorPresentationAspectPart_TAB_LABEL);
				l.setLayoutData(new GridData());
				l= new Label(parent, SWT.BORDER);
				l.setText(fTabId);
				l.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			} else {
				Group g= new Group(parent, SWT.NONE);
				g.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
				GridLayout layout = new GridLayout(2, false);
				g.setLayout(layout);
				g.setText(Messages.EditorPresentationAspectPart_TAB);
				fNewButton = new Button(g, SWT.RADIO);
				fNewButton.setText(Messages.EditorPresentationAspectPart_CREATE_TAB);
				fNewButton.setLayoutData(new GridData());
				fNewButton.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						fCreate= true;
						updateSelection(fCreate);
						validate();
					}
				});
				fIdField = new Text(g, SWT.BORDER);
				fIdField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
				fIdField.addModifyListener(new ModifyListener() {
					public void modifyText(ModifyEvent e) {
						fCreateId= fIdField.getText().trim();
						validate();
					}
				});
				fExistingButton = new Button(g, SWT.RADIO);
				fExistingButton.setText(Messages.EditorPresentationAspectPart_REUSE_TAB);
				fExistingButton.setLayoutData(new GridData());
				fExistingButton.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						fCreate= false;
						updateSelection(fCreate);
						validate();
					}
				});
				fTabsCombo= new DecoratedCombo(g, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
				AspectEditorUtil.adaptCombo(fTabsCombo.getCombo());
				fTabsCombo.getCombo().getAccessible().addAccessibleListener(new AccessibleAdapter() {
					@Override
					public void getName(AccessibleEvent e) {
						e.result = Messages.EditorPresentationAspectPart_EXISTING_TABS;
					}
				});
				fTabsCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
				fTabsCombo.setLabelProvider(new LabelProvider() {
					@Override
					public String getText(Object o) {
						if (o instanceof String) {
							return (String)o;
						}
						return null;
					}
				});
				fTabsCombo.addSelectionChangedListener(new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						Object section= fTabsCombo.getValue();
						if (NONE.equals(section)) {
							fTabId= null;
						} else {
							fTabId= (String)section;
						}
						validate();
					}
				});
				
				valueSet= new Object[fTabs.size()+1];
				valueSet[0]= NONE;
				for (int i= 1; i <= fTabs.size(); i++) {
					valueSet[i]= fTabs.get(i-1);
				}
				fTabsCombo.setValueSet(valueSet);
				if (fTabId == null) {
					fTabsCombo.setValue(NONE);
				} else {
					fTabsCombo.setValue(fTabId);
				}
				updateSelection(fCreate);
			}
			
			createPropertiesControl(parent, fProperties);
		}
		
		private void updateSelection(boolean create) {
			fTabsCombo.getCombo().setEnabled(!create);
			fExistingButton.setSelection(!create);
			fIdField.setEnabled(create);
			fNewButton.setSelection(create);
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
			// empty
		}
	}
	
	private static class SharedSectionDialog extends ProcessAttachmentIconSelectionDialog {

		private List<String> fTabs;
		private List<String> fSections;

//		private Label fEnterIdsLabel;
		private Label fCreateTabIdLabel;
		private String fCreateTabId;
		private Text fTabIdField;
		private Label fCreateSectionIdLabel;
		private String fCreateSectionId;
		private Text fSectionIdField;
		private boolean fUnlockSharing;
		private Button fRemoveSharing;
		private Button fUnlock;
		private boolean fIsInShared;
		private String fCurrentSectionId;
		private String fCurrentTabId;
		
		protected SharedSectionDialog(Shell parentShell, String title, List<String> tabs, List<String> sections, ResourceManager resourceManager, boolean isInShared, String currentSectionId, String currentTabId) {
			super(parentShell, title, null, null, resourceManager);

			fIsInShared = isInShared;
			fUnlockSharing = true;
			fCurrentSectionId = currentSectionId;
			fCurrentTabId = currentTabId;
			
			if (tabs == null) {
				fTabs= new ArrayList<String>();
			} else {
				fTabs= new ArrayList<String>(tabs);
				Collections.sort(fTabs);
			}
			if (sections == null) {
				fSections= new ArrayList<String>();
			} else {
				fSections= new ArrayList<String>(sections);
				Collections.sort(fSections);
			}

			setValidator(new IDialogValidator() {
				public IStatus isValid() {
					if (!fUnlockSharing) {
						if(fIsInShared){
							if (fCreateTabId == null || "".equals(fCreateTabId)) { //$NON-NLS-1$
								return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_ID_EMPTY);
							}
							if (fCreateTabId.split("\\s").length > 1) { //$NON-NLS-1$
								return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_WHITESPACES);
							}
							if (fTabs.contains(fCreateTabId)) {
								return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SAME_TAB_ID);
							}
						}
						if (fCreateSectionId == null || "".equals(fCreateSectionId)) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SECTION_EMPTY);
						}
						if (fCreateSectionId.split("\\s").length > 1) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_WHITESPACES);
						}
						if (fSections.contains(fCreateSectionId)) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SAME_SECTION_ID);
						}
					} 
					return Status.OK_STATUS;
				}
			});
		}

		public String getTabId() {
			return fCreateTabId;
		}
		
		public String getSectionId() {
			return fCreateSectionId;
		}
		
		public boolean shouldUnlockSharing() {
			return fUnlockSharing;
		}
		
		@SuppressWarnings("deprecation")
		@Override
		protected void createUpperDialogPart(Composite parent) {
		
			GridLayout layout = new GridLayout();
			layout.numColumns=2;
			parent.setLayout(layout);

			GridData gd = new GridData();
			gd.horizontalSpan=2;
			gd.widthHint=500;
			
			GridData explanationData = new GridData();
			explanationData.horizontalIndent= IDialogConstants.INDENT;
			explanationData.horizontalSpan=2;
			explanationData.widthHint=479;
			
			Label msg= new Label(parent, SWT.WRAP);
			msg.setText(Messages.EditorPresentationAspectPart_SECTION_LOCKED);
			msg.setLayoutData(gd);
			
			GridData gd2 = new GridData();
			gd2.horizontalSpan=2;
			
			Group g= new Group(parent, SWT.NONE);
			g.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			GridLayout layout2 = new GridLayout(2, false);
			g.setLayout(layout2);
			
			Composite radioComp = new Composite(g, SWT.NONE);
			GridLayout radioLayout= new GridLayout(2, false);
			radioLayout.marginWidth= 0;
			radioComp.setLayout(radioLayout);
			radioComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			
			g.getAccessible().addAccessibleListener(new AccessibleAdapter() {
				@Override
				public void getName(AccessibleEvent e) {
					if(e.childID == ACC.CHILDID_SELF){
						e.result = Messages.EditorPresentationAspectEditor_LOCKED_SECTION_ACCESSIBLE_TEXT; 
					}
				}
			});
			
			fUnlock = new Button(radioComp, SWT.RADIO);
			fUnlock.setText(Messages.EditorPresentationAspectPart_SHARED_SECTION_UNLOCK);
			fUnlock.setLayoutData(gd2);
			fUnlock.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					fUnlockSharing= true;
					updateSelection(fUnlockSharing);
					validate();
				}
			});
			
			Composite unlockComp = new Composite(radioComp, SWT.NONE);
			GridLayout unlockLayout= new GridLayout(2, false);
			unlockLayout.marginWidth= 0;
			unlockComp.setLayout(unlockLayout);
			unlockComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			
			Label unlockExplanation = new Label (unlockComp, SWT.WRAP);
			unlockExplanation.setText(Messages.EditorPresentationAspectPart_SHARED_SECTION_UNLOCK_EXPLANATION);
			unlockExplanation.setLayoutData(explanationData);
			
			GridData explanationData2 = new GridData();
			explanationData2.horizontalIndent= IDialogConstants.INDENT;
			explanationData2.horizontalSpan=2;

			fRemoveSharing = new Button(radioComp, SWT.RADIO);
			
			Composite removeComp = new Composite(radioComp, SWT.NONE);
			GridLayout removeLayout= new GridLayout(2, false);
			removeLayout.marginWidth= 0;
			removeComp.setLayout(removeLayout);
			removeComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			
			Label removeSharingExplanation = new Label (removeComp, SWT.WRAP);
			
			if(fIsInShared){
				fRemoveSharing.setText(Messages.EditorPresentationAspectPart_SHARED_SECTION_TAB_REMOVE_SHARING);
				removeSharingExplanation.setText(Messages.EditorPresentationAspectPart_SHARED_SECTION_TAB_REMOVE_SHARING_EXPLANATION);
			}
			else{
				fRemoveSharing.setText(Messages.EditorPresentationAspectPart_SHARED_SECTION_REMOVE_SHARING);
				removeSharingExplanation.setText(Messages.EditorPresentationAspectPart_SHARED_SECTION_REMOVE_SHARING_EXPLANATION);
			}
			fRemoveSharing.setLayoutData(gd2);
			removeSharingExplanation.setLayoutData(explanationData);
			
			fRemoveSharing.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					fUnlockSharing= false;
					updateSelection(fUnlockSharing);
					validate();
				}
			});
			
			
			GridData data = new GridData();
			data.horizontalIndent= IDialogConstants.INDENT;
			data.widthHint = 200;
			data.horizontalAlignment = GridData.END;
			
			if(fIsInShared){
				fCreateTabIdLabel= new Label(removeComp, SWT.WRAP);
				fCreateTabIdLabel.setText(Messages.EditorPresentationAspectPart_SHARED_TAB_REMOVE_SHARING_TAB);
				fCreateTabIdLabel.setLayoutData(data);
				fTabIdField = new Text(removeComp, SWT.BORDER);
				fTabIdField.setText(fCurrentTabId+"_duplicate"); //$NON-NLS-1$
				fCreateTabId=fCurrentTabId+"_duplicate"; //$NON-NLS-1$
				fTabIdField.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, false));
				fTabIdField.addModifyListener(new ModifyListener() {
					public void modifyText(ModifyEvent e) {
						fCreateTabId= fTabIdField.getText().trim();
						validate();
					}
				});
			}

			fCreateSectionIdLabel= new Label(removeComp, SWT.WRAP);
			fCreateSectionIdLabel.setText(Messages.EditorPresentationAspectPart_SHARED_SECTION_REMOVE_SHARING_SECTION);
			fCreateSectionIdLabel.setLayoutData(data);
			fSectionIdField = new Text(removeComp, SWT.BORDER);
			fSectionIdField.setText(fCurrentSectionId+"_duplicate"); //$NON-NLS-1$
			fCreateSectionId = fCurrentSectionId+"_duplicate"; //$NON-NLS-1$
			fSectionIdField.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, false));
			
			fSectionIdField.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					fCreateSectionId= fSectionIdField.getText().trim();
					validate();
				}
			});
		
			updateSelection(fUnlockSharing);
		}
		
		private void updateSelection(boolean unlock) {
			fUnlock.setSelection(unlock);
			
			fRemoveSharing.setSelection(!unlock);
			if (fIsInShared){
				fCreateTabIdLabel.setEnabled(!unlock);
				fTabIdField.setEnabled(!unlock);
			}
			fCreateSectionIdLabel.setEnabled(!unlock);
			fSectionIdField.setEnabled(!unlock);
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
			// empty
		}
	}
	
	private static class SharedTabDialog extends ProcessAttachmentIconSelectionDialog {

		private List<String> fTabs;

//		private Label fEnterIdsLabel;
		private Label fCreateTabIdLabel;
		private String fCreateTabId;
		private Text fTabIdField;
		private boolean fUnlockSharing;
		private Button fRemoveSharing;
		private Button fUnlock;
		private String fCurrentTabId;

		
		protected SharedTabDialog(Shell parentShell, String title, List<String> tabs, ResourceManager resourceManager, String currentTabId) {
			super(parentShell, title, null, null, resourceManager);

			fUnlockSharing=true;
			fCurrentTabId = currentTabId;
			
			if (tabs == null) {
				fTabs= new ArrayList<String>();
			} else {
				fTabs= new ArrayList<String>(tabs);
				Collections.sort(fTabs);
			}

			setValidator(new IDialogValidator() {
				public IStatus isValid() {
					if (!fUnlockSharing) {
						if (fCreateTabId == null || "".equals(fCreateTabId)) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_ID_EMPTY);
						}
						if (fCreateTabId.split("\\s").length > 1) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_WHITESPACES);
						}
						if (fTabs.contains(fCreateTabId)) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SAME_TAB_ID);
						}
					} 
					return Status.OK_STATUS;
				}
			});
		}

		public String getTabId() {
			return fCreateTabId;
		}
		
		public boolean shouldUnlockSharing() {
			return fUnlockSharing;
		}
		
		@SuppressWarnings("deprecation")
		@Override
		protected void createUpperDialogPart(Composite parent) {
		
			GridLayout layout = new GridLayout();
			layout.numColumns=2;
			parent.setLayout(layout);

			GridData gd = new GridData();
			gd.horizontalSpan=2;
			gd.widthHint=500;
			
			GridData explanationData = new GridData();
			explanationData.horizontalIndent= IDialogConstants.INDENT;
			explanationData.horizontalSpan=2;
			explanationData.widthHint=479;
			
			Label msg= new Label(parent, SWT.WRAP);
			msg.setText(Messages.EditorPresentationAspectPart_TAB_LOCKED);
			msg.setLayoutData(gd);
			
			GridData gd2 = new GridData();
			gd2.horizontalSpan=2;
			
			Group g= new Group(parent, SWT.NONE);
			g.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			GridLayout layout2 = new GridLayout(2, false);
			g.setLayout(layout2);
			
			Composite radioComp = new Composite(g, SWT.NONE);
			GridLayout radioLayout= new GridLayout(2, false);
			radioLayout.marginWidth= 0;
			radioComp.setLayout(radioLayout);
			radioComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			
			g.getAccessible().addAccessibleListener(new AccessibleAdapter() {
				@Override
				public void getName(AccessibleEvent e) {
					if(e.childID == ACC.CHILDID_SELF){
						e.result = Messages.EditorPresentationAspectEditor_LOCKED_TAB_ACCESSIBLE_TEXT; 
					}
				}
			});
			
			fUnlock = new Button(radioComp, SWT.RADIO);
			fUnlock.setText(Messages.EditorPresentationAspectPart_SHARED_TAB_UNLOCK);
			fUnlock.setLayoutData(gd2);
			fUnlock.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					fUnlockSharing= true;
					updateSelection(fUnlockSharing);
					validate();
				}
			});
			
			Composite unlockComp = new Composite(radioComp, SWT.NONE);
			GridLayout unlockLayout= new GridLayout(2, false);
			unlockLayout.marginWidth= 0;
			unlockComp.setLayout(unlockLayout);
			unlockComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			
			Label unlockExplanation = new Label (unlockComp, SWT.WRAP);
			unlockExplanation.setText(Messages.EditorPresentationAspectPart_SHARED_TAB_UNLOCK_EXPLANATION);
			unlockExplanation.setLayoutData(explanationData);

			fRemoveSharing = new Button(radioComp, SWT.RADIO);
			
			Composite removeComp = new Composite(radioComp, SWT.NONE);
			GridLayout removeLayout= new GridLayout(2, false);
			removeLayout.marginWidth= 0;
			removeComp.setLayout(removeLayout);
			removeComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			
			fRemoveSharing.setText(Messages.EditorPresentationAspectPart_SHARED_TAB_REMOVE_SHARING);
			fRemoveSharing.setLayoutData(gd2);
			
			fRemoveSharing.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					fUnlockSharing= false;
					updateSelection(fUnlockSharing);
					validate();
				}
			});
			
			GridData explanationData2 = new GridData();
			explanationData2.horizontalIndent= IDialogConstants.INDENT;
			explanationData2.horizontalSpan=2;

			Label removeSharingExplanation = new Label (removeComp, SWT.WRAP);
			removeSharingExplanation.setText(Messages.EditorPresentationAspectPart_SHARED_TAB_REMOVE_SHARING_EXPLANATION);
			removeSharingExplanation.setLayoutData(explanationData2);
			
			GridData data = new GridData();
			data.horizontalIndent= IDialogConstants.INDENT;
			data.widthHint = 200;
			data.horizontalAlignment = GridData.END;
			
			fCreateTabIdLabel= new Label(removeComp, SWT.WRAP);
			fCreateTabIdLabel.setText(Messages.EditorPresentationAspectPart_SHARED_TAB_REMOVE_SHARING_TAB);
			fCreateTabIdLabel.setLayoutData(data);
			fTabIdField = new Text(removeComp, SWT.BORDER);
			fTabIdField.setText(fCurrentTabId+"_duplicate"); //$NON-NLS-1$
			fCreateTabId=fCurrentTabId+"_duplicate"; //$NON-NLS-1$
			fTabIdField.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, false));
			fTabIdField.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					fCreateTabId= fTabIdField.getText().trim();
					validate();
				}
			});
			updateSelection(fUnlockSharing);
		}
		
		private void updateSelection(boolean unlock) {
			fUnlock.setSelection(unlock);
			fRemoveSharing.setSelection(!unlock);
			fCreateTabIdLabel.setEnabled(!unlock);
			fTabIdField.setEnabled(!unlock);
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
			// empty
		}
	}
	
	private static class AddEditExistingCreateNewSectionDialog extends ProcessAttachmentIconSelectionDialog {
		private DecoratedCombo fSectionsCombo;
		private Text fTitleField;
		private List<String> fSections;
		private String fTitle;
		private String fSlot;
		private String fSectionId;
		private String fCreateId;
		private Map<String, String> fProperties= new HashMap<String, String>();
		private List<String> fSlots;
		private boolean fCreate;
		private Text fIdField;
		private Button fExistingButton;
		private Button fNewButton;
		private boolean fIsHeader= false;
		
		protected AddEditExistingCreateNewSectionDialog(Shell parentShell, String title, String sectionid, String sectionTitle, String slot, Map<String, String> properties, List<String> sections, Set<String> slots, ResourceManager resourceManager) {
			super(parentShell, title, null, null, resourceManager);
			fTitle= sectionTitle;
			fSlot= slot;
			fSectionId= sectionid;
			if (sections == null) {
				fSections= new ArrayList<String>();
			} else {
				fSections= new ArrayList<String>(sections);
				Collections.sort(fSections);
			}
			if (properties == null) {
				fProperties= new HashMap<String, String>();
			} else {
				fProperties= new HashMap<String, String>(properties);
			}
			if (slots == null) {
				fSlots= new ArrayList<String>();
			} else {
				fSlots= new ArrayList<String>(slots);
			}
			setValidator(new IDialogValidator() {
				public IStatus isValid() {
					if ((fTitle == null || "".equals(fTitle)) && !fIsHeader) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_TITLE_EMPTY);
					}
					if ((fSlot == null || "none".equals(fSlot)) && !fIsHeader) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SLOT_EMPTY);
					}
					if (fCreate) {
						if (fCreateId == null || "".equals(fCreateId)) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SECTION_EMPTY);
						}
						if (fCreateId.split("\\s").length > 1) { //$NON-NLS-1$
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_WHITESPACES);
						}
						if (fSections.contains(fCreateId)) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_SAME_SECTION_ID);
						}
					} else {
						if (fSectionId == null) {
							return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_SECTION);
						}
					}
					return Status.OK_STATUS;
				}
			});
		}
		
		public void setHeader(boolean header) {
			fIsHeader= true;
		}
		
		public String getSlot() {
			return fSlot;
		}
		
		public String getTitel() {
			return fTitle;
		}
		
		public boolean isCreate() {
			return fCreate;
		}
		
		public String getSectionId() {
			if (fCreate) {
				return fCreateId;
			}
			return fSectionId;
		}
		
		public Map<String, String> getProperties() {
			return fProperties;
		}
		
		@Override
		protected void createUpperDialogPart(Composite parent) {
			Label l= null;
			if (!fIsHeader) {
				l= new Label(parent, SWT.NONE);
				l.setText(Messages.EditorPresentationAspectPart_TITLE_LABEL);
				l.setLayoutData(new GridData());
				fTitleField= new Text(parent, SWT.BORDER);
				if (fTitle != null) {
					fTitleField.setText(fTitle);
				}
				fTitleField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
				fTitleField.addModifyListener(new ModifyListener() {
					public void modifyText(ModifyEvent e) {
						fTitle= fTitleField.getText().trim();
						validate();
					}
				});
			
				l= new Label(parent, SWT.NONE);
				l.setText(Messages.EditorPresentationAspectPart_SLOT_LABEL);
				l.setLayoutData(new GridData());
				final DecoratedCombo slotCombo= new DecoratedCombo(parent, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
				AspectEditorUtil.adaptCombo(slotCombo.getCombo());
				slotCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
				slotCombo.setLabelProvider(new LabelProvider() {
					@Override
					public String getText(Object o) {
						if (o instanceof String) {
							return WorkItemEditorTabs.getSlotName((String)o);
						}
						return null;
					}
				});
				slotCombo.addSelectionChangedListener(new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						Object slot= slotCombo.getValue();
						if (NONE.equals(slot)) {
							fSlot= null;
						} else {
							fSlot= (String)slot;
						}
						validate();
					}
				});
				Object[] valueSet= new Object[fSlots.size()+1];
				valueSet[0]= NONE;
				for (int i= 1; i <= fSlots.size(); i++) {
					valueSet[i]= fSlots.get(i-1);
				}
				slotCombo.setValueSet(valueSet);
				if (fSlot == null) {
					slotCombo.setValue(NONE);
				} else {
					slotCombo.setValue(fSlot);
				}
			}
			
			if (fSectionId != null) {
				l= new Label(parent, SWT.NONE);
				l.setText(Messages.EditorPresentationAspectPart_SECTION_LABEL);
				l.setLayoutData(new GridData());
				l= new Label(parent, SWT.BORDER);
				l.setText(fSectionId);
				l.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			} else {
				Group g= new Group(parent, SWT.NONE);
				g.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
				GridLayout layout = new GridLayout(2, false);
				g.setLayout(layout);
				g.setText(Messages.EditorPresentationAspectPart_SECTION);
				fNewButton = new Button(g, SWT.RADIO);
				fNewButton.setText(Messages.EditorPresentationAspectPart_CREATE_SECTION);
				fNewButton.setLayoutData(new GridData());
				fNewButton.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						fCreate= true;
						updateSelection(fCreate);
						validate();
					}
				});
				fIdField = new Text(g, SWT.BORDER);
				fIdField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
				fIdField.addModifyListener(new ModifyListener() {
					public void modifyText(ModifyEvent e) {
						fCreateId= fIdField.getText().trim();
						validate();
					}
				});
				fExistingButton = new Button(g, SWT.RADIO);
				fExistingButton.setText(Messages.EditorPresentationAspectPart_REUSE_SECTION);
				fExistingButton.setLayoutData(new GridData());
				fExistingButton.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						fCreate= false;
						updateSelection(fCreate);
						validate();
					}
				});
			
				fSectionsCombo= new DecoratedCombo(g, SWT.READ_ONLY, DecoratedCombo.NO_DECORATION_OPTION);
				AspectEditorUtil.adaptCombo(fSectionsCombo.getCombo());
				fSectionsCombo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
				fSectionsCombo.setLabelProvider(new LabelProvider() {
					@Override
					public String getText(Object o) {
						if (o instanceof String) {
							return (String)o;
						}
						return null;
					}
				});
				fSectionsCombo.addSelectionChangedListener(new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						Object section= fSectionsCombo.getValue();
						if (NONE.equals(section)) {
							fSectionId= null;
						} else {
							fSectionId= (String)section;
						}
						validate();
					}
				});
				
				Object[] valueSet= new Object[fSections.size()+1];
				valueSet[0]= NONE;
				for (int i= 1; i <= fSections.size(); i++) {
					valueSet[i]= fSections.get(i-1);
				}
				fSectionsCombo.setValueSet(valueSet);
				if (fSectionId == null) {
					fSectionsCombo.setValue(NONE);
				} else {
					fSectionsCombo.setValue(fSectionId);
				}
				updateSelection(fCreate);
			}
			createPropertiesControl(parent, fProperties);
		}
		
		private void updateSelection(boolean create) {
			fSectionsCombo.getCombo().setEnabled(!create);
			fExistingButton.setSelection(!create);
			fIdField.setEnabled(create);
			fNewButton.setSelection(create);
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
			// empty
		}
	}
	
	public static class DuplicateEntryIdSelectionDialog extends ProcessAttachmentIconSelectionDialog {
		private Text fIdField;
		private String fId;
		private List<String> fAllElements;
		private String fEnterIdMessage;
		
		public DuplicateEntryIdSelectionDialog(Shell parentShell, String title, final String idExistsMessage, String enterIdMessage, List<String> allElements, ResourceManager resourceManager) {
			super(parentShell, title, null, null, resourceManager);
			fAllElements= new ArrayList<String>(allElements);
			fEnterIdMessage= enterIdMessage;
			setValidator(new IDialogValidator() {
				public IStatus isValid() {
					if (fId== null || "".equals(fId)) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_EMPTY_ID);
					} else if (fId.split("\\s").length > 1) { //$NON-NLS-1$
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.EditorPresentationAspectPart_NO_WHITESPACES);
					}
					if (fAllElements.contains(fId)) {
						return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, idExistsMessage);
					}
					return Status.OK_STATUS;
				}
			});
		}
		
		public String getId() {
			return fId.trim();
		}
		
		@Override
		protected void createUpperDialogPart(Composite parent) {
			Composite message= new Composite(parent, SWT.NONE);
			GridLayout layout= new GridLayout();
			layout.marginWidth= 0;
			message.setLayout(layout);
			message.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
			Label msg= new Label(message, SWT.WRAP);
			msg.setText(fEnterIdMessage);
			msg.setLayoutData(new GridData());
			
			Label l= new Label(parent, SWT.NONE);
			l.setText(Messages.EditorPresentationAspectPart_ID_LABEL);
			l.setLayoutData(new GridData());
			fIdField= new Text(parent, SWT.BORDER);
			if (fId != null) {
				fIdField.setText(fId);
			}
			fIdField.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			fIdField.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					fId= fIdField.getText().trim();
					validate();
				}
			});
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
			// empty
		}
	}

	private static void createPropertiesControl(Composite parent, final Map<String, String> properties) {
		Composite props= new Composite(parent, SWT.NONE);
		GridData data= new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
		data.verticalIndent= 5;
		props.setLayoutData(data);
		GridLayout layout= new GridLayout(2, false);
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		props.setLayout(layout);
		Label l= new Label(props, SWT.NONE);
		l.setText(Messages.EditorPresentationAspectPart_PROPERTIES_LABEL);
		l.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
		Table t= new Table(props, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		t.setHeaderVisible(true);
		data= new GridData(SWT.FILL, SWT.FILL, true, true);
		data.minimumHeight= t.getItemHeight()*6;
		data.widthHint= 280;
//		data.heightHint= t.getItemHeight()*6;
//		data.widthHint= 400;
		t.setLayoutData(data);
		TableColumn c= new TableColumn(t, SWT.NONE);
		c.setWidth(140);
		c.setResizable(true);
		c.setText(KEY_LABEL);
		c= new TableColumn(t, SWT.NONE);
		c.setWidth(140);
		c.setResizable(true);
		c.setText(VALUE_LABEL);
		
		final TableViewer propertiesViewer= new TableViewer(t);
		propertiesViewer.setColumnProperties(new String[]{KEY_LABEL, VALUE_LABEL});
		propertiesViewer.setLabelProvider(new ITableLabelProvider() {
			@SuppressWarnings("unchecked")
			public String getColumnText(Object element, int columnIndex) {
				if (element instanceof Map.Entry) {
					if (columnIndex == 0) {
						return ((Map.Entry<String, String>)element).getKey();
					} else if (columnIndex == 1) {
						return ((Map.Entry<String, String>)element).getValue();
					}
				}
				return null;
			}
			public Image getColumnImage(Object element, int columnIndex) {
				return null;
			}
			public void addListener(ILabelProviderListener listener) {
			}
			public void dispose() {
			}
			public boolean isLabelProperty(Object element, String property) {
				return false;
			}
			public void removeListener(ILabelProviderListener listener) {
			}
		});
		propertiesViewer.setContentProvider(new IStructuredContentProvider() {
			@SuppressWarnings("rawtypes")
			public Object[] getElements(Object inputElement) {
				if (inputElement instanceof Set) {
					return ((Set)inputElement).toArray();
				}
				return null;
			}

			public void dispose() {
			}

			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		});
		propertiesViewer.setInput(properties.entrySet());
		
		propertiesViewer.addOpenListener(new IOpenListener(){
			@SuppressWarnings("unchecked")
			public void open(OpenEvent event) {
				KeyValuePairDialog dialog = new KeyValuePairDialog(new SameShellProvider(propertiesViewer.getControl().getShell()));
				Entry<String, String> firstElement = null;

				if (propertiesViewer.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection structured = (IStructuredSelection) propertiesViewer.getSelection();
					firstElement = (Entry<String, String>) structured.getFirstElement();
					dialog.setInitialKeyValue(firstElement);
				}

				dialog.create();
				dialog.getShell().setText(Messages.EditorPresentationAspectPart_EDIT_PROPERTIES);
				dialog.open();
				if (dialog.getReturnCode() == Window.OK) {
					String key = dialog.getKey();
					if (firstElement != null) {
						properties.remove(firstElement.getKey());
					}
					if (key != null) {
						String value = dialog.getValue();
						if (value == null) {
							value= ""; //$NON-NLS-1$
						}
						if (key.length() > 0 || value.length() > 0) {
							properties.put(key, value);
						}
					}
					propertiesViewer.refresh();
				}
			}
		});
		
		t.addKeyListener(new KeyAdapter(){
			@SuppressWarnings("unchecked")
			@Override
			public void keyPressed(KeyEvent e) {
				if (e.keyCode == SWT.F2) {
					KeyValuePairDialog dialog = new KeyValuePairDialog(new SameShellProvider(propertiesViewer.getControl().getShell()));
					Entry<String, String> firstElement = null;

					if (propertiesViewer.getSelection() instanceof IStructuredSelection) {
						IStructuredSelection structured = (IStructuredSelection) propertiesViewer.getSelection();
						firstElement = (Entry<String, String>) structured.getFirstElement();
						dialog.setInitialKeyValue(firstElement);
					}

					dialog.create();
					dialog.getShell().setText(Messages.EditorPresentationAspectPart_EDIT_PROPERTIES);
					dialog.open();
					if (dialog.getReturnCode() == Window.OK) {
						String key = dialog.getKey();
						if (firstElement != null) {
							properties.remove(firstElement.getKey());
						}
						if (key != null) {
							String value = dialog.getValue();
							if (value == null) {
								value= ""; //$NON-NLS-1$
							}
							if (key.length() > 0 || value.length() > 0) {
								properties.put(key, value);
							}
						}
						propertiesViewer.refresh();
					}
				}
			}
		});
		
		Composite buttonBar= new Composite(props, SWT.NONE);
		data= new GridData(SWT.FILL, SWT.FILL, false, true);
		data.verticalIndent= t.getHeaderHeight();
		buttonBar.setLayoutData(data);
		layout= new GridLayout();
		buttonBar.setLayout(layout);
		final Button addButton= new Button(buttonBar, SWT.FLAT);
		addButton.setText("+"); //$NON-NLS-1$
		addButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		addButton.addSelectionListener(new SelectionAdapter() {
			@SuppressWarnings("rawtypes")
			@Override
			public void widgetSelected(SelectionEvent e) {
				properties.put(KEY_LABEL, VALUE_LABEL);
				propertiesViewer.refresh();
				for (Map.Entry entry: properties.entrySet()) {
					if (KEY_LABEL.equals(entry.getKey()) && VALUE_LABEL.equals(entry.getValue())) {
						propertiesViewer.editElement(entry, 0);
						break;
					}
				}
			}
		});
		final Button removeButton= new Button(buttonBar, SWT.FLAT);
		removeButton.setText("-"); //$NON-NLS-1$
		removeButton.setEnabled(false);
		removeButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		removeButton.addSelectionListener(new SelectionAdapter() {
			@SuppressWarnings("unchecked")
			@Override
			public void widgetSelected(SelectionEvent e) {
				for (Map.Entry<String, String> entry: (List<Map.Entry<String, String>>)((IStructuredSelection)propertiesViewer.getSelection()).toList()) {
					properties.remove(entry.getKey());
				}
				propertiesViewer.refresh();
			}
		});
		propertiesViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				removeButton.setEnabled(!propertiesViewer.getSelection().isEmpty());
			}
		});
		propertiesViewer.setCellEditors(new CellEditor[]{new TextCellEditor(t), new TextCellEditor(t)});
		propertiesViewer.setColumnProperties(new String[] { KEY_LABEL, VALUE_LABEL});
		ICellModifier modifier= new ICellModifier() {

			public boolean canModify(Object element, String property) {
				return true;
			}

			@SuppressWarnings("unchecked")
			public Object getValue(Object element, String property) {
				if (element instanceof Map.Entry) {
					if (property.equals(KEY_LABEL)) {
						return ((Map.Entry<String, String>)element).getKey();
					}
					if (property.equals(VALUE_LABEL)) {
						return ((Map.Entry<String, String>)element).getValue();
					}
				}
				return null;
			}
			
			@SuppressWarnings("unchecked")
			public void modify(Object element, String property, final Object value) {
				if (element instanceof Item) {
					element = ((Item) element).getData();
				}
				if (!(element instanceof Map.Entry) || !(value instanceof String)) {
					return;
				}
				Map.Entry<String, String> entry= (Map.Entry<String, String>)element;
				if (property.equals(KEY_LABEL)) {
					properties.remove(entry.getKey());
					properties.put((String)value, entry.getValue());
				} else if (property.equals(VALUE_LABEL)) {
					properties.put(entry.getKey(), (String)value);
				}
				propertiesViewer.refresh();
			}
		};
		propertiesViewer.setCellModifier(modifier);
		if (!"gtk".equals(SWT.getPlatform())) { //$NON-NLS-1$
			Utils.updateColumnWidths(propertiesViewer.getTable(), 1, 1);
		}
	}
}
