package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

import java.util.Map.Entry;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.window.IShellProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.ibm.team.repository.common.util.NLS;

/**
 * KeyValuePairDialog is a dialog for editing a selected key and value pair.
 *
 */
public class KeyValuePairDialog extends Dialog {

	/**
	 * Create an instance of the receiver.
	 * @param parentShell
	 */
	public KeyValuePairDialog(IShellProvider parentShell) {
		super(parentShell);
	}

	private Text fKeyField;
	private String fCachedKey;
	private Text fValueField;
	private String fCachedValue;

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite entryComposite = (Composite) super.createDialogArea(parent);
		entryComposite.setLayout(new GridLayout(2, false));

		Label key = new Label(entryComposite, SWT.NONE);
		key.setText(NLS.bind(Messages.KeyValuePairDialog_LABEL_COLON, Messages.EditorPresentationAspectPart_KEY_LABEL));

		fKeyField = new Text(entryComposite, SWT.BORDER);
		if (fCachedKey != null) {
			fKeyField.setText(fCachedKey);
		}
		GridData data = new GridData(SWT.FILL, SWT.NONE, true, false);
		data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
		fKeyField.setLayoutData(data);

		Label value = new Label(entryComposite, SWT.NONE);
		value.setText(NLS.bind(Messages.KeyValuePairDialog_LABEL_COLON, Messages.EditorPresentationAspectPart_VALUE_LABEL));

		fValueField = new Text(entryComposite, SWT.BORDER);
		if (fCachedValue != null) {
			fValueField.setText(fCachedValue);
		}
		fValueField.setLayoutData(data);

		return entryComposite;
	}

	/**
	 * Return the value in the key field.
	 * 
	 * @return String
	 */
	public String getKey() {
		return fCachedKey;
	}

	/**
	 * Return the value in the value field.
	 * 
	 * @return String
	 */
	public String getValue() {
		return fCachedValue;
	}

	/**
	 * Set the initial key and value.
	 * 
	 * @param element
	 */
	public void setInitialKeyValue(Entry<String, String> element) {
		if (element != null) {
			fCachedKey= element.getKey();
			fCachedValue= element.getValue();
		} else {
			fCachedKey = null;
			fCachedValue = null;
		}
	}
	
	@Override
	protected void okPressed() {
		fCachedKey = fKeyField.getText();
		fCachedValue = fValueField.getText();
		super.okPressed();
	}

}
