/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation;

import java.util.ArrayList;
import java.util.List;

import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorIdBinding.ReadOnlyType;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypeManager;

public class WorkitemTypeEditorIdBindingManager {
	public static final String EDITOR_ID_BINDING = "com.ibm.team.workitem.editor.configuration.workitemTypeEditorIdBinding"; //$NON-NLS-1$
	
	public static List<EditorIdBinding> readEditorIdBindings(ModelElement configurationElement, List<ReadOnlyType> existingTypes) {
		List<EditorIdBinding> bindings= new ArrayList<EditorIdBinding>();
		if (configurationElement != null) {
			for (ModelElement element: (List<ModelElement>)configurationElement.getChildElements()) {
				if (EditorPresentationManager.LAYOUT_BINDING.equals(element.getName())) {
					String type= element.getAttribute(EditorPresentationManager.WORKITEM_TYPE);
					String editorId= element.getAttribute(EditorPresentationManager.EDITOR_ID);
					String target= element.getAttribute(EditorPresentationManager.TARGET);
					if (type != null && editorId != null) {
						ReadOnlyType typ= null;
						for (ReadOnlyType t: existingTypes) {
							if (t.getId().equals(type)) {
								typ= t;
								break;
							}
						}
						if (typ == null) {
							typ= new ReadOnlyType(type, null);
							existingTypes.add(typ);
						}
						bindings.add(new EditorIdBinding(target, typ, editorId));
					}
				}
			}
		}
		return bindings;
	}
	
	public static void writeEditorIdBindings(IMemento memento, List<EditorIdBinding> editorIdBindings) {
		for (EditorIdBinding binding: editorIdBindings) {
			if (binding.getType() == null || binding.getEditorId() == null) {
				continue;
			}
			IMemento b= memento.createChild(EditorPresentationManager.LAYOUT_BINDING);
			b.putString(EditorPresentationManager.WORKITEM_TYPE, binding.getType().getId());
			b.putString(EditorPresentationManager.EDITOR_ID, binding.getEditorId());
			String target= binding.getTarget();
			if (target != null && !"".equals(target.trim())) { //$NON-NLS-1$
				b.putString(EditorPresentationManager.TARGET, target);
			}
		}
	}
	
	public static List<ReadOnlyType> readWorkItemTypes(ModelElement typesElement) {
		List<ReadOnlyType> types= new ArrayList<ReadOnlyType>();
		
		final TypeManager.TypeFactory<ReadOnlyType> typeFactory = new TypeManager.TypeFactory<ReadOnlyType>() {
			@Override
			public ReadOnlyType newType(String id, String name, String iconUrl, String dimmedIconUrl, List<String> aliases) {
				if (id == null) return null;
				return new ReadOnlyType(id, name, iconUrl, aliases);
			}
		};
		
		if (typesElement != null) {
			for (ModelElement wf: (List<ModelElement>)typesElement.getChildElements()) {
				if (wf.getName().equals(TypeManager.TYPE)) {
					
					final ReadOnlyType type = TypeManager.readTypeElement(wf, typeFactory);
					if (type != null) types.add(type);
				}
			}
		}
		return types;
	}
	
}
