/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2009, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.configurationlet;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.SameShellProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.Widget;

import com.ibm.icu.text.Collator;
import com.ibm.team.links.common.registry.IEndPointDescriptor;
import com.ibm.team.process.common.IProcessItemHandle;
import com.ibm.team.process.common.ModelElement;
import com.ibm.team.process.internal.common.links.LinkTypes;
import com.ibm.team.process.internal.ide.ui.editors.ProjectLinkListLabelProvider;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.IPrefixProvider;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.ProcessAttachmentIconSelectionDialog;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.AddEditPresentationDialog;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorPresentation;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorPresentation.Attribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.KeyValuePairDialog;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.valueprovider.IProcessConfigurationDataProvider;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WFWorkflow.WFState;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.workflow.WorkflowManager;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedCombo;
import com.ibm.team.workitem.ide.ui.internal.editor.DecoratedControl;
import com.ibm.team.workitem.ide.ui.internal.editor.links.LinkTypeFilter;
import com.ibm.team.workitem.rcp.ui.internal.util.Utils;

public abstract class Configurationlet {
	private static final String KEY_LABEL = Messages.Configurationlet_KEY;
	private static final String VALUE_LABEL = Messages.Configurationlet_VALUE;
	private Map<String, String> fProperties;
	private final Set<String> fManagedKeys= new HashSet<String>();
	private IProcessConfigurationDataProvider fProcessProvider;
	private Attribute fAttribute;
	
	public void configure(Map<String, String> properties, IProcessConfigurationDataProvider processProvider, EditorPresentation.Attribute attribute) {
		fProperties= properties;
		fProcessProvider= processProvider;
		fAttribute= attribute;
	}
	
	protected ModelElement getConfigurationData(String dataId) {
		if (fProcessProvider != null) {
			return fProcessProvider.getConfigurationData(dataId);
		}
		return null;
	}
	
	protected IProcessItemHandle getProcessItem() {
		if (fProcessProvider != null)
			return fProcessProvider.getProcessItem();
		
		return null;
	}
	
	protected Attribute getAttribute() {
		return fAttribute;
	}
	
	protected void setProperty(String key, String value) {
		fProperties.put(key, value);
	}
	
	protected String getProperty(String key) {
		return fProperties.get(key);
	}
	
	protected void unSetProperty(String key) {
		fProperties.remove(key);
	}
	
	protected void addManagedKey(String key) {
		fManagedKeys.add(key);
	}
	
	public abstract void createContent(Composite parent);
	
	/**
	 * @param description the description is shown as tooltip on the label and the check box
	 */
	protected void createBooleanControl(Composite parent, String label, String description, final String key) {
		createBooleanControl(parent, label, description, key, false);
	}

	/**
	 * @param description
	 *            the description is shown as tooltip on the label and the check
	 *            box
	 */
	protected void createBooleanControl(Composite parent, String label, String description, final String key, boolean defaultValue) {
		addManagedKey(key);
		Label l= new Label(parent, SWT.NONE);
		l.setText(NLS.bind(Messages.Configurationlet_LABEL_COMMA, label));
		l.setToolTipText(description);
		l.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		final Button check= new Button(parent, SWT.CHECK);
		check.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false));
		check.setToolTipText(description);
		String property= fProperties.get(key);
		check.setSelection(property == null ? defaultValue : EditorPresentationManager.TRUE.equals(property));
		check.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (check.getSelection()) {
					fProperties.put(key, EditorPresentationManager.TRUE);
				} else {
					fProperties.put(key, EditorPresentationManager.FALSE);
				}
			}
		});
	}
	/**
	 * The combo box will display the values of the values map and on selection change
	 * write back the key of the values map according to the selection, or remove the
	 * key from the properties map if the property-value equals the nullValue
	 * @param values maps the possible property-values to display names for these values
	 * @param description the description is shown as tooltip on both the label and the combo box
	 */
	protected DecoratedCombo createComboBox(Composite parent, String label, String description, final String key, final Map<String, String> values, final String nullValue) {
		addManagedKey(key);
		Label l= new Label(parent, SWT.NONE);
		l.setText(NLS.bind(Messages.Configurationlet_LABEL_COMMA, label));
		l.setToolTipText(description);
		l.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		final DecoratedCombo combo= new DecoratedCombo(parent, SWT.READ_ONLY, DecoratedControl.NO_DECORATION_OPTION);
		AspectEditorUtil.adaptCombo(combo.getCombo());
		combo.getLayoutControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		combo.getCombo().setToolTipText(description);
		List<String> valueSet= new ArrayList<String>(values.keySet());
		final Collator collator= Collator.getInstance();
		Collections.sort(valueSet, new Comparator<String>() {
			@Override
			public int compare(String o1, String o2) {
				if (o1.equals(nullValue)) {
					return -1;
				}
				if (o2.equals(nullValue)) {
					return 1;
				}
				String name1= getDisplayName(o1, values);
				String name2= getDisplayName(o2, values);
				return collator.compare(name1, name2);
			}
		});
		combo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof String) {
					return getDisplayName((String) element, values);
				}
				return null;
			}
		});
		combo.setValueSet(valueSet.toArray());
		Object value= fProperties.get(key);
		if (value == null) {
			combo.setValue(nullValue);
		} else {
			combo.setValue(value);
		}
		combo.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				String value= (String) combo.getValue();
				if (value.equals(nullValue)) {
					value= null;
				}
				if (!AspectEditorUtil.equals(value, fProperties.get(key))) {
					if (value == null) {
						fProperties.remove(key);
					} else {
						fProperties.put(key, value);
					}
				}
			}
		});
		return combo;
	}
	
	protected TableViewer createCheckedTable(Composite parent, String label, String description, final String key, final Map<String, String> values) {
		addManagedKey(key);
		Label l= new Label(parent, SWT.NONE);
		l.setText(NLS.bind(Messages.Configurationlet_LABEL_COMMA, label));
		l.setToolTipText(description);
		l.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true));
		
		Composite filterArea= new Composite(parent, SWT.NONE);
		filterArea.setLayout(new GridLayout());
		filterArea.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		final Composite linkTableArea= new Composite(filterArea, SWT.NONE);
		linkTableArea.setLayout(new GridLayout());
		linkTableArea.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));


		final TableViewer viewer= new TableViewer(linkTableArea, SWT.CHECK | SWT.BORDER | SWT.MULTI | SWT.FULL_SELECTION | SWT.NO_SCROLL | SWT.V_SCROLL);
		final Table table= viewer.getTable();
		table.setHeaderVisible(false);
		table.setLinesVisible(true);
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		table.setToolTipText(description);
		table.pack();
		table.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				if (event.detail == SWT.CHECK) {
					Widget item= event.item;
					if (item instanceof TableItem) {
						String value= fProperties.get(key);
						String linkType= (String)item.getData();
						Set<String> linkTypes;
						if (value== null || "[]".equals(value)) { //$NON-NLS-1$
							linkTypes= new HashSet<String>();
						} else {
							linkTypes= LinkTypeFilter.parseLinkTypeSet(value);
						}
						if (((TableItem)item).getChecked()) {
							linkTypes.add(linkType);
						} else {
							linkTypes.remove(linkType);
						}
						fProperties.put(key, linkTypes.isEmpty() ? null : linkTypes.toString());
					}
				}
			}
		});
		
		TableViewerColumn col= new TableViewerColumn(viewer, SWT.LEFT);
		col.getColumn().setMoveable(false);
		col.getColumn().setResizable(true);
		col.getColumn().pack();
		
		((GridData)linkTableArea.getLayoutData()).heightHint= 10 * table.getItemHeight();
		
		TableColumnLayout colLayout= new TableColumnLayout();
		colLayout.setColumnData(col.getColumn(), new ColumnWeightData(100, true));
		linkTableArea.setLayout(colLayout);
		
		Composite buttonArea= new Composite(filterArea, SWT.NONE);
		buttonArea.setLayout(new GridLayout(2, true));
		buttonArea.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));

		Button selButton= new Button(buttonArea, SWT.NONE);
		selButton.setText(Messages.LinksConfigurationlet_LABEL_SELECT_ALL_BUTTON);
		selButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		selButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent evt) {
				TableItem[] items= table.getItems();
				if (items != null) {
					for (TableItem cur : items) {
						cur.setChecked(true);
					}
					Set<String> linkTypes= values.keySet();
					fProperties.put(key, linkTypes.toString());
					viewer.setSelection(new StructuredSelection(linkTypes));
				}
			}
		});
		Button deselButton= new Button(buttonArea, SWT.NONE);
		deselButton.setText(Messages.LinksConfigurationlet_LABEL_DESELECT_ALL_BUTTON);
		deselButton.setLayoutData(new GridData(SWT.FILL, SWT.NONE, true, false));
		deselButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent evt) {
				TableItem[] items= table.getItems();
				if (items != null) {
					for (TableItem cur : items) {
						cur.setChecked(false);
					}
					fProperties.put(key, null);
					viewer.setSelection(new StructuredSelection());
				}
			}
		});
		
		viewer.setContentProvider(new IStructuredContentProvider() {
			@Override
			public Object[] getElements(Object inputElement) {
				if (inputElement instanceof Object[]) {
					Object[] input= (Object[])inputElement;
					if (input.length > 0 && input[0] instanceof String) {
						return (Object[])inputElement;
					}
				}
				return new Object[0];
			}
			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
			@Override
			public void dispose() {}
		});
		
		List<String> valueSet= new ArrayList<String>(values.keySet());
		final Collator collator= Collator.getInstance();
		Collections.sort(valueSet, new Comparator<String>() {
			@Override
			public int compare(String o1, String o2) {
				String name1= getDisplayName(o1, values);
				String name2= getDisplayName(o2, values);
				return collator.compare(name1, name2);
			}
		});
		viewer.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof String) {
					return getDisplayName((String) element, values);
				}
				return null;
			}
		});
		
		viewer.setInput(valueSet.toArray());
		
		Set<String> linkTypes= LinkTypeFilter.parseLinkTypeSet(fProperties.get(key));
		TableItem[] items= table.getItems();
		if (items != null) {
			for (TableItem cur : items) {
				String linkType= (String)cur.getData();
				if (linkTypes.contains(linkType)) {
					cur.setChecked(true);
				}
			}
			viewer.setSelection(new StructuredSelection(linkTypes));
		}
		
		return viewer;
	}

	private String getDisplayName(String value, Map<String, String> values) {
		String name= values.get(value);
		if (name == null) {
			name= value;
		}
		return name;
	}
	
	/**
	 * Subclasses should override this method.
	 */
	public IStatus isValid(){
		return Status.OK_STATUS;
	}

	/**
	 * Subclasses should override this method.
	 * Used to add listeners to the form fields that need validation
	 */
	public void setupValidation(AddEditPresentationDialog dialog) {}
	
	protected void createReadonlyPropertyControl(Composite parent) {
		String nullValue= Messages.Configurationlet_INHERIT_FROM_ATTRIBUTE;
		Map<String, String> valueSet= new HashMap<String, String>();
		valueSet.put(nullValue, nullValue);
		valueSet.put(EditorPresentationManager.TRUE, Messages.Configurationlet_TRUE);
		valueSet.put(EditorPresentationManager.FALSE, Messages.Configurationlet_FALSE);
		createComboBox(parent, Messages.Configurationlet_READ_ONLY, Messages.Configurationlet_READ_ONLY_DESCRIPTION, EditorPresentationManager.READONLY, valueSet, nullValue);
	}
	
	protected void createLabelVisiblePropertyControl(Composite parent) {
		String nullValue= Messages.Configurationlet_DEPENDING_ON_CONTENT;
		Map<String, String> valueSet= new HashMap<String, String>();
		valueSet.put(nullValue, nullValue);
		valueSet.put(EditorPresentationManager.TRUE, Messages.Configurationlet_TRUE);
		valueSet.put(EditorPresentationManager.FALSE, Messages.Configurationlet_FALSE);
		createComboBox(parent, Messages.Configurationlet_SHOW_LABEL, Messages.Configurationlet_SHOW_LABEL_DESCRIPTION, EditorPresentationManager.LABEL_VISIBLE, valueSet, nullValue);
	}
	
	protected void createVisibilityControls(Composite parent) {
		createBooleanControl(parent, Messages.Configurationlet_HIDE_IF_EMPTY, Messages.Configurationlet_HIDE_IF_EMPTY_DESCRIPTION, EditorPresentationManager.HIDEIFEMPTY);
		createBooleanControl(parent, Messages.Configurationlet_HIDE_ON_CREATION, Messages.Configurationlet_HIDE_ON_CREATION_DESCRIPTION, EditorPresentationManager.HIDEIFCREATION);
		
		addManagedKey(EditorPresentationManager.HIDEIFINSTATE);
		Label l= new Label(parent, SWT.NONE);
		l.setText(NLS.bind(Messages.Configurationlet_LABEL_COMMA, Messages.Configurationlet_HIDE_IN_STATES));
		l.setToolTipText(Messages.Configurationlet_HIDE_IN_STATES_DESCRIPTION);
		l.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Composite content= new Composite(parent, SWT.NONE);
		content.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		content.setLayout(GridLayoutFactory.fillDefaults().numColumns(2).create());
		final Text text= new Text(content, SWT.READ_ONLY | SWT.BORDER);
		text.setBackground(content.getBackground());
		text.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		final List<WFWorkflow> workflows = WorkflowManager.readWorkflows(getConfigurationData(AspectEditorUtil.WORKFLOWS_CONFIGURATION_POINT));
		String value= fProperties.get(EditorPresentationManager.HIDEIFINSTATE);
		if (value != null) {
			String displayNames= getStateNames(value, workflows);
			text.setText(displayNames);
			text.setToolTipText(displayNames);
		}
		final Button edit= new Button(content, SWT.PUSH);
		edit.setText(Messages.Configurationlet_EDIT);
		edit.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
		edit.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				EditWorkflowStatesDialog d= new EditWorkflowStatesDialog(edit.getShell(), Messages.Configurationlet_EDIT_STATES, null, null, workflows, fProperties.get(EditorPresentationManager.HIDEIFINSTATE), null);
				if (d.open() == Window.OK) {
					String value= d.getStates();
					if (value == null || value.trim().length() == 0) {
						unSetProperty(EditorPresentationManager.HIDEIFINSTATE);
						text.setText(""); //$NON-NLS-1$
					} else {
						setProperty(EditorPresentationManager.HIDEIFINSTATE, value);
						String displayNames= getStateNames(value, workflows);
						text.setText(displayNames);
						text.setToolTipText(displayNames);
					}
				}
			}
		});
		
		
		createHideIfEmptyCombo(parent);
		createShowWithProjectLinkCombo(parent);
	}

	private void createHideIfEmptyCombo(Composite parent) {
		String nullValue= Messages.Configurationlet_UNCONFIGURED_LINK_TYPE;
		Map<String, String> valueSet= new HashMap<String, String>();
		valueSet.put(nullValue, nullValue);
		List<IEndPointDescriptor> endPoints = AspectEditorUtil.getEndPoints();
		for (IEndPointDescriptor desc: endPoints) {
			valueSet.put(AspectEditorUtil.getXMLString(desc), desc.getDisplayName());
		}
		createComboBox(parent, Messages.Configurationlet_HIDE_IF_NO_LINKS_EXIST, Messages.Configurationlet_HIDE_IF_NO_LINKS_EXIST_DESCRIPTION, EditorPresentationManager.HIDEIFENDPOINTEMPTY, valueSet, nullValue);
	}
	
	private void createShowWithProjectLinkCombo(Composite parent) {
		String nullValue= Messages.Configurationlet_UNCONFIGURED_LINK_TYPE;
		Map<String, String> valueSet= new HashMap<String, String>();
		valueSet.put(nullValue, nullValue);
		for (String link : LinkTypes.getLinkTypesForTagName("cmServiceProviders")) { //$NON-NLS-1$
			valueSet.put(link, ProjectLinkListLabelProvider.getLinkTypeLabel(link));
		}
		createComboBox(parent, Messages.Configurationlet_HIDE_IF_NO_ASSOCIATION_EXISTS, Messages.Configurationlet_HIDE_IF_NO_ASSOCIATION_EXISTS_DESC, EditorPresentationManager.HIDEIFNOPROJECTLINK, valueSet, nullValue);
	}
	
	private String getStateNames(String stateIds, Collection<WFWorkflow> workflows) {
		Collection<String> states= new HashSet<String>();
		for (String s: stateIds.split(",")) { //$NON-NLS-1$
			for (WFWorkflow flow: workflows) {
				for (WFState state: flow.getStates()) {
					String stateId= s.trim();
					if (stateId.equals(state.getId())) {
						states.add(state.getName());
					}
				}
			}
		}
		List<String> stateNames= new ArrayList<String>(states);
		final Collator collator= Collator.getInstance();
		Collections.sort(stateNames, new Comparator<String>() {
			@Override
			public int compare(String o1, String o2) {
				return collator.compare(o1, o2);
			}
		});
		String names= ""; //$NON-NLS-1$
		for (String s: stateNames) {
			if(!names.equals("")){ //$NON-NLS-1$
				names= NLS.bind(Messages.Configurationlet_LIST_OF_STATE_NAMES, names, s);
			} else {
				names = s;
			}
		}
		return names;
	}
	
	/**
	 * Creates a text field - the content will be put as value in the properties map.
	 * If the text value is the empty string or only contains whitespaces, the key
	 * is removed from the map.
	 * @param description the description is shown as tooltip on the label
	 */
	protected Text createTextField(Composite parent, String label, String description, final String key) {
		addManagedKey(key);
		Label l= new Label(parent, SWT.NONE);
		l.setText(NLS.bind(Messages.Configurationlet_LABEL_COMMA, label));
		l.setToolTipText(description);
		l.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		final Text text= new Text(parent, SWT.BORDER);
		text.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		String value= fProperties.get(key);
		if (value != null) {
			text.setText(value);
		}
		text.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				String content= text.getText().trim();
				if (content.length() > 0) {
					fProperties.put(key, content);
				} else {
					fProperties.remove(key);
				}
			}
		});
		return text;
	}
	
	protected Composite createNumberField(Composite parent, String label, String description, final String key, int min, int max, int increment, int defaultValue) {
		addManagedKey(key);
		Label l= new Label(parent, SWT.NONE);
		l.setText(NLS.bind(Messages.Configurationlet_LABEL_COMMA, label));
		l.setToolTipText(description);
		l.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));

		final Spinner spinner= new Spinner(parent, SWT.BORDER);
		spinner.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		String value= fProperties.get(key);
		spinner.setMinimum(min);
		spinner.setPageIncrement(increment);
		spinner.setMaximum(max);

		if (value != null) {
			int v= defaultValue;
			try {
				v= new Integer(value);
			} catch (NumberFormatException e) {}
			spinner.setSelection(v);
		}
		spinner.addModifyListener(new ModifyListener() {
			@Override
			public void modifyText(ModifyEvent e) {
				String content= spinner.getText().trim();
				if (content.length() > 0) {
					fProperties.put(key, content);
				} else {
					fProperties.remove(key);
				}
			}
		});
		return spinner;
	}

	protected void createPropertiesViewer(Composite parent) {
		Composite props= new Composite(parent, SWT.NONE);
		GridData data= new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
		data.verticalIndent= 5;
		props.setLayoutData(data);
		GridLayout layout= new GridLayout(2, false);
		layout.marginWidth= 0;
		layout.marginHeight= 0;
		props.setLayout(layout);
		Label l= new Label(props, SWT.NONE);
		l.setText(Messages.Configurationlet_PROPERTIES);
		l.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
		Table t= new Table(props, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		t.setHeaderVisible(true);
		data= new GridData(SWT.FILL, SWT.FILL, true, true);
		data.minimumHeight= t.getItemHeight()*6;
		data.widthHint= 280;
//		data.heightHint= t.getItemHeight()*6;
//		data.widthHint= 400;
		t.setLayoutData(data);
		TableColumn c= new TableColumn(t, SWT.NONE);
		c.setWidth(140);
		c.setResizable(true);
		c.setText(KEY_LABEL);
		c= new TableColumn(t, SWT.NONE);
		c.setWidth(140);
		c.setResizable(true);
		c.setText(VALUE_LABEL);
		
		final TableViewer propertiesViewer= new TableViewer(t);
		propertiesViewer.setColumnProperties(new String[]{KEY_LABEL, VALUE_LABEL});
		propertiesViewer.setLabelProvider(new ITableLabelProvider() {
			@Override
			public String getColumnText(Object element, int columnIndex) {
				if (element instanceof Map.Entry) {
					if (columnIndex == 0) {
						return ((Map.Entry<String, String>)element).getKey();
					} else if (columnIndex == 1) {
						return ((Map.Entry<String, String>)element).getValue();
					}
				}
				return null;
			}
			@Override
			public Image getColumnImage(Object element, int columnIndex) {
				return null;
			}
			@Override
			public void addListener(ILabelProviderListener listener) {
			}
			@Override
			public void dispose() {
			}
			@Override
			public boolean isLabelProperty(Object element, String property) {
				return false;
			}
			@Override
			public void removeListener(ILabelProviderListener listener) {
			}
		});
		propertiesViewer.setContentProvider(new IStructuredContentProvider() {
			@Override
			public Object[] getElements(Object inputElement) {
				Set<Entry<String, String>> input= new HashSet<Entry<String,String>>(fProperties.entrySet());
				for (Iterator<Entry<String, String>> it= input.iterator(); it.hasNext();) {
					if (fManagedKeys.contains(it.next().getKey())) {
						it.remove();
					}
				}
				return input.toArray();
			}

			@Override
			public void dispose() {
			}

			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		});
		propertiesViewer.setInput(fProperties);
		
		propertiesViewer.addOpenListener(new IOpenListener(){
			@Override
			public void open(OpenEvent event) {
				KeyValuePairDialog dialog = new KeyValuePairDialog(new SameShellProvider(propertiesViewer.getControl().getShell()));
				Entry<String, String> firstElement = null;

				if (propertiesViewer.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection structured = (IStructuredSelection) propertiesViewer.getSelection();
					firstElement = (Entry<String, String>) structured.getFirstElement();
					dialog.setInitialKeyValue(firstElement);
				}

				dialog.create();
				dialog.getShell().setText(Messages.Configurationlet_EDIT_PROPERTIES);
				dialog.open();
				if (dialog.getReturnCode() == Window.OK) {
					String key = dialog.getKey();
					if (firstElement != null) {
						fProperties.remove(firstElement.getKey());
					}
					if (key != null) {
						String value = dialog.getValue();
						if (value == null) {
							value= "";  //$NON-NLS-1$
						}
						if (key.length() > 0 || value.length() > 0) {
							fProperties.put(key, value);
						}
					}
					propertiesViewer.refresh();
				}
			}
		});
		
		t.addKeyListener(new KeyAdapter(){
			@Override
			public void keyPressed(KeyEvent e) {
				if (e.keyCode == SWT.F2) {
					KeyValuePairDialog dialog = new KeyValuePairDialog(new SameShellProvider(propertiesViewer.getControl().getShell()));
					Entry<String, String> firstElement = null;

					if (propertiesViewer.getSelection() instanceof IStructuredSelection) {
						IStructuredSelection structured = (IStructuredSelection) propertiesViewer.getSelection();
						firstElement = (Entry<String, String>) structured.getFirstElement();
						dialog.setInitialKeyValue(firstElement);
					}

					dialog.create();
					dialog.getShell().setText(Messages.Configurationlet_EDIT_PROPERTIES);
					dialog.open();
					if (dialog.getReturnCode() == Window.OK) {
						String key = dialog.getKey();
						if (firstElement != null) {
							fProperties.remove(firstElement.getKey());
						}
						if (key != null) {
							String value = dialog.getValue();
							if (value == null) {
								value= "";  //$NON-NLS-1$
							}
							if (key.length() > 0 || value.length() > 0) {
								fProperties.put(key, value);
							}
						}
						propertiesViewer.refresh();
					}
				}
			}
		});
		
		Composite buttonBar= new Composite(props, SWT.NONE);
		data= new GridData(SWT.FILL, SWT.FILL, false, true);
		data.verticalIndent= t.getHeaderHeight();
		buttonBar.setLayoutData(data);
		layout= new GridLayout();
		buttonBar.setLayout(layout);
		final Button addButton= new Button(buttonBar, SWT.FLAT);
		addButton.setText(Messages.Configurationlet_ADD_PROPERTY);
		GridData btnData = new GridData(SWT.FILL, SWT.CENTER, false, false);
		btnData.widthHint = 50;
		addButton.setLayoutData(btnData);
		addButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fProperties.put(KEY_LABEL, VALUE_LABEL);
				propertiesViewer.refresh();
				for (Map.Entry entry: fProperties.entrySet()) {
					if (KEY_LABEL.equals(entry.getKey()) && VALUE_LABEL.equals(entry.getValue())) {
						propertiesViewer.editElement(entry, 0);
						break;
					}
				}
			}
		});
		final Button removeButton= new Button(buttonBar, SWT.FLAT);
		removeButton.setText(Messages.Configurationlet_REMOVE_PROPERTY);
		removeButton.setEnabled(false);
		btnData = new GridData(SWT.FILL, SWT.CENTER, false, false);
		btnData.widthHint = 50;
		removeButton.setLayoutData(btnData);
		removeButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				for (Map.Entry<String, String> entry: (List<Map.Entry<String, String>>)((IStructuredSelection)propertiesViewer.getSelection()).toList()) {
					fProperties.remove(entry.getKey());
				}
				propertiesViewer.refresh();
			}
		});
		propertiesViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				removeButton.setEnabled(!propertiesViewer.getSelection().isEmpty());
			}
		});
		propertiesViewer.setCellEditors(new CellEditor[]{new TextCellEditor(t), new TextCellEditor(t)});
		propertiesViewer.setColumnProperties(new String[] { KEY_LABEL, VALUE_LABEL});
		ICellModifier modifier= new ICellModifier() {

			@Override
			public boolean canModify(Object element, String property) {
				return true;
			}

			@Override
			public Object getValue(Object element, String property) {
				if (element instanceof Map.Entry) {
					if (property.equals(KEY_LABEL)) {
						return ((Map.Entry<String, String>)element).getKey();
					}
					if (property.equals(VALUE_LABEL)) {
						return ((Map.Entry<String, String>)element).getValue();
					}
				}
				return null;
			}
			
			@Override
			public void modify(Object element, String property, final Object value) {
				if (element instanceof Item) {
					element = ((Item) element).getData();
				}
				if (!(element instanceof Map.Entry) || !(value instanceof String)) {
					return;
				}
				Map.Entry<String, String> entry= (Map.Entry<String, String>)element;
				if (property.equals(KEY_LABEL)) {
					fProperties.remove(entry.getKey());
					fProperties.put((String)value, entry.getValue());
				} else if (property.equals(VALUE_LABEL)) {
					fProperties.put(entry.getKey(), (String)value);
				}
				propertiesViewer.refresh();
			}
		};
		propertiesViewer.setCellModifier(modifier);
		if (!"gtk".equals(SWT.getPlatform())) {  //$NON-NLS-1$
			Utils.updateColumnWidths(propertiesViewer.getTable(), 1, 1);
		}
	}
	
	private static class EditWorkflowStatesDialog extends ProcessAttachmentIconSelectionDialog {
		private String fStates;
		private final List<WFWorkflow> fWorkflows;
		private CheckboxTreeViewer fWorkflowViewer;
		protected EditWorkflowStatesDialog(Shell parentShell, String title, String iconUrl, IPrefixProvider prefixProvider, List<WFWorkflow> workflows, String states, ResourceManager resourceManager) {
			super(parentShell, title, iconUrl, prefixProvider, resourceManager);
			fWorkflows= workflows;
			fStates= states;
		}
		
		public String getStates() {
			return fStates;
		}
		
		@Override
		protected void createUpperDialogPart(Composite parent) {
			Label l= new Label(parent, SWT.NONE);
			l.setText(Messages.Configurationlet_SELECT_STATES);
			GridData data = new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1);
			l.setLayoutData(data);
			Tree t= new Tree(parent, SWT.CHECK | SWT.FULL_SELECTION | SWT.BORDER);
			data= new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1);
			data.verticalIndent= 5;
			t.setLayoutData(data);
			TreeColumn c= new TreeColumn(t, SWT.NONE);
			c.setWidth(300);
			fWorkflowViewer= new CheckboxTreeViewer(t);
			fWorkflowViewer.setLabelProvider(new CellLabelProvider() {
				@Override
				public void update(ViewerCell cell) {
					if (cell.getElement() instanceof WFWorkflow) {
						cell.setText(((WFWorkflow)cell.getElement()).getName());
					} else if (cell.getElement() instanceof WFState) {
						cell.setText(((WFState)cell.getElement()).getName());
					} else if (cell.getElement() instanceof String){
						cell.setText((String) cell.getElement());
					} else {
						cell.setText(null);
					}
				}
			});
			fWorkflowViewer.setContentProvider(new ITreeContentProvider() {
				@Override
				public Object[] getChildren(Object parentElement) {
					if (parentElement instanceof WFWorkflow) {
						return ((WFWorkflow)parentElement).getStates().toArray();
					}
					return new Object[0];
				}
				@Override
				public Object getParent(Object element) {
					if (element instanceof WFState) {
						return ((WFState)element).internalFlow();
					}
					return null;
				}
				@Override
				public boolean hasChildren(Object element) {
					return getChildren(element).length > 0;
				}
				@Override
				public Object[] getElements(Object inputElement) {
					return fWorkflows.toArray();
				}
				@Override
				public void dispose() {
				}
				@Override
				public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
				}
				
			});
			fWorkflowViewer.setAutoExpandLevel(2);
			fWorkflowViewer.addCheckStateListener(new ICheckStateListener() {
				@Override
				public void checkStateChanged(CheckStateChangedEvent event) {
					if (event.getElement() instanceof WFWorkflow && event.getChecked()) {
						fWorkflowViewer.setChecked(event.getElement(), false);
					} else {
						WFState state= (WFState) event.getElement();
						if (!event.getChecked()) {
							String states= ""; //$NON-NLS-1$
							for (String s: fStates.split(",")) { //$NON-NLS-1$
								String stateId= s.trim();
								if (!state.getId().equals(stateId)) {
									states= states + ", " + stateId; //$NON-NLS-1$
								}
							}
							fStates= states;
						} else {
							fStates= fStates + ", " + state.getId(); //$NON-NLS-1$
						}
						setChecked();
					}
				}
			});
			fWorkflowViewer.setInput(fWorkflows);
			setChecked();
			
		}
		
		private void setChecked() {
			if (fStates == null) {
				return;
			}
			String[] strings = fStates.split(","); //$NON-NLS-1$
			Collection<WFState> checked= new HashSet<WFState>();
			for (String s: strings) {
				String stateId= s.trim();
				for (WFWorkflow flow: fWorkflows) {
					for (WFState state: flow.getStates()) {
						if (stateId.equals(state.getId())) {
							checked.add(state);
						}
					}
				}
			}
			fWorkflowViewer.setCheckedElements(checked.toArray());
		}
		
		@Override
		protected void createIconDialogPart(Composite parent) {
		}
	}
	
}
