/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.configurationlet;

import org.eclipse.swt.widgets.Composite;

import com.ibm.team.process.common.IProcessItemHandle;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.common.model.AttributeTypes;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorPresentation.Attribute;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.enumeration.Messages;

/**
 * Subclass of the DefaultAttributeConfigurationlet that will check if the
 * enumeration is a persisted enumeration. Persisted enumerations have the
 * ability to add new literals to the enumeration from the work item editor.
 */
public class EnumerationAttributeConfigurationlet extends DefaultAttributeConfigurationlet {

	@Override
	protected void createPropertyControls(Composite parent) {
		super.createPropertyControls(parent);
		createAllowAnyValuePropertyControl(parent);
	}

	protected void createAllowAnyValuePropertyControl(Composite parent) {
		Attribute attribute= getAttribute();
		final IProcessItemHandle processItem= getProcessItem();
		if (processItem instanceof IProjectAreaHandle && attribute != null && attribute.getAttributeType() != null) {
			final String attributeType= attribute.getAttributeType();
			if (AttributeTypes.isEnumerationAttributeType(attributeType) || AttributeTypes.isEnumerationListAttributeType(attributeType)) {
				createBooleanControl(parent, Messages.EnumerationAttributeConfigurationlet_ALLOW_ADD_VALUE, Messages.EnumerationAttributeConfigurationlet_ALLOW_ADD_VALUE_DESCRIPTION, EditorPresentationManager.ALLOWANYVALUE);
			}
		}
	}
}
