/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.configurationlet;


import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TableItem;

import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.links.common.registry.IEndPointDescriptor;
import com.ibm.team.links.common.registry.ILinkType;
import com.ibm.team.workitem.common.internal.presentations.EditorPresentationManager;
import com.ibm.team.workitem.ide.ui.internal.WorkItemIDEUIPlugin;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.AspectEditorUtil;
import com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.AddEditPresentationDialog;



public class LinksConfigurationlet extends Configurationlet {
	
	private TableViewer fLinkTypesViewer;
	
	@Override
	public void createContent(Composite parent) {
		
		parent.setLayout(GridLayoutFactory.fillDefaults().numColumns(2).create());
		Label spacer= new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData data = new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1);
		data.verticalIndent= 10;
		spacer.setLayoutData(data);
		
		Map<String, String> valueSet= new HashMap<String, String>();		
		for (IEndPointDescriptor cur : AspectEditorUtil.getEndPoints()) {
			ILinkType linkType= cur.getLinkType();
			if (linkType != null) {
				XMLString typeId= XMLString.createFromXMLText(linkType.getLinkTypeId());
				XMLString typeName= XMLString.createFromXMLText(cur.getDisplayName());
				valueSet.put(typeId.getPlainText(), typeName.getPlainText());
			}
		}
		fLinkTypesViewer = createCheckedTable(parent, Messages.LinksConfigurationlet_LINK_TYPE_FILTER, Messages.LinksConfigurationlet_LINK_TYPE_FILTER_DESCRIPTION, EditorPresentationManager.LINK_TYPE_FILTER, valueSet);
		
		createLabelVisiblePropertyControl(parent);
		createVisibilityControls(parent);
	}
	
	@Override
	public void setupValidation(final AddEditPresentationDialog dialog) {
		fLinkTypesViewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				dialog.validate();
			}
		});
	}
	
	public IStatus isValid() {
		// need to check selection directly because getProperties() seems to be updated after the check
		TableItem[] tableItems= fLinkTypesViewer.getTable().getItems();
		if (tableItems != null) {
			for (TableItem cur : tableItems) {
				if (cur.getChecked()) {
					return Status.OK_STATUS;
				}
			}
		}
		return new Status(IStatus.ERROR, WorkItemIDEUIPlugin.PLUGIN_ID, Messages.LinksConfigurationlet_NO_LINK_TYPES_UNCONFIGURED);
	}
}
