/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     IBM Corporation - modifications made for Jazz are (c) Copyright IBM Corporation 2008
 * 
 * @copiedFrom parts of the code have been copied from from org.eclipse.jdt.internal.debug.ui.console.JavaExceptionHyperLink
 *  Reason: cannot reuse because of differing interfaces and internal package
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.editor.comments;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.jdt.core.IClassFile;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.Signature;
import org.eclipse.jdt.debug.core.IJavaExceptionBreakpoint;
import org.eclipse.jdt.debug.core.JDIDebugModel;
import org.eclipse.jdt.internal.debug.ui.actions.JavaBreakpointPropertiesAction;
import org.eclipse.jdt.internal.debug.ui.propertypages.JavaBreakpointPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.StructuredSelection;

import com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider;
import com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler;
import com.ibm.team.workitem.java.ide.ui.internal.WorkItemJavaIDEUIPlugin;
import com.ibm.team.workitem.rcp.ui.internal.WorkItemRCPUIPlugin;

import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;

/**
 * PUSHDOWN: CPLed by IBM - from
 * org.eclipse.jdt.internal.debug.ui.console.JavaExceptionHyperLink
 */
public class BreakpointPropertiesHyperlinkHandler extends HyperlinkHandler {

	private final static String SCHEME= "breakpointProperties"; //TODO Mixed-Case not allowed in Schemes! //$NON-NLS-1$

	public static URI createURI(String qualifiedTypeName) {
		try {
			return new URI(SCHEME + "://" + qualifiedTypeName); //$NON-NLS-1$
		} catch (URISyntaxException x) {
			throw new RuntimeException(x);
		}
	}

	public static URI createURI2(String qualifiedTypeName) throws URISyntaxException {
		return new URI(SCHEME + "://" + qualifiedTypeName); //$NON-NLS-1$
	}

	public boolean handles(URI uri) {
		return SCHEME.equalsIgnoreCase(uri.getScheme());
	}

	public void open(final URI uri, ContextProvider context) {
		WorkItemRCPUIPlugin.runInUI(Messages.BreakpointPropertiesHyperlinkHandler_OPENING_BREAKPOINT_PROPERTIES, new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				try {
					String exceptionName= uri.getHost();

					// check for an existing breakpoint
					IBreakpoint[] breakpoints= DebugPlugin.getDefault().getBreakpointManager().getBreakpoints(JDIDebugModel.getPluginIdentifier());
					for (int i= 0; i < breakpoints.length; i++) {
						IBreakpoint breakpoint= breakpoints[i];
						if (breakpoint instanceof IJavaExceptionBreakpoint) {
							IJavaExceptionBreakpoint exceptionBreakpoint= (IJavaExceptionBreakpoint) breakpoint;
							if (exceptionName.equals(exceptionBreakpoint.getTypeName())) {
								showProperties(exceptionBreakpoint);
								return;
							}
						}
					}
					// create a new exception breakpoint
					Object sourceElement= getSourceElement(exceptionName);
					if (sourceElement != null) {
						IResource res= null;
						IType type= null;
						if (sourceElement instanceof IJavaElement) {
							IJavaElement element= (IJavaElement) sourceElement;
							res= element.getResource();
							if (res == null) {
								res= element.getJavaProject().getResource();
							}
						}
						if (res == null) {
							res= ResourcesPlugin.getWorkspace().getRoot();
						}
						if (sourceElement instanceof ICompilationUnit) {
							type= ((ICompilationUnit) sourceElement).findPrimaryType();
						} else if (sourceElement instanceof IClassFile) {
							type= ((IClassFile) sourceElement).getType();
						}
						boolean checked= false;
						if (type != null) {
							checked= isChecked(type);
						}
						Map map= new HashMap();
						map.put(JavaBreakpointPage.ATTR_DELETE_ON_CANCEL, JavaBreakpointPage.ATTR_DELETE_ON_CANCEL);
						IJavaExceptionBreakpoint breakpoint= JDIDebugModel.createExceptionBreakpoint(res, exceptionName, true, true, checked, false, map);
						showProperties(breakpoint);
					}
				} catch (CoreException x) {
					throw new InvocationTargetException(x);
				}
			}
		}, new NullProgressMonitor());
	}

	/**
	 * Show the properties dialog for the given breakpoint.
	 * 
	 * @param exceptionBreakpoint
	 */
	private void showProperties(IJavaExceptionBreakpoint breakpoint) {
		JavaBreakpointPropertiesAction action= new JavaBreakpointPropertiesAction();
		action.selectionChanged(null, new StructuredSelection(breakpoint));
		action.run(null);
	}

	/**
	 * Returns the source element associated with the given type name, or
	 * <code>null</code> if none.
	 * 
	 * @param typeName type name to search for source element
	 * @return the source element associated with the given type name, or
	 * <code>null</code> if none
	 */
	private static IType getSourceElement(String typeName) {
		try {
			// search for the type in the workspace
			IWorkspaceRoot root= ResourcesPlugin.getWorkspace().getRoot();
			IJavaProject[] projects= JavaCore.create(root).getJavaProjects();
			for (int i= 0; i < projects.length; i++) {
				IType type= findType(projects[i], typeName);
				if (type != null) {
					return type;
				}
			}
			return null;
		} catch (JavaModelException e) {
			return null;
		}
	}

	/**
	 * Finds a type by its qualified type name (dot separated).
	 * 
	 * @param jproject The Java project to search in
	 * @param fullyQualifiedName The fully qualified name (type name with
	 * enclosing type names and package (all separated by dots))
	 * @return The type found, or <code>null<code> if no type found
	 * The method does not find inner types. Waiting for a Java Core solution
	 */
	private static IType findType(IJavaProject jproject, String fullyQualifiedName) throws JavaModelException {

		String pathStr= fullyQualifiedName.replace('.', '/') + ".java"; //$NON-NLS-1$
		IJavaElement jelement= jproject.findElement(new Path(pathStr));
		if (jelement == null) {
			// try to find it as inner type
			String qualifier= Signature.getQualifier(fullyQualifiedName);
			if (qualifier.length() > 0) {
				IType type= findType(jproject, qualifier); // recursive!
				if (type != null) {
					IType res= type.getType(Signature.getSimpleName(fullyQualifiedName));
					if (res.exists()) {
						return res;
					}
				}
			}
		} else if (jelement.getElementType() == IJavaElement.COMPILATION_UNIT) {
			String simpleName= Signature.getSimpleName(fullyQualifiedName);
			return ((ICompilationUnit) jelement).getType(simpleName);
		} else if (jelement.getElementType() == IJavaElement.CLASS_FILE) {
			return ((IClassFile) jelement).getType();
		}
		return null;
	}

	/**
	 * returns if the exception should be marked as 'checked' or not
	 * <p>
	 * NOTE: from
	 * org.eclipse.jdt.internal.debug.ui.breakpoints.AddExceptionAction, replace
	 * after moving to 3.2M5
	 * </p>
	 * 
	 * @param type the type of the exception
	 * @return true if the exception is to be 'checked' false otherwise
	 * @since 3.2
	 */
	public static boolean isChecked(IType type) {
		if (type != null) {
			try {
				ITypeHierarchy hierarchy= type.newSupertypeHierarchy(new NullProgressMonitor());
				IType curr= type;
				while (curr != null) {
					String name= curr.getFullyQualifiedName('.');
					if ("java.lang.RuntimeException".equals(name) || "java.lang.Error".equals(name)) { //$NON-NLS-1$ //$NON-NLS-2$
						return false;
					}
					curr= hierarchy.getSuperclass(curr);
				}
			} catch (JavaModelException x) {
				WorkItemJavaIDEUIPlugin.getDefault().log(Messages.BreakpointPropertiesHyperlinkHandler_EXCEPTION_DETERMINING_TYPE, x);
			}
		}
		return true;
	}
}
