/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     IBM Corporation - modifications made for Jazz are (c) Copyright IBM Corporation 2008
 * 
 * @copiedFrom parts of the code have been copied from from org.eclipse.jdt.internal.debug.ui.console.JavaStackTraceHyperlink
 *  Reason: cannot reuse because of differing interfaces and internal package
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.editor.comments;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchMatch;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.core.search.SearchRequestor;
import org.eclipse.jdt.internal.ui.javaeditor.EditorUtility;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;

import com.ibm.team.foundation.rcp.core.hyperlinks.ContextProvider;
import com.ibm.team.foundation.rcp.core.hyperlinks.HyperlinkHandler;
import com.ibm.team.workitem.java.ide.ui.internal.WorkItemJavaIDEUIPlugin;
import com.ibm.team.workitem.rcp.ui.internal.WorkItemRCPUIPlugin;

import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * PUSHDOWN: CPLed by IBM - from
 * org.eclipse.jdt.internal.debug.ui.console.JavaStackTraceHyperlink
 * 
 * @deprecated Use {@link JavaElementHyperlinkHandler} instead
 */
public class CompilationUnitHyperlinkHandler extends HyperlinkHandler {

	public final static String SCHEME= "javaElement"; //TODO Mixed-Case not allowed in Schemes! //$NON-NLS-1$

	private static CompilationUnitHyperlinkHandler fgCompilationUnitHyperlinkHandler;

	public CompilationUnitHyperlinkHandler() {
		fgCompilationUnitHyperlinkHandler= this;
	}

	public static CompilationUnitHyperlinkHandler getCompilationUnitHyperlinkHandler() {
		return fgCompilationUnitHyperlinkHandler;
	}

	/**
	 * @deprecated Use
	 * {@link JavaElementHyperlinkHandler#createURI(org.eclipse.jdt.core.IJavaElement)}
	 * instead
	 */
	public static URI createURI(IType javaElement) {
		return createURI(getQualifiedElementName(javaElement));
	}

	/**
	 * @deprecated Use
	 * {@link JavaElementHyperlinkHandler#createURI(String, String, String[])}
	 * instead
	 */
	public static URI createURI(String qualifiedType, String methodName) {
		return createURI(qualifiedType + "#" + methodName + "()V"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * @deprecated Use {@link JavaElementHyperlinkHandler} instead
	 */
	public static URI createURI(String qualifiedElement) {
		try {
			return new URI(SCHEME + "://" + qualifiedElement); //$NON-NLS-1$
		} catch (URISyntaxException x) {
			throw new RuntimeException(x);
		}
	}

	private static String getQualifiedElementName(IType javaElement) {
		String pkg= javaElement.getPackageFragment().getElementName();
		if (pkg.length() > 0)
			pkg+= "."; //$NON-NLS-1$
		return pkg + javaElement.getElementName();
	}

	public boolean handles(URI uri) {
		return SCHEME.equalsIgnoreCase(uri.getScheme());
	}

	public void open(final URI uri, ContextProvider context, IProgressMonitor monitor) {
		try {
			String typeName= uri.getHost();
			int lineNumber= uri.getPort();

			// #5852: NPE when opening linked test
			if (typeName == null && lineNumber == -1) {
				String string= uri.toString();
				int startHost= string.indexOf("://"); //$NON-NLS-1$
				if (startHost == -1)
					return;

				int startFragment= string.indexOf('#');
				if (startFragment == -1)
					return;

				typeName= string.substring(startHost + 3, startFragment);
			}

			// documents start at 0
			if (lineNumber > 0) {
				lineNumber--;
			}

			final String finalTypeName= typeName;
			final int finalLineNumber= lineNumber;
			final IType[] types= findTypes(typeName, new NullProgressMonitor());
			runInUI(Messages.CompilationUnitHyperlinkHandler_OPENING_JAVA_ELEMENT, new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
					if (types.length > 0) {
						try {
							Object sourceElement= types[0];
							IEditorInput editorInput= EditorUtility.getEditorInput(sourceElement);
							if (editorInput != null) {
								String editorId= getEditorId(editorInput, sourceElement);
								if (editorId != null) {
									IEditorPart editorPart= getActivePage().openEditor(editorInput, editorId);
									if (editorPart instanceof ITextEditor && finalLineNumber >= 0) {
										ITextEditor textEditor= (ITextEditor) editorPart;
										IDocumentProvider provider= textEditor.getDocumentProvider();
										try {
											provider.connect(editorInput);
											IDocument document= provider.getDocument(editorInput);
											IRegion line= document.getLineInformation(finalLineNumber);
											textEditor.selectAndReveal(line.getOffset(), line.getLength());
										} catch (BadLocationException e) {
											// ignore
										} finally {
											provider.disconnect(editorInput);
										}
									}
								}
							}
						} catch (CoreException x) {
							WorkItemJavaIDEUIPlugin.getDefault().log(x.getStatus());
							errorDialog(Messages.CompilationUnitHyperlinkHandler_EXCEPTION_WHILE_FOLLOWING_LINK, x);
						}
					} else
						// did not find source
						MessageDialog.openInformation(getActiveWorkbenchShell(), Messages.CompilationUnitHyperlinkHandler_INFORMATION, MessageFormat.format(Messages.CompilationUnitHyperlinkHandler_SOURCE_NOT_FOUND, new String[] { finalTypeName }));
				}
			}, new NullProgressMonitor());
		} catch (CoreException x) {
			WorkItemJavaIDEUIPlugin.getDefault().log(x.getStatus());
		}
	}

	private void runInUI(String name, IRunnableWithProgress runnable, IProgressMonitor monitor) {
		WorkItemRCPUIPlugin.runInUI(name, runnable, monitor);
	}

	private String getEditorId(IEditorInput input, Object inputObject) {
		IEditorRegistry registry= PlatformUI.getWorkbench().getEditorRegistry();
		IEditorDescriptor descriptor= registry.getDefaultEditor(input.getName());
		if (descriptor != null)
			return descriptor.getId();

		return null;
	}

	private IWorkbenchPage getActivePage() {
		return getActiveWorkbenchWindow().getActivePage();
	}

	private void errorDialog(String message, CoreException e) {
		ErrorDialog.openError(getActiveWorkbenchShell(), Messages.CompilationUnitHyperlinkHandler_ERROR, message, e.getStatus());
	}

	private Shell getActiveWorkbenchShell() {
		return getActiveWorkbenchWindow().getShell();
	}

	private IWorkbenchWindow getActiveWorkbenchWindow() {
		return PlatformUI.getWorkbench().getActiveWorkbenchWindow();
	}

	public static IType[] findTypes(String typeName, IProgressMonitor monitor) throws CoreException {

		final List results= new ArrayList();

		SearchRequestor collector= new SearchRequestor() {
			public void acceptSearchMatch(SearchMatch match) throws CoreException {
				Object element= match.getElement();
				if (element instanceof IType)
					results.add(element);
			}
		};

		SearchEngine engine= new SearchEngine();
		SearchPattern pattern= SearchPattern.createPattern(typeName, IJavaSearchConstants.TYPE, IJavaSearchConstants.DECLARATIONS, SearchPattern.R_EXACT_MATCH);
		engine.search(pattern, new SearchParticipant[] { SearchEngine.getDefaultSearchParticipant() }, SearchEngine.createWorkspaceScope(), collector, monitor);

		return (IType[]) results.toArray(new IType[results.size()]);
	}

	/**
	 * Returns the source element associated with the given type name, or
	 * <code>null</code> if none.
	 * 
	 * @param typeName type name to search for source element
	 * @return the source element associated with the given type name, or
	 * <code>null</code> if none
	 * @deprecated use {@link #findTypes(String, IProgressMonitor)} instead
	 */
	public static IType getSourceElement(String typeName) {
		try {
			IType[] types= findTypes(typeName, null);
			if (types.length > 0)
				return types[0];
		} catch (CoreException x) {
			WorkItemJavaIDEUIPlugin.getDefault().log(x.getStatus());
		}
		return null;
	}
}