/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.integration;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.jdt.junit.model.ITestCaseElement;
import org.eclipse.jdt.junit.model.ITestElement.FailureTrace;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.links.common.IReference;
import com.ibm.team.links.common.factory.IReferenceFactory;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.client.WorkItemWorkingCopy;
import com.ibm.team.workitem.common.model.IWorkItem;
import com.ibm.team.workitem.common.model.WorkItemEndPoints;
import com.ibm.team.workitem.ide.ui.internal.integration.CreateWorkItemAction;
import com.ibm.team.workitem.java.ide.ui.internal.WorkItemJavaIDEUIPlugin;
import com.ibm.team.workitem.java.ide.ui.internal.editor.comments.JavaElementHyperlinkHandler;

import java.net.URI;

public class CreateWorkItemFromTest extends CreateWorkItemAction {

	public CreateWorkItemFromTest() {}

	protected void initializeWorkItem(WorkItemWorkingCopy workingCopy, ISelection selection, IProgressMonitor monitor) throws TeamRepositoryException {
		super.initializeWorkItem(workingCopy, selection, monitor);

		ITestCaseElement element= getTestCaseElement(selection);
		if (element == null)
			return;

		IWorkItem workItem= workingCopy.getWorkItem();

		XMLString summary= XMLString.createFromPlainText(NLS.bind(Messages.CreateWorkItemFromTest_TEST_FAILED, element.getTestClassName(), element.getTestMethodName()));
		workItem.setHTMLSummary(summary);

		if (workItem.getHTMLDescription().getPlainText().length() == 0)
			workItem.setHTMLDescription(summary);

		addLink(workingCopy, element);
	}

	protected String extractContent(ISelection selection) {
		ITestCaseElement element= getTestCaseElement(selection);
		if (element == null)
			return null;
		FailureTrace failureTrace= element.getFailureTrace();
		if (failureTrace == null)
			return null;
		return failureTrace.getTrace();
	}

	protected void updateEnablement(IAction action, ISelection selection) {
		action.setEnabled(true);
	}

	private ITestCaseElement getTestCaseElement(ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			Object element= ((IStructuredSelection) selection).getFirstElement();
			if (element instanceof ITestCaseElement)
				return (ITestCaseElement) element;
		}
		return null;
	}

	private void addLink(final WorkItemWorkingCopy workingCopy, ITestCaseElement element) {
		final String method= element.getTestMethodName();
		final String type= element.getTestClassName();
		SafeRunnable.run(new ISafeRunnable() {
			public void run() throws Exception {
				URI uri= JavaElementHyperlinkHandler.createURI(type, method, null);
				String name= NLS.bind(Messages.CreateWorkItemFromTest_METHOD_NAME, type, method);
				IReference reference= IReferenceFactory.INSTANCE.createReferenceFromURI(uri, name);
				workingCopy.getReferences().add(WorkItemEndPoints.RELATED_ARTIFACT, reference);
			}

			public void handleException(Throwable exception) {
				WorkItemJavaIDEUIPlugin.getDefault().log(exception.getMessage(), exception);
			}
		});
	}
}
