/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.integration;

import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.model.IStackFrame;
import org.eclipse.jdt.debug.core.IJavaStackFrame;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.internal.debug.ui.JDIModelPresentation;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import com.ibm.team.workitem.ide.ui.internal.integration.CreateWorkItemAction;
import com.ibm.team.workitem.java.ide.ui.internal.WorkItemJavaIDEUIPlugin;

/**
 * Create a Work Item from a {@link IJavaThread}.
 */
public class CreateWorkItemFromThread extends CreateWorkItemAction {
	private static final String TAB= "\t"; //$NON-NLS-1$
	private static final String SEPARATOR= System.getProperty("line.separator"); //$NON-NLS-1$

	public CreateWorkItemFromThread() {}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.integration.CreateWorkItemAction#extractContent(org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	protected String extractContent(ISelection selection) {
		if (!selection.isEmpty() && selection instanceof IStructuredSelection) {
			IStructuredSelection structuredSelection= (IStructuredSelection) selection;
			Object element= structuredSelection.getFirstElement();
			if (element instanceof IJavaThread) {
				IJavaThread thread= (IJavaThread) element;

				JDIModelPresentation pres= new JDIModelPresentation();

				try {
					StringBuilder str= new StringBuilder();
					str.append(pres.getText(element)).append(SEPARATOR);
					IStackFrame[] stackFrames= thread.getStackFrames();
					for (IStackFrame f : stackFrames) {
						if (f instanceof IJavaStackFrame) {
							IJavaStackFrame frame= (IJavaStackFrame) f;
							str.append(TAB).append("at ").append(frame.getDeclaringTypeName()); //$NON-NLS-1$
							str.append(".").append(frame.getMethodName()); //$NON-NLS-1$
							str.append("(").append(frame.getSourceName()).append(":"); //$NON-NLS-1$//$NON-NLS-2$
							if (frame.getLineNumber() != -1)
								str.append(frame.getLineNumber());
							else
								str.append(Messages.CreateWorkItemFromThread_UNKNOWN_LINE);
							str.append(")"); //$NON-NLS-1$
							str.append(SEPARATOR);
						}
					}
					return str.toString();
				} catch (DebugException e) {
					WorkItemJavaIDEUIPlugin.getDefault().log(e.getMessage(), e);
				}
			}
		}

		return null;
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.integration.CreateWorkItemAction#updateEnablement(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	protected void updateEnablement(IAction action, ISelection selection) {
		int count= 0;
		if (!selection.isEmpty() && selection instanceof IStructuredSelection) {
			IStructuredSelection structuredSelection= (IStructuredSelection) selection;
			count= structuredSelection.size();
		}

		action.setEnabled(count == 1);
	}
}