/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.workitem.java.ide.ui.internal.integration;

import org.eclipse.jdt.junit.model.ITestCaseElement;
import org.eclipse.jdt.junit.model.ITestElement.FailureTrace;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;

import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.workitem.ide.ui.internal.integration.FindSimilarWorkItemsAction;

/**
 * Find similar work items from a JUnit test.
 */
@SuppressWarnings("restriction")
public class FindSimilarWorkItemsFromTest extends FindSimilarWorkItemsAction {

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.integration.FindSimilarWorkItemsAction#getQueryTitle()
	 */
	@Override
	protected String getQueryTitle() {
		return Messages.FindSimilarWorkItemsFromTest_SIMILAR_WI_TEST;
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.integration.FindSimilarWorkItemsAction#extractSummary(org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	protected String extractSummary(ISelection selection) {
		ITestCaseElement element= getTestCaseElement(selection);
		if (element != null)
			return NLS.bind(Messages.FindSimilarWorkItemsFromTest_TEST_FAILED, element.getTestClassName(), element.getTestMethodName());

		return null;
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.integration.FindSimilarWorkItemsAction#extractContent(org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	protected String extractContent(ISelection selection) {
		ITestCaseElement element= getTestCaseElement(selection);
		if (element == null)
			return null;

		FailureTrace failureTrace= element.getFailureTrace();
		if (failureTrace == null)
			return null;

		return failureTrace.getTrace();
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.integration.FindSimilarWorkItemsAction#updateEnablement(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	protected void updateEnablement(IAction action, ISelection selection) {
		action.setEnabled(true);
	}

	private ITestCaseElement getTestCaseElement(ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			Object element= ((IStructuredSelection) selection).getFirstElement();
			if (element instanceof ITestCaseElement)
				return (ITestCaseElement) element;
		}

		return null;
	}
}